#!/usr/local/sbin/php -q
<?php
/*
*! FILE NAME  : start_gps_compass.php
*! DESCRIPTION: Looks for USB GPS (currently Garmin GPS 18 USB) and compass
*!              (currently Ocean Server OS-5000), re-initializes Exif header
*!              and starts the devices if found
*!
*! Copyright (C) 2008 Elphel, Inc
*! -----------------------------------------------------------------------------**
*!
*!  This program is free software: you can redistribute it and/or modify
*!  it under the terms of the GNU General Public License as published by
*!  the Free Software Foundation, either version 3 of the License, or
*!  (at your option) any later version.
*!
*!  This program is distributed in the hope that it will be useful,
*!  but WITHOUT ANY WARRANTY; without even the implied warranty of
*!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*!  GNU General Public License for more details.
*!
*!  You should have received a copy of the GNU General Public License
*!  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*! -----------------------------------------------------------------------------**
*!  $Log: start_gps_compass.php,v $
*!  Revision 1.3  2009/02/20 21:49:52  elphel
*!  bugfix - was killing daemons after detection, not before (as should)
*!
*!  Revision 1.2  2009/02/19 22:38:37  elphel
*!  8.0.2.3 - added several USB serial adapters, nmea->exif support, detection of NMEA GPS at boot
*!
*!  Revision 1.1.1.1  2008/11/27 20:04:01  elphel
*!
*!
*!  Revision 1.2  2008/11/20 07:03:40  elphel
*!  exit value now encodes detected devices
*!
*!  Revision 1.1  2008/04/07 09:14:47  elphel
*!  Discover/startup GPS and compass
*!
*/
//look uptime, sleep if just started (no USB yet)
 $wait_time=20; // wait for $wait_time if both devices were not found, retry
    $compass=null;
    $GPS=null;
/// kill those GPS and compass if they are running already
    exec('killall -q garminusb2exif');
    exec('killall -q nmea2exif');
    exec('killall -q compass');
    find_gps_compass();
//    echo "compass: ";var_dump($compass);echo "\n";
//    echo "GPS:     ";var_dump($GPS);echo "\n";
   if ((!$compass || !$GPS) && ((exec('date +%s')+0) <30)) {
          echo "waiting $wait_time sec for USB devices to start up\n";
          sleep($wait_time);
          echo "Retrying detection\n";
          find_gps_compass();
    }
//    echo "compass: ";var_dump($compass);echo "\n";
//    echo "GPS:     ";var_dump($GPS);echo "\n";
    $noGPS=    ($compass || $GPS)?"" : "noGPS"; 
    $nocompass=($compass)?"" : "nocompass"; 
    $cmd="/usr/html/exif.php init $noGPS $nocompass";
    echo "Initializing Exif template: $cmd\n";
    exec($cmd);
    if ($GPS) {
      echo "Starting ".$GPS["name"]. " as ". $GPS["file"]. "\n";
      if (strpos($GPS['name'],'NMEA')!==false) {
        $cmd ="/usr/local/bin/nmea2exif ". $GPS["file"]." &";
      } else {
        $cmd ="/usr/local/bin/garminusb2exif ". $GPS["file"]." &";
      }
      echo "exec: $cmd \n";
      popen($cmd,"r");
    }
    if ($compass) {
      echo "Starting ".$compass["name"]. " as ". $compass["file"]. "\n";
      exec ("stty -F ".$compass["file"]." -echo speed 19200");
      $cmd="/usr/local/bin/compass ". $compass["file"]." &";
      echo "exec: $cmd \n";
      popen($cmd,"r");
    }
    exit (($compass?2:0) | ($GPS?1:0));

/// compass/GPS programs should be killed before detection
function find_gps_compass($timeout=5) {
    $ttyOptions=array (
      '-echo speed 4800', /// maibe other options, like '-echo speed 4800 -ixoff'
      '-echo speed 9600',
      '-echo speed 2400',
      '-echo speed 19200');
    global $GPS, $compass;
    $devices=array(
              "compass"=>array(array("name"=>"Ocean Server OS-5000","driver"=>"cp2101")),
              "GPS"    =>array(array("name"=>"Garmin GPS 18 USB","driver"=>"garmin_gps")));
    exec("ls /sys/bus/usb-serial/devices",$usb_ser_devs); // ("ttyUSB0","ttyUSB1")
    $devs = new SimpleXMLElement("<?xml version='1.0' standalone='yes'?><USB_serial_devices/>");
    $serialDevices=array();
    $compassPresent=false;
    $gpsPresent=false;
    $driver=array();
//echo "usb_ser_devs"; var_dump($usb_ser_devs);
    foreach ($usb_ser_devs as $dev) {
//echo "dev=$dev\n";
      $arr=split("/",exec("ls /sys/bus/usb-serial/devices/".$dev."/driver -l"));
      $driver[$dev]=$arr[count($arr)-1];
      $serialDevices[$dev]='';
      foreach ($devices["compass"] as $d) {
        if ($d["driver"]==$driver[$dev]) {
          $dd=$devs->addChild ('compass');
          $dd->addChild ('name',$d["name"]);
          $dd->addChild ('driver',$d["driver"]);
          $dd->addChild ('file',"/dev/".$dev);
          $serialDevices[$dev]='compass';
          $compassPresent=true;
        }
      }
      if ($serialDevices[$dev]) continue;
      foreach ($devices["GPS"] as $d) {
        if ($d["driver"]==$driver[$dev]) {
          $dd=$devs->addChild ('GPS');
          $dd->addChild ('name',$d["name"]);
          $dd->addChild ('driver',$d["driver"]);
          $dd->addChild ('file',"/dev/".$dev);
          $serialDevices[$dev]='gps';
          $gpsPresent=true;
        }
      }
    }
/// Do we need to look for a NMEA GPS?
    if (!$gpsPresent) foreach ($serialDevices as $dev=>$type) if (!$type && !$gpsPresent) {
      printf ("could be $dev\n");
      foreach ($ttyOptions as $ttyOpt) {
        $cmd='stty -F /dev/'.$dev.' '.$ttyOpt;
        echo "Trying:  $cmd\n";
        exec ($cmd);
        unset ($fullOutput);
        exec ('timeout '.$timeout.' cat /dev/'.$dev, &$fullOutput);
//var_dump($fullOutput);
        foreach ($fullOutput as $line) if (strpos ($line, '$GP')===0) {
          $gpsPresent=true;
          break;
        }
        if ($gpsPresent) {
          $dd=$devs->addChild ('GPS');
          $dd->addChild ('name','NMEA 0183 GPS receiver');
          $dd->addChild ('driver',$driver[$dev]);
          $dd->addChild ('file',"/dev/".$dev);
          $serialDevices[$dev]='gps';
          echo "\nfound NMEA GPS unit\n";
          break;
        }
//      var_dump($fullOutput);
      }
    }
/// list unused USB serial adapters
    foreach ($serialDevices as $dev=>$type) if (!$type) {
          $dd=$devs->addChild ('unused');
          $dd->addChild ('name','USB serial converter');
          $dd->addChild ('driver',$driver[$dev]);
          $dd->addChild ('file',"/dev/".$dev);
    }

    $state_file=fopen("/var/html/gps_compass.xml","w");
    fwrite($state_file,$devs->asXML());
    fclose($state_file);

// start here from XML
    foreach ($devs->compass as $a) {
      $compass=array("file"=>(string) $a->file, "name"=> (string) $a->name);
      break; // use first one
    }
    foreach ($devs->GPS as $a) {
      $GPS=array("file"=>(string) $a->file, "name"=> (string) $a->name);
      break; // use first one
    }
}
  
//ls /sys/bus/usb-serial/devices/ttyUSB0/driver -l
//         $xml->addChild ('error','No sync capable board detected, use "role=self" for the onboard timer');
//    $sxml=$xml->asXML();
?>
