/*
** -----------------------------------------------------------------------------**
** tlv5630.v
**
** Interface for the DAC TLV5630
**
** Copyright (C) 2003-2006 Elphel, Inc.
**
** -----------------------------------------------------------------------------**
**  This file is part of x324
**  x324 is free software - hardware description language (HDL) code.
** 
**  This program is free software: you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation, either version 3 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program.  If not, see <http://www.gnu.org/licenses/>.
** -----------------------------------------------------------------------------**
**
**  $Log: tlv5630.v,v $
**  Revision 1.1.1.1  2008/11/27 20:04:02  elphel
**
**
**  Revision 1.1.1.1  2007/08/17 12:07:19  elphel
**  This is a fresh tree based on elphel353-2.10
**
**  Revision 1.6  2007/08/17 12:07:19  spectr_rain
**  switch to GPL3 license
**
**  Revision 1.5  2007/07/20 09:46:13  spectr_rain
**  *** empty log message ***
**
**  Revision 1.4  2007/05/25 02:09:09  elphel
**  347 update for mechanical shutter
**
**  Revision 1.1  2006/09/24 16:44:36  elphel
**  Split file into separate module files, added real time clock module, project files
**
*/


// no ready output, just count on data coming not too fast
module tlv5630 (clk, //25MHz, output SCLK generated externally to match ThD=5ns
                we,  // WE from serial interface
                a,   // [3:0] - register select , LSB also LD on/off select
                d,   //[11:0] - data 
                cswr,// writing to register
                csld,// writing to LD on/off
                sd,  // serial data to DAC
                fs,  // frame sync
                ld, // LD enable (active low)
                clken); // enable DAC clk
 input       clk,
             we,
             cswr,
             csld;
 input [3:0] a;
 input [11:0] d;
 output      sd,
             fs,
             ld,
             clken;
 reg sd,fs;
 wire fs0, ld0; // active high
 reg [15:0] ad;
 wire strt= we && cswr;
 wire clken,clk_off;
  assign ld= !ld0;
 //assign fs= !fs0;
 //assign sd= ad[15];

   FD  i_fs0 (.C(clk), .D (strt), .Q(fs0));
   FDE i_ld0 (.C(clk), .CE(we && csld), .D(a[0]), .Q(ld0));
   SRL16 i_clk_off (.Q(clk_off),       .A0(1'b1), .A1(1'b1), .A2(1'b1), .A3(1'b1), .CLK(clk), .D(!fs));  //
   FD  i_clken (.C(clk), .D (strt || (clken && !clk_off)), .Q(clken));
  always @ (posedge clk) begin
   if (strt) ad[15:0] <= {a[3:0],d[11:0]};
   else      ad[15:0] <= {ad[14:0],1'b0};

   sd <= ad[15];
   fs <=!fs0;

  end
 endmodule

 
