/*!***************************************************************************
*! FILE NAME  : x359.tf
*! DESCRIPTION: testbench
*! Copyright (C) 2008 Elphel, Inc.
*! -----------------------------------------------------------------------------**
*!  This program is free software: you can redistribute it and/or modify
*!  it under the terms of the GNU General Public License as published by
*!  the Free Software Foundation, either version 3 of the License, or
*!  (at your option) any later version.
*!
*!  This program is distributed in the hope that it will be useful,
*!  but WITHOUT ANY WARRANTY; without even the implied warranty of
*!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*!  GNU General Public License for more details.
*!
*!  You should have received a copy of the GNU General Public License
*!  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*! -----------------------------------------------------------------------------**
*!
*!  $Log: x359.tf,v $
*!  Revision 1.1.1.1  2008/11/27 20:04:02  elphel
*!
*!
*!  Revision 1.2  2008/04/24 17:45:55  dzhimiev
*!  did't close the header comment
*!
*!  Revision 1.1  2008/04/23 01:55:49  dzhimiev
*!  1. added x359 files to src lists
*!  2. x359 read/write DDR
*!  3. x359 3 channels mux directly to out
*!  4. x359 one channel through DDR and another directly frames switching at out
*!
*/

`timescale 1ns / 1ps

module x359_tf;
   parameter DCLK_PER = 50;	//20MHz
   parameter I2C_PER = 1000;

	// Inputs
	reg SDNCLK_FB;
	reg SDCLK_FB;
	reg DCLK;
	reg MRST;
	reg ARO;
	reg ARST;
	wire PX_BPF1;
	wire PX_HACT1;
	wire PX_VACT1;
	wire PX_BPF2;
	wire PX_HACT2;
	wire PX_VACT2;
	wire PX_BPF3;
	wire PX_HACT3;
	wire PX_VACT3;

	// Outputs
	wire RUN;
	wire SDCLKE;
	wire SDCLK;
	wire SDNCLK;
	wire SDLDM;
	wire SDUDM;
	wire SDWE;
	wire SDCAS;
	wire SDRAS;
	wire [14:0] SDA;
	wire [9:0] PXD;
	wire BPF;
	wire HACT;
	wire VACT;
	wire PX_DCLK1;
	wire PX_MRST1;
	wire PX_ARO1;
	wire PX_ARST1;
	wire PX_DCLK2;
	wire PX_MRST2;
	wire PX_ARO2;
	wire PX_ARST2;
	wire PX_DCLK3;
	wire PX_MRST3;
	wire PX_ARO3;
	wire PX_ARST3;

	// Bidirs
	wire [15:0] SDD;
	wire UDQS;
	wire LDQS;
	wire SCL0;
	wire SDA0;
	wire CNVSYNC;
	wire CNVCLK;
	wire [11:0] PXD1;
	wire PX_SCL1;
	wire PX_SDA1;
	wire SENSPGM1;
	wire [11:0] PXD2;
	wire PX_SCL2;
	wire PX_SDA2;
	wire SENSPGM2;
	wire [11:0] PXD3;
	wire PX_SCL3;
	wire PX_SDA3;
	wire SENSPGM3;
	wire ALWAYS0;
   
// test registers/wires
   reg I2C_CLK;
   reg HOST_SDA;   
   reg HOST_SCL;   
   reg HOST_SDA_EN;   
   reg HOST_SCL_EN;   
   
   assign SDA0= HOST_SDA_EN ? HOST_SDA : 'bz;  
   assign SCL0= HOST_SCL_EN ? HOST_SCL : 'bz;  

	// param
	//defparam i_x359.i_dcm_359_2.i_dcm2.PHASE_SHIFT=7;
	// Instantiate the Unit Under Test (UUT)
	x359 i_x359 (
		.RUN(RUN), 
		.SDCLKE(SDCLKE), 
		.SDNCLK_FB(SDNCLK_FB), 
		.SDCLK(SDCLK), 
		.SDNCLK(SDNCLK), 
		.SDCLK_FB(SDCLK_FB), 
		.SDLDM(SDLDM), 
		.SDUDM(SDUDM), 
		.SDWE(SDWE), 
		.SDCAS(SDCAS), 
		.SDRAS(SDRAS), 
		.SDA(SDA), 
		.SDD(SDD), 
		.UDQS(UDQS), 
		.LDQS(LDQS), 
		.PXD(PXD), 
		.DCLK(DCLK), 
		.BPF(BPF), 
		.HACT(HACT), 
		.VACT(VACT), 
		.MRST(MRST), 
		.ARO(ARO), 
		.ARST(ARST), 
		.SCL0(SCL0), 
		.SDA0(SDA0), 
		.CNVSYNC(CNVSYNC), 
		.CNVCLK(CNVCLK), 
		.PXD1(PXD1), 
		.PX_DCLK1(PX_DCLK1), 
		.PX_BPF1(PX_BPF1), 
		.PX_HACT1(PX_HACT1), 
		.PX_VACT1(PX_VACT1), 
		.PX_MRST1(PX_MRST1), 
		.PX_ARO1(PX_ARO1), 
		.PX_ARST1(PX_ARST1), 
		.PX_SCL1(PX_SCL1), 
		.PX_SDA1(PX_SDA1), 
		.SENSPGM1(SENSPGM1), 
		.PXD2(PXD2), 
		.PX_DCLK2(PX_DCLK2), 
		.PX_BPF2(PX_BPF2), 
		.PX_HACT2(PX_HACT2), 
		.PX_VACT2(PX_VACT2), 
		.PX_MRST2(PX_MRST2), 
		.PX_ARO2(PX_ARO2), 
		.PX_ARST2(PX_ARST2), 
		.PX_SCL2(PX_SCL2), 
		.PX_SDA2(PX_SDA2), 
		.SENSPGM2(SENSPGM2), 
		.PXD3(PXD3), 
		.PX_DCLK3(PX_DCLK3), 
		.PX_BPF3(PX_BPF3), 
		.PX_HACT3(PX_HACT3), 
		.PX_VACT3(PX_VACT3), 
		.PX_MRST3(PX_MRST3), 
		.PX_ARO3(PX_ARO3), 
		.PX_ARST3(PX_ARST3), 
		.PX_SCL3(PX_SCL3), 
		.PX_SDA3(PX_SDA3), 
		.SENSPGM3(SENSPGM3), 
		.ALWAYS0(ALWAYS0)
	);

// Instance of Micron MT48LC8M16LFFF8
// cheating - no such actual signal :-(
reg SDCKE;
initial begin
  SDCKE=0;
  #1000;
  SDCKE=1;
end
wire SDCLK_D;
wire SDNCLK_D;
assign #(2) SDCLK_D=SDCLK;
assign #(2) SDNCLK_D=SDNCLK;

//`define sg5B;

	ddr i_mt46v16m16fg (.Dq(SDD[15:0]),
                    .Dqs({UDQS,LDQS}),
                    .Addr(SDA[12:0]),
                    .Ba(SDA[14:13]),
                    .Clk(SDCLK_D),
                    .Clk_n(SDNCLK_D),
//                  .Cke(1'b1),
//                  .Cke(SDCKE),
                    .Cke(SDCLKE),
                    .Cs_n(1'b0),
                    .Ras_n(SDRAS),
                    .Cas_n(SDCAS),
                    .We_n(SDWE),
                    .Dm({SDUDM,SDLDM})
										);

	initial begin
		// Initialize Inputs
		SDNCLK_FB = 0;
		SDCLK_FB = 0;
		DCLK = 0;
		MRST = 0;
		ARO = 0;
		ARST = 0;
		//PX_BPF1 = 0;
		//PX_HACT1 = 0;
		//PX_VACT1 = 0;
		//PX_BPF2 = 0;
		//PX_HACT2 = 0;
		//PX_VACT2 = 0;
		//PX_BPF3 = 0;
		//PX_HACT3 = 0;
		//PX_VACT3 = 0;

		// Wait 100 ns for global reset to finish
		#100;

		MRST = 1;
		ARO = 1;
		ARST = 1;        
		// Add stimulus here

	end
      
      
   always #(DCLK_PER/2)	DCLK =	   ~DCLK;
   always #(I2C_PER/2)	I2C_CLK =	~I2C_CLK;

//   parameter DCLK_PER = 50	//20MHz
//   parameter I2C_PER = 1000;
    	initial begin
    $dumpfile("x359_sim.lxt");
    $dumpvars(0,x359_tf);
      DCLK = 0;
      I2C_CLK = 0;
// test registers/wires
      I2C_CLK =0;
      HOST_SDA=1;   
      HOST_SCL=1;   
      HOST_SDA_EN=0;
      HOST_SCL_EN=0;   
      
//#1000;
// global set/reset for simulation
		glbl.GSR_int = 1'b1;
		glbl.GTS_int = 1'b1;
		#2000;
		glbl.GSR_int = 1'b0;
		#20;
		glbl.GTS_int = 1'b0;
		#50;

#10000;

	// frequency rotate 90 for ddr correct work - really?!
	i2c_start;
	i2c_send_2_bytes('h1001); i2c_send_4_bytes('h01020304);
	i2c_stop;

#10000;

	i2c_start;
	i2c_send_2_bytes('hbaa0); i2c_send_4_bytes('h00410000);
	i2c_stop;
	
#10000;

	// Read control sequence register - should receive 0x11223344 or 0x44332211
  i2c_start;
  i2c_sendbyte ('h10); i2c_sendbyte ('h68);
	i2c_stop; 
	i2c_restart;      
  i2c_sendbyte ('h11);	
	i2c_readbyte (1);
	i2c_readbyte (1);
	i2c_readbyte (1);
	i2c_readbyte (0);
  i2c_stop;		
		
#10000;
	// frequency rotate 90 for ddr correct work
	i2c_start;
	i2c_sendbyte ('h10); i2c_sendbyte ('h08); i2c_sendbyte ('h00); i2c_sendbyte ('h00);	i2c_sendbyte ('h00); i2c_sendbyte ('h04);	
	i2c_stop;
	
#10000;

	i2c_start;
	i2c_send_2_bytes('h1035);i2c_send_4_bytes('h00000001);
	i2c_stop;	
	
#20000;

	i2c_start;
	i2c_send_2_bytes('h1036);i2c_send_4_bytes('b00000000_00000000_00000000_00000111);
	i2c_stop;

#10000;

	// Read control sequence register - should receive 0x11223344 or 0x44332211
  i2c_start;
  i2c_sendbyte ('h10); i2c_sendbyte ('h60);
	i2c_stop; 
	i2c_restart;      
  i2c_sendbyte ('h11);	
	i2c_readbyte (1);
	i2c_readbyte (1);
	i2c_readbyte (1);
	i2c_readbyte (0);
  i2c_stop;

#40000;
  // DDR initialization sequence
	i2c_start;
	i2c_send_2_bytes('h3823);
	i2c_send_4_bytes('h00017fff);
	i2c_send_4_bytes('h00002002);
	i2c_send_4_bytes('h00000163);
	i2c_send_4_bytes('h00008000);
	i2c_send_4_bytes('h00008000);
	i2c_send_4_bytes('h00017fff);
  i2c_stop;		
	
#10000;	

	i2c_start;
	i2c_send_2_bytes('h3836);i2c_send_4_bytes('b00000000_00000000_00000000_00000010);
	i2c_stop;

#10000;


	i2c_start; 
	i2c_send_2_bytes('h1027);
	i2c_send_4_bytes('h00000000); 
	i2c_send_4_bytes('h00000001); 
	i2c_stop;
	#10000;
//	$finish;
// here, i'll try to write some data to DDR	
////////////////////////////////////////////////////////////////
	i2c_start; i2c_send_2_bytes('h102c); i2c_send_4_bytes('h00004001); i2c_stop;	
	i2c_start; i2c_send_2_bytes('h102d); i2c_send_4_bytes('h000001f0); i2c_stop;	
	i2c_start; i2c_send_2_bytes('h102e); i2c_send_4_bytes('h00000003); i2c_stop;	
	i2c_start; i2c_send_2_bytes('h1027); i2c_send_4_bytes('h00000021); i2c_stop;	
	#10000;	
////////////////////////////////////////////////////////////////
	//i2c_start; i2c_send_2_bytes('h1034); i2c_send_4_bytes('h11223344); i2c_stop;	
	//#10000;
////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////
//	i2c_start; i2c_send_2_bytes('h1031); i2c_send_4_bytes('h01020304); i2c_stop;
//	i2c_start; i2c_send_2_bytes('h1031); i2c_send_4_bytes('h05060708); i2c_stop;
//	i2c_start; i2c_send_2_bytes('h1031); i2c_send_4_bytes('h090a0b0c); i2c_stop;
//	i2c_start; i2c_send_2_bytes('h1031); i2c_send_4_bytes('h0d0e0f10); i2c_stop;
//	
//	i2c_start; i2c_send_2_bytes('h1031); i2c_send_4_bytes('h11121314); i2c_stop;
//	i2c_start; i2c_send_2_bytes('h1031); i2c_send_4_bytes('h15161718); i2c_stop;
//	i2c_start; i2c_send_2_bytes('h1031); i2c_send_4_bytes('h191a1b1c); i2c_stop;
//	i2c_start; i2c_send_2_bytes('h1031); i2c_send_4_bytes('h1d1e1f20); i2c_stop;
//	
//	i2c_start; i2c_send_2_bytes('h1031); i2c_send_4_bytes('h21222324); i2c_stop;
//	i2c_start; i2c_send_2_bytes('h1031); i2c_send_4_bytes('h25262728); i2c_stop;
//	i2c_start; i2c_send_2_bytes('h1031); i2c_send_4_bytes('h292a2b2c); i2c_stop;
//	i2c_start; i2c_send_2_bytes('h1031); i2c_send_4_bytes('h2d2e2f30); i2c_stop;
//	
//	i2c_start; i2c_send_2_bytes('h1031); i2c_send_4_bytes('h31323334); i2c_stop;
//	i2c_start; i2c_send_2_bytes('h1031); i2c_send_4_bytes('h35363738); i2c_stop;
//	i2c_start; i2c_send_2_bytes('h1031); i2c_send_4_bytes('h393a3b3c); i2c_stop;
//	i2c_start; i2c_send_2_bytes('h1031); i2c_send_4_bytes('h3d3e3f40); i2c_stop;
//	#10000;
////////////////////////////////////////////////////////////////
//	i2c_start; i2c_send_2_bytes('h1034); i2c_send_4_bytes('h11223344); i2c_stop;	
//	i2c_start; i2c_send_2_bytes('h1034); i2c_send_4_bytes('h11223344); i2c_stop;	
//	i2c_start; i2c_send_2_bytes('h1034); i2c_send_4_bytes('h11223344); i2c_stop;	
//	i2c_start; i2c_send_2_bytes('h1034); i2c_send_4_bytes('h11223344); i2c_stop;	
//	i2c_start; i2c_send_2_bytes('h1034); i2c_send_4_bytes('h11223344); i2c_stop;
	#1000;	

//////////////////////////////////////////////////////////////////////////////
	i2c_start; i2c_send_2_bytes('h102c); i2c_send_4_bytes('h00004000); i2c_stop;	// 4 writes
	i2c_start; i2c_send_2_bytes('h102d); i2c_send_4_bytes('h000001f0); i2c_stop;	
	i2c_start; i2c_send_2_bytes('h102e); i2c_send_4_bytes('h0000001f); i2c_stop;	
	//i2c_start; i2c_send_2_bytes('h1027); i2c_send_4_bytes('h00000021); i2c_stop;	
//	#10000;	

//	i2c_start; i2c_send_2_bytes('h1031); i2c_send_4_bytes('h61626364); i2c_stop;
//	i2c_start; i2c_send_2_bytes('h1031); i2c_send_4_bytes('h65666768); i2c_stop;
//	i2c_start; i2c_send_2_bytes('h1031); i2c_send_4_bytes('h696a6b6c); i2c_stop;
//	i2c_start; i2c_send_2_bytes('h1031); i2c_send_4_bytes('h6d6e6f70); i2c_stop;
//	
//	i2c_start; i2c_send_2_bytes('h1034); i2c_send_4_bytes('h11223344); i2c_stop;	// 4 writes
//	i2c_start; i2c_send_2_bytes('h1034); i2c_send_4_bytes('h11223344); i2c_stop;	
//	i2c_start; i2c_send_2_bytes('h1034); i2c_send_4_bytes('h11223344); i2c_stop;	
//	i2c_start; i2c_send_2_bytes('h1034); i2c_send_4_bytes('h11223344); i2c_stop;	
//	#1000;		
	
//////////////////////////////////////////////////////////////////////////////
	//i2c_start; i2c_send_2_bytes('h1027); i2c_send_4_bytes('h00000021); i2c_stop;	
	#100000;
	//i2c_start; i2c_send_2_bytes('h1027); i2c_send_4_bytes('h00000001); i2c_stop;	
////////////////////////////////////////////////////////////////
//	i2c_start; i2c_send_2_bytes('h102e); i2c_send_4_bytes('h00000003); i2c_stop;
//	i2c_start; i2c_send_2_bytes('h102c); i2c_send_4_bytes('h00000000); i2c_stop;	
//
//	i2c_start; i2c_send_2_bytes('h102e); i2c_send_4_bytes('h00000007); i2c_stop;
//	i2c_start; i2c_send_2_bytes('h102c); i2c_send_4_bytes('h00000003); i2c_stop;
//	
//	#200000;

//	i2c_start; i2c_send_2_bytes('h102c); i2c_send_4_bytes('h00004000); i2c_stop;

//	i2c_start; i2c_send_2_bytes('h1034); i2c_send_4_bytes('h11223344); i2c_stop;	
//	#1000;	
//	i2c_start; i2c_send_2_bytes('h1034); i2c_send_4_bytes('h11223344); i2c_stop;	
//	#1000;	
//	i2c_start; i2c_send_2_bytes('h1034); i2c_send_4_bytes('h11223344); i2c_stop;	
//	#1000;

		
//	i2c_start; i2c_send_2_bytes('h1034); i2c_send_4_bytes('h11223344); i2c_stop;	
//	#10000;
//////////////////////////////////////////////////////////////////
	//i2c_start; i2c_send_2_bytes('h1027); i2c_send_4_bytes('h00000021); i2c_stop;	

//   i2c_start;
//   i2c_sendbyte ('h10);
//   i2c_sendbyte ('h33);
//	 i2c_restart;      
//   i2c_sendbyte ('h11);
//	 
//	 repeat(14) begin
//		i2c_readbyte (1);
//		i2c_readbyte (1);
//		i2c_readbyte (1);
//		i2c_readbyte (1);
//	 end
// 	 
//   i2c_readbyte (1);
//   i2c_readbyte (1);
//   i2c_readbyte (1);
//   i2c_readbyte (0);	 
//	 
//   i2c_stop;
//
//#40000;

//$finish;
//i2c_start; i2c_send_2_bytes('h1027); i2c_send_4_bytes('h00000001); i2c_stop;

//////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////
// here start write from sens channel 0

// initialize

	i2c_start; i2c_send_2_bytes('h1027); i2c_send_4_bytes('h00000003); i2c_stop; // +auto refreshes

	i2c_start; i2c_send_2_bytes('h1021); i2c_send_4_bytes('h000007f0); i2c_stop;	
	i2c_start; i2c_send_2_bytes('h1022); i2c_send_4_bytes('h0000001f); i2c_stop;	
	i2c_start; i2c_send_2_bytes('h1020); i2c_send_4_bytes('h00004000); i2c_stop;	

	i2c_start; i2c_send_2_bytes('h1027); i2c_send_4_bytes('h00000007); i2c_stop;	

	#20000;

	i2c_start; i2c_send_2_bytes('h1029); i2c_send_4_bytes('h000007f0); i2c_stop;	
	i2c_start; i2c_send_2_bytes('h102a); i2c_send_4_bytes('h0000001f); i2c_stop;	
	i2c_start; i2c_send_2_bytes('h1028); i2c_send_4_bytes('h00000000); i2c_stop;	
	
	#10000;
	//i2c_start; i2c_send_2_bytes('h1027); i2c_send_4_bytes('h00000017); i2c_stop;
	i2c_start; i2c_send_2_bytes('h1027); i2c_send_4_bytes('h00000037); i2c_stop;

#1000000;

	i2c_start; i2c_send_2_bytes('h102c); i2c_send_4_bytes('h00000000); i2c_stop;

#100000;

	i2c_start; i2c_send_2_bytes('h102c); i2c_send_4_bytes('h00000004); i2c_stop;

// now read what was written from channel 0
   i2c_start;
   i2c_sendbyte ('h10);
   i2c_sendbyte ('h33);
	 i2c_restart;      
   i2c_sendbyte ('h11);
	 
	 repeat(14) begin
		i2c_readbyte (1);
		i2c_readbyte (1);
		i2c_readbyte (1);
		i2c_readbyte (1);
	 end
 	 
   i2c_readbyte (1);
   i2c_readbyte (1);
   i2c_readbyte (1);
   i2c_readbyte (0);	 
	 
   i2c_stop;


#5000000000;

//      $finish;
   end  
	 
	 

  sensor12bits #(.ncols(1620),.nrows(4),.t_afterHACT(100),.nVLO(100)) 
i_sensor12bits1(.MCLK(PX_DCLK1),	// Master clock
								.MRST(PX_MRST1),	// Master Reset - active low
								.ARO (PX_ARO1),	  // Array read Out.
								.ARST(PX_ARST1),	// Array Reset. Active low
								.OE  (1'b0),      // output enable active low
								.SCL (PX_SCL1),   // I2C clock
								.SDA (PX_SDA1),	  // I2C data
								.OFST(1'b1),      // I2C address ofset by 2: for simulation 0 - still mode, 1 - video mode.
								.D   (PXD1[11:0]),// [9:0] data output
								.DCLK(),          // Data output clock
								.BPF (PX_BPF1),   // Black Pixel Flag
								.HACT(PX_HACT1),  // Horizontal Active
								.VACT(PX_VACT1)   // Vertical Active		
                );

  sensor12bits #(.ncols(1620),.nrows(4),.t_afterHACT(100),.nVLO(100)) 
i_sensor12bits2(.MCLK(PX_DCLK2),	// Master clock
								.MRST(PX_MRST2),	// Master Reset - active low
								.ARO (PX_ARO2),	  // Array read Out.
								.ARST(PX_ARST2),	// Array Reset. Active low
								.OE  (1'b0),      // output enable active low
								.SCL (PX_SCL2),   // I2C clock
								.SDA (PX_SDA2),	  // I2C data
								.OFST(1'b1),      // I2C address ofset by 2: for simulation 0 - still mode, 1 - video mode.
								.D   (PXD2[11:0]),// [9:0] data output
								.DCLK(),          // Data output clock
								.BPF (PX_BPF2),   // Black Pixel Flag
								.HACT(PX_HACT2),  // Horizontal Active
								.VACT(PX_VACT2)   // Vertical Active		
                );
 
  sensor12bits #(.ncols(1620),.nrows(4),.t_afterHACT(100),.nVLO(100)) 
i_sensor12bits3(.MCLK(PX_DCLK3),	// Master clock
								.MRST(PX_MRST3),	// Master Reset - active low
								.ARO (PX_ARO3),	  // Array read Out.
								.ARST(PX_ARST3),	// Array Reset. Active low
								.OE  (1'b0),      // output enable active low
								.SCL (PX_SCL3),   // I2C clock
								.SDA (PX_SDA3),	  // I2C data
								.OFST(1'b1),      // I2C address ofset by 2: for simulation 0 - still mode, 1 - video mode.
								.D   (PXD3[11:0]),// [9:0] data output
								.DCLK(),          // Data output clock
								.BPF (PX_BPF3),   // Black Pixel Flag
								.HACT(PX_HACT3),  // Horizontal Active
								.VACT(PX_VACT3)   // Vertical Active		
                );
								
parameter I2CDLY = 10;

   task i2c_start;  // SCL is supposed to be 1, SDA - released
     begin
       $display ("i2c START at %t", $time);
       HOST_SDA=1;
       HOST_SCL=1;
       HOST_SDA_EN=0;
       HOST_SCL_EN=1;
       wait (~I2C_CLK);wait (I2C_CLK);
       HOST_SDA_EN=1;
       HOST_SDA=0;
       wait (~I2C_CLK);wait (I2C_CLK);
       HOST_SCL=0;
     end
   endtask   

   task i2c_stop;  // SCL is supposed to be 0, SDA - any
     begin
       $display ("i2c STOP at %t", $time);
       HOST_SCL=0;
       HOST_SCL_EN=1;
       wait (~I2C_CLK);wait (I2C_CLK); // wait for bus turnover (it was likely reading ACKN)
       HOST_SDA=0;
       HOST_SDA_EN=1;
       wait (~I2C_CLK);wait (I2C_CLK);
       HOST_SCL=1;
       wait (~I2C_CLK);wait (I2C_CLK);
       HOST_SDA=1;
       wait (~I2C_CLK);wait (I2C_CLK);
       HOST_SDA_EN=0;
     end
   endtask
   
   task i2c_restart;  // SCL is supposed to be 0, SDA - any
     begin
       $display ("i2c RESTART at %t", $time);
       HOST_SCL=0;
       HOST_SCL_EN=1;
       wait (~I2C_CLK);wait (I2C_CLK); // wait for bus turnover (it was likely reading ACKN)
       HOST_SDA=1;
       HOST_SDA_EN=1;
       wait (~I2C_CLK);wait (I2C_CLK);
       HOST_SCL=1;
       HOST_SDA_EN=0;
       wait (~I2C_CLK);wait (I2C_CLK);
       HOST_SDA=0;
       HOST_SDA_EN=1;
       wait (~I2C_CLK);wait (I2C_CLK);
       HOST_SCL=0;
     end
   endtask

   task i2c_sendbyte;  // SCL is supposed to be 0, SDA - any
     input [7:0] b;
     reg ackn;
     integer i;
     begin
       $display ("i2c send (%x) at %t", b,$time);
       HOST_SCL=0;
       HOST_SCL_EN=1;
       wait (~I2C_CLK);wait (I2C_CLK); // wait for bus turnover (it was reading ACKN)
       for (i=7;i>=0;i=i-1) begin
//         # (I2CDLY);
         HOST_SDA=b[i];
         HOST_SDA_EN=1;
         wait (~I2C_CLK);wait (I2C_CLK);
         HOST_SCL=1;
         if (HOST_SDA) begin
           HOST_SDA_EN=0;
         end
         wait (~I2C_CLK);wait (I2C_CLK);
         HOST_SCL=0;
       end
       HOST_SDA_EN=0; // float for acknowledge
       wait (~I2C_CLK);wait (I2C_CLK);
       ackn=SDA0;
       #1;
       $display ("i2c acknowledge (%x)", ackn);
       HOST_SCL=1;
       wait (~I2C_CLK);wait (I2C_CLK);
       HOST_SCL=0;
     end
   endtask   

   task i2c_readbyte;  // SCL is supposed to be 0, SDA - any (more)
     input more;
     reg [7:0] d;
     integer i;
     begin
       $display ("i2c read (more=%x) at %t", more, $time);
       HOST_SCL=0;
       HOST_SCL_EN=1;
       HOST_SDA_EN=0;
       for (i=7;i>=0;i=i-1) begin
         wait (~I2C_CLK);wait (I2C_CLK);
         d[i]=SDA0;
         #1;
         HOST_SCL=1;
         wait (~I2C_CLK);wait (I2C_CLK);
         HOST_SCL=0;
       end
//       # (I2CDLY);
       wait (~I2C_CLK);wait (I2C_CLK); // wait for bus turnover (it was reading)
       HOST_SDA=~more;
       HOST_SDA_EN=1; 
       wait (~I2C_CLK);wait (I2C_CLK);
       HOST_SCL=1;
       if (HOST_SDA) begin
         HOST_SDA_EN=0;
       end
       wait (~I2C_CLK);wait (I2C_CLK);
       HOST_SCL=0;
       $display ("i2c read returned %x", d);
     end
   endtask   

		task i2c_send_4_bytes;
			input [31:0] di;
			begin
				i2c_sendbyte(di[31:24]);
				i2c_sendbyte(di[23:16]);
				i2c_sendbyte(di[15:8]);
				i2c_sendbyte(di[7:0]);
			end	
		endtask
		
		task i2c_send_2_bytes;
			input [15:0] di;
			begin
				i2c_sendbyte(di[15:8]);
				i2c_sendbyte(di[7:0]);
			end	
		endtask

endmodule

