/*!***************************************************************************
*! FILE NAME  : x359.v
*! DESCRIPTION: Top module
*! Copyright (C) 2008 Elphel, Inc.
*! -----------------------------------------------------------------------------**
*!  This program is free software: you can redistribute it and/or modify
*!  it under the terms of the GNU General Public License as published by
*!  the Free Software Foundation, either version 3 of the License, or
*!  (at your option) any later version.
*!
*!  This program is distributed in the hope that it will be useful,
*!  but WITHOUT ANY WARRANTY; without even the implied warranty of
*!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*!  GNU General Public License for more details.
*!
*!  You should have received a copy of the GNU General Public License
*!  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*! -----------------------------------------------------------------------------**
*!
*!  $Log: x359.v,v $
*!  Revision 1.3  2009/02/24 09:37:00  dzhimiev
*!  added reset from a counter on long "i_dcm_359_1.LOCKED" being low
*!
*!  Revision 1.7  2008/06/16 07:50:53  dzhimiev
*!  1. mcontr address range changed from 0x0820-0x082F to 0x0840-0x085F - for making extra channels
*!  2. added 1 readout channel for 3rd sensor
*!  3. 3 sensors supported
*!
*!  Revision 1.4  2008/05/01 20:46:29  dzhimiev
*!  working with sensors in snapshot mode
*!
*!  Revision 1.2  2008/04/23 01:55:49  dzhimiev
*!  1. added x359 files to src lists
*!  2. x359 read/write DDR
*!  3. x359 3 channels mux directly to out
*!  4. x359 one channel through DDR and another directly frames switching at out
*!
*/
module x359(
            RUN,       //low when programmed (switches external AND gate)
// SDRAM interface
            SDCLKE,    // SDRAM clock enable
            SDNCLK_FB, // SDRAM clock feedback inverted
            SDCLK,     // SDRAM clock inverted
            SDNCLK,    // SDRAM clock non-inverted
            SDCLK_FB,  // SDRAM clock feedback non-inverted
            SDLDM,     // SDRAM mask (low  byte)
            SDUDM,     // SDRAM mask (high byte)

            SDWE,      // SDRAM WE
            SDCAS,     // SDRAM CAS
            SDRAS,     // SDRAM RAS

            SDA,       // SDRAM address {BA[1:0],A[12:0]}
            SDD,       // SDRAM data bus D[15:0]
            UDQS,      // SDRAM data strobe - high byte
            LDQS,      // SDRAM data strobe - low byte
// port to host (simulating CMOS 5MPix interface)
            PXD,        // pixel data [11:2] to 10353
            DCLK,       // pixel clock from 10353
            BPF,        // pixel clock to 10353
            HACT,       // line sync   to 10353
            VACT,       // frame sync  to 10353
            MRST,       // sensor reset (from 10353)
            ARO,        // sensor trigger (from 10353)
            ARST,       // sensor standby (from 10353)
            SCL0,       // i2c clock from 10353
            SDA0,       // i2c data  from/to 10353
            CNVSYNC,    // pixel data [0] to 10353 (DC-DC converter sync from 10353)
            CNVCLK,     // pixel data [1] to 10353 (DC-DC converter clock from 10353)

// port 1 to sensor board
            PXD1,       // pixel data [11:2] from sensor
            PX_DCLK1,   // pixel clock to sensor
            PX_BPF1,    // pixel clock from sensor
            PX_HACT1,   // line sync   from sensor
            PX_VACT1,   // frame sync  from sensor
            PX_MRST1,   // sensor reset to sensor
            PX_ARO1,    // sensor trigger to sensor
            PX_ARST1,   // sensor standby to sensor
            PX_SCL1,    // i2c clock to sensor
            PX_SDA1,    // i2c data  to/from sensor
//            PX_CNVSYNC1,// pixel data [0] from sensor (DC-DC converter sync to sensor)
//            PX_CNVCLK1, // pixel data [1] from sensor (DC-DC converter clock to sensor)
            SENSPGM1,   // program attached sensor FPGA 

// port 2 to sensor board
            PXD2,       // pixel data [11:2] from sensor
            PX_DCLK2,   // pixel clock to sensor
            PX_BPF2,    // pixel clock from sensor
            PX_HACT2,   // line sync   from sensor
            PX_VACT2,   // frame sync  from sensor
            PX_MRST2,   // sensor reset to sensor
            PX_ARO2,    // sensor trigger to sensor
            PX_ARST2,   // sensor standby to sensor
            PX_SCL2,    // i2c clock to sensor
            PX_SDA2,    // i2c data  to/from sensor
//            PX_CNVSYNC2,// pixel data [0] from sensor (DC-DC converter sync to sensor)
//            PX_CNVCLK2, // pixel data [1] from sensor (DC-DC converter clock to sensor)
            SENSPGM2,   // program attached sensor FPGA 

// port 3 to sensor board
            PXD3,       // pixel data [11:2] from sensor
            PX_DCLK3,   // pixel clock to sensor
            PX_BPF3,    // pixel clock from sensor
            PX_HACT3,   // line sync   from sensor
            PX_VACT3,   // frame sync  from sensor
            PX_MRST3,   // sensor reset to sensor
            PX_ARO3,    // sensor trigger to sensor
            PX_ARST3,   // sensor standby to sensor
            PX_SCL3,    // i2c clock to sensor
            PX_SDA3,    // i2c data  to/from sensor
//            PX_CNVSYNC3,// pixel data [0] from sensor (DC-DC converter sync to sensor)
//            PX_CNVCLK3, // pixel data [1] from sensor (DC-DC converter clock to sensor)
            SENSPGM3    // program attached sensor FPGA 
            ,ALWAYS0    // pull-down to have 0 that compiler does not n\know is a constant
            );

    parameter MODELREV=32'h0359000a;
    
		wire USE5MPIX; // later will add control, currently - compile-time
	 
    output        RUN;
    
	 output			SDCLKE;
	 input			SDNCLK_FB;
	 output			SDCLK;
	 output			SDNCLK;
	 input			SDCLK_FB;
	 output			SDLDM;
	 output			SDUDM;

	 output			SDWE;
	 output			SDCAS;
	 output			SDRAS;

	 output [14:0]	SDA;
	 inout  [15:0]	SDD;
	 inout         UDQS;
	 inout         LDQS;

// port to host (simulating CMOS 5MPix interface)
    output [9:0]  PXD;       // pixel data [11:2] to 10353
    input         DCLK;      // pixel clock from 10353
    output        BPF;       // pixel clock to 10353
    output        HACT;      // line sync   to 10353
    output        VACT;      // frame sync  to 10353
    input         MRST;      // sensor reset (from 10353)
    input         ARO;       // sensor trigger (from 10353)
    input         ARST;      // sensor standby (from 10353)
    inout         SCL0;      // i2c clock from 10353
    inout         SDA0;      // i2c data  from/to 10353
    inout         CNVSYNC;   // pixel data [0] to 10353 (DC-DC converter sync from 10353)
    inout         CNVCLK;    // pixel data [1] to 10353 (DC-DC converter clock from 10353)

    inout  [11:0] PXD1;       // pixel data [11:2] from sensor
    output        PX_DCLK1;   // pixel clock to sensor
    input         PX_BPF1;    // pixel clock from sensor
    input         PX_HACT1;   // line sync   from sensor
    input         PX_VACT1;   // frame sync  from sensor
    output        PX_MRST1;   // sensor reset to sensor
    output        PX_ARO1;    // sensor trigger to sensor
    output        PX_ARST1;   // sensor standby to sensor
    inout         PX_SCL1;    // i2c clock to sensor
    inout         PX_SDA1;    // i2c data  to/from sensor
//    inout         PX_CNVSYNC1;// pixel data [0] from sensor (DC-DC converter sync to sensor)
//    inout         PX_CNVCLK1; // pixel data [1] from sensor (DC-DC converter clock to sensor)
	inout         SENSPGM1;   // program attached sensor FPGA 
	
	inout  [11:0] PXD2;       // pixel data [11:2] from sensor
	output        PX_DCLK2;   // pixel clock to sensor
	input         PX_BPF2;    // pixel clock from sensor
	input         PX_HACT2;   // line sync   from sensor
	input         PX_VACT2;   // frame sync  from sensor
	output        PX_MRST2;   // sensor reset to sensor
	output        PX_ARO2;    // sensor trigger to sensor
	output        PX_ARST2;   // sensor standby to sensor
	inout         PX_SCL2;    // i2c clock to sensor
	inout         PX_SDA2;    // i2c data  to/from sensor
//    inout         PX_CNVSYNC2;// pixel data [0] from sensor (DC-DC converter sync to sensor)
//    inout         PX_CNVCLK2; // pixel data [1] from sensor (DC-DC converter clock to sensor)
	inout         SENSPGM2;   // program attached sensor FPGA 
	
	inout  [11:0] PXD3;       // pixel data [11:2] from sensor
	output        PX_DCLK3;   // pixel clock to sensor
	input         PX_BPF3;    // pixel clock from sensor
	input         PX_HACT3;   // line sync   from sensor
	input         PX_VACT3;   // frame sync  from sensor
	output        PX_MRST3;   // sensor reset to sensor
	output        PX_ARO3;    // sensor trigger to sensor
	output        PX_ARST3;   // sensor standby to sensor
	inout         PX_SCL3;    // i2c clock to sensor
	inout         PX_SDA3;    // i2c data  to/from sensor
//    inout         PX_CNVSYNC3;// pixel data [0] from sensor (DC-DC converter sync to sensor)
//    inout         PX_CNVCLK3; // pixel data [1] from sensor (DC-DC converter clock to sensor)
	inout         SENSPGM3;   // program attached sensor FPGA 
	
	inout ALWAYS0;  // will be pulled down to fool the software - it does not know it is always 0.
	
	wire       nevr;
	wire       sdcl_fb,idclk,imrst,iaro,iarst;
	wire       ipx_bpf1,ipx_hact1,ipx_vact1;
	wire       ipx_bpf2,ipx_hact2,ipx_vact2;
	wire       ipx_bpf3,ipx_hact3,ipx_vact3;
	wire [11:0] ipxd1;
	wire [11:0] ipxd2;
	wire [11:0] ipxd3;
	wire        pclk;
	wire         isdclk=nevr;
	wire         never= nevr & sdcl_fb && idclk && imrst && iaro && iarst &&
							&ipxd1[11:0] && ipx_bpf1 && ipx_hact1 && ipx_vact1 &&
							&ipxd2[11:0] && ipx_bpf2 && ipx_hact2 && ipx_vact2 &&
							&ipxd3[11:0] && ipx_bpf3 && ipx_hact3 && ipx_vact3;
                        
	wire       icnvsync, icnvclk;
	wire       ipx_cnvsync1, ipx_cnvclk1;
	wire       ipx_cnvsync2, ipx_cnvclk2;
	wire       ipx_cnvsync3, ipx_cnvclk3;
	wire       ipx_dclk1, ipx_dclk2, ipx_dclk3;
                        
	reg  [11:0] pxdr=0;
	reg         ivact=0;
 	reg	        ibpf=0;
	reg 				ihact=0;
	reg         mrstr,aror,arstr;
	reg         data12bits, data10bits;

	wire [31:0]	sddo_p;		// data out to SDRAM, input to pad FF
	wire [31:0]	sddi_r;		// data from SDRAM  (1 cycle delayed)
	wire  [1:0] 	sddm_p; 	// write mask (sync with data)
	wire [12:0]	sda_p;		// SDRAM address,  input to pad FF
	wire [ 1:0]	sdba_p;		// same as above	
	wire [ 9:0]	sens_a;		// [7:0] channel 0 address (MSB - block #)
	wire [15:0]	sens_d;		// [15:0] channel 0 data in
	wire [15:0]	fpn_d;	
	reg	 [11:0]	ch2a=0;		// SDRAM channel 2 buffer ad         ARO;       // sensor trigger (from 10353)	 
	wire [15:0]	ch2do;		// SDRAM channel 2 data out	
	reg	 [11:0]	ch4a=0;		// SDRAM channel 2 buffer ad         ARO;       // sensor trigger (from 10353)	 
	wire [15:0]	ch4do;		// SDRAM channel 2 data out		
	wire  [4:0]	sr_nextFrame;	//[3:0]	
		
	reg trig_aro1=0, trig_aro2=0;
	reg aror1=1,aror2=1,aror3=1;
		
	wire     ipx_mrst1	= mrstr, 		ipx_mrst2	= mrstr, 		ipx_mrst3	= mrstr;
	wire     ipx_aro1		=  aror, 		ipx_aro2	=  aror2, 		ipx_aro3	=  aror3;
	//wire     ipx_aro1		=  trig_aro1, 		ipx_aro2	=  trig_aro2, 		ipx_aro3	=  aror;
	wire     ipx_arst1	= arstr, 		ipx_arst2	= arstr, 		ipx_arst3	= arstr;
			
	assign     ipx_cnvsync1 = icnvsync;
	assign     ipx_cnvclk1  = icnvclk;
	
	assign     ipx_cnvsync2 = icnvsync;
	assign     ipx_cnvclk2  = icnvclk;
	
	assign     ipx_cnvsync3 = icnvsync;
	assign     ipx_cnvclk3  = icnvclk;
	
	assign     ipx_dclk1 = idclk;
	//assign     ipx_dclk2 = idclk;
	//assign     ipx_dclk3 = idclk;
    
	PULLDOWN	i_PD_ALWAYS0(.O(ALWAYS0));
	IBUF			i_always0   (.I(ALWAYS0),.O(nevr));
	OBUF			i_RUN    		(.I(never),.O(RUN));
	
	reg SDCLKE_r=1'b0;
	
	always @ (negedge pclk) begin
		SDCLKE_r <= 1'b1;
	end	
  
  OBUF     i_SDCLKE (.I(SDCLKE_r), .O(SDCLKE));	

  IBUFDS   i_sdcl_fb(.O(sdcl_fb),.I(SDNCLK_FB),.IB(SDCLK_FB)); // not used

	assign PXD[9:0]=pxdr[11:2];
	wire sclk0, sclk180, sclk270;	

  OBUF     i_BPF     (.I(ibpf),.O(BPF));
  OBUF     i_HACT    (.I(ihact),.O(HACT));
  OBUF     i_VACT    (.I(ivact),.O(VACT));
  IBUF     i_DCLK    (.I(DCLK),.O(idclk));
  IBUF     i_MRST    (.I(MRST),.O(imrst));
  IBUF     i_ARO     (.I(ARO), .O(iaro));
  IBUF     i_ARST    (.I(ARST),.O(iarst));

//  IOBUF    i_SCL0    (.I(1'b0), .T(!never), .O(), .IO(SCL0));
//  IOBUF    i_SDA0    (.I(1'b0), .T(!never), .O(), .IO(SDA0));
  //IOBUF    i_CNVSYNC (.I(pxdr[ 0]), .T(data10bits), .O(icnvsync), .IO(CNVSYNC));
  //IOBUF    i_CNVCLK  (.I(pxdr[ 1]), .T(data10bits), .O(icnvclk), .IO(CNVCLK));
  assign CNVCLK =pxdr[1];
	assign CNVSYNC=pxdr[0];
//--- && &ipxd1[11:0] && (ipx_bpf1 && ipx_hact1 && ipx_vact1
  //IOBUF    i_PXD100     (.I(ipx_cnvsync1), .T(data12bits), .IO(PXD1[ 0]), .O(ipxd1[ 0]));
  //IOBUF    i_PXD101     (.I(ipx_cnvclk1),  .T(data12bits), .IO(PXD1[ 1]), .O(ipxd1[ 1]));
  IOBUF    i_PXD100     (.I(1'b0), .T(!never), .IO(PXD1[ 0]), .O(ipxd1[ 0]));
  IOBUF    i_PXD101     (.I(1'b0), .T(!never), .IO(PXD1[ 1]), .O(ipxd1[ 1]));	
  IOBUF    i_PXD102     (.I(1'b0), .T(!never), .IO(PXD1[ 2]), .O(ipxd1[ 2]));
  IOBUF    i_PXD103     (.I(1'b0), .T(!never), .IO(PXD1[ 3]), .O(ipxd1[ 3]));
  IOBUF    i_PXD104     (.I(1'b0), .T(!never), .IO(PXD1[ 4]), .O(ipxd1[ 4]));
  IOBUF    i_PXD105     (.I(1'b0), .T(!never), .IO(PXD1[ 5]), .O(ipxd1[ 5]));
  IOBUF    i_PXD106     (.I(1'b0), .T(!never), .IO(PXD1[ 6]), .O(ipxd1[ 6]));
  IOBUF    i_PXD107     (.I(1'b0), .T(!never), .IO(PXD1[ 7]), .O(ipxd1[ 7]));
  IOBUF    i_PXD108     (.I(1'b0), .T(!never), .IO(PXD1[ 8]), .O(ipxd1[ 8]));
  IOBUF    i_PXD109     (.I(1'b0), .T(!never), .IO(PXD1[ 9]), .O(ipxd1[ 9]));
  IOBUF    i_PXD110     (.I(1'b0), .T(!never), .IO(PXD1[10]), .O(ipxd1[10]));
  IOBUF    i_PXD111     (.I(1'b0), .T(!never), .IO(PXD1[11]), .O(ipxd1[11]));
  OBUF     i_PX_DCLK1   (.I(ipx_dclk1),        .O(PX_DCLK1));
  IBUFG     i_PX_BPF1    (.I(PX_BPF1),          .O(ipx_bpf1));
  IBUF     i_PX_HACT1   (.I(PX_HACT1),         .O(ipx_hact1));
  IBUF     i_PX_VACT1   (.I(PX_VACT1),         .O(ipx_vact1));
  OBUF     i_PX_MRST1   (.I(ipx_mrst1),        .O(PX_MRST1));
  OBUF     i_PX_ARO1    (.I(ipx_aro1),         .O(PX_ARO1));
  OBUF     i_PX_ARST1   (.I(ipx_arst1),        .O(PX_ARST1));
//  IOBUF    i_PX_SCL1    (.I(1'b0), .T(!never), .O(), .IO(PX_SCL1));
//  IOBUF    i_PX_SDA1    (.I(1'b0), .T(!never), .O(), .IO(PX_SDA1));
  IOBUF    i_SENSPGM1   (.I(1'b0), .T(!never), .O(), .IO(SENSPGM1));

//--- && &ipxd1[11:0] && (ipx_bpf1 && ipx_hact1 && ipx_vact1
//  IOBUF    i_PXD200     (.I(ipx_cnvsync2), .T(data12bits), .IO(PXD2[ 0]), .O(ipxd2[ 0]));
//  IOBUF    i_PXD201     (.I(ipx_cnvclk2),  .T(data12bits), .IO(PXD2[ 1]), .O(ipxd2[ 1]));
  IOBUF    i_PXD200     (.I(1'b0), .T(!never), .IO(PXD2[ 0]), .O(ipxd2[ 0]));
  IOBUF    i_PXD201     (.I(1'b0), .T(!never), .IO(PXD2[ 1]), .O(ipxd2[ 1]));
  IOBUF    i_PXD202     (.I(1'b0), .T(!never), .IO(PXD2[ 2]), .O(ipxd2[ 2]));
  IOBUF    i_PXD203     (.I(1'b0), .T(!never), .IO(PXD2[ 3]), .O(ipxd2[ 3]));
  IOBUF    i_PXD204     (.I(1'b0), .T(!never), .IO(PXD2[ 4]), .O(ipxd2[ 4]));
  IOBUF    i_PXD205     (.I(1'b0), .T(!never), .IO(PXD2[ 5]), .O(ipxd2[ 5]));
  IOBUF    i_PXD206     (.I(1'b0), .T(!never), .IO(PXD2[ 6]), .O(ipxd2[ 6]));
  IOBUF    i_PXD207     (.I(1'b0), .T(!never), .IO(PXD2[ 7]), .O(ipxd2[ 7]));
  IOBUF    i_PXD208     (.I(1'b0), .T(!never), .IO(PXD2[ 8]), .O(ipxd2[ 8]));
  IOBUF    i_PXD209     (.I(1'b0), .T(!never), .IO(PXD2[ 9]), .O(ipxd2[ 9]));
  IOBUF    i_PXD210     (.I(1'b0), .T(!never), .IO(PXD2[10]), .O(ipxd2[10]));
  IOBUF    i_PXD211     (.I(1'b0), .T(!never), .IO(PXD2[11]), .O(ipxd2[11]));
  OBUF     i_PX_DCLK2   (.I(ipx_dclk2),         .O(PX_DCLK2));
  IBUF     i_PX_BPF2    (.I(PX_BPF2),          .O(ipx_bpf2));
  IBUF     i_PX_HACT2   (.I(PX_HACT2),         .O(ipx_hact2));
  IBUF     i_PX_VACT2   (.I(PX_VACT2),         .O(ipx_vact2));
  OBUF     i_PX_MRST2   (.I(ipx_mrst2),        .O(PX_MRST2));
  OBUF     i_PX_ARO2    (.I(ipx_aro2),         .O(PX_ARO2));
  OBUF     i_PX_ARST2   (.I(ipx_arst2),        .O(PX_ARST2));
//  IOBUF    i_PX_SCL2    (.I(1'b0), .T(!never), .O(), .IO(PX_SCL2));
//  IOBUF    i_PX_SDA2    (.I(1'b0), .T(!never), .O(), .IO(PX_SDA2));
  IOBUF    i_SENSPGM2   (.I(1'b0), .T(!never), .O(), .IO(SENSPGM2));

//--- && &ipxd1[11:0] && (ipx_bpf1 && ipx_hact1 && ipx_vact1
//  IOBUF    i_PXD300     (.I(ipx_cnvsync3), .T(data12bits), .IO(PXD3[ 0]), .O(ipxd3[ 0]));
//  IOBUF    i_PXD301     (.I(ipx_cnvclk3),  .T(data12bits), .IO(PXD3[ 1]), .O(ipxd3[ 1]));
  IOBUF    i_PXD300     (.I(1'b0), .T(!never), .IO(PXD3[ 0]), .O(ipxd3[ 0]));
  IOBUF    i_PXD301     (.I(1'b0), .T(!never), .IO(PXD3[ 1]), .O(ipxd3[ 1]));
  IOBUF    i_PXD302     (.I(1'b0), .T(!never), .IO(PXD3[ 2]), .O(ipxd3[ 2]));
  IOBUF    i_PXD303     (.I(1'b0), .T(!never), .IO(PXD3[ 3]), .O(ipxd3[ 3]));
  IOBUF    i_PXD304     (.I(1'b0), .T(!never), .IO(PXD3[ 4]), .O(ipxd3[ 4]));
  IOBUF    i_PXD305     (.I(1'b0), .T(!never), .IO(PXD3[ 5]), .O(ipxd3[ 5]));
  IOBUF    i_PXD306     (.I(1'b0), .T(!never), .IO(PXD3[ 6]), .O(ipxd3[ 6]));
  IOBUF    i_PXD307     (.I(1'b0), .T(!never), .IO(PXD3[ 7]), .O(ipxd3[ 7]));
  IOBUF    i_PXD308     (.I(1'b0), .T(!never), .IO(PXD3[ 8]), .O(ipxd3[ 8]));
  IOBUF    i_PXD309     (.I(1'b0), .T(!never), .IO(PXD3[ 9]), .O(ipxd3[ 9]));
  IOBUF    i_PXD310     (.I(1'b0), .T(!never), .IO(PXD3[10]), .O(ipxd3[10]));
  IOBUF    i_PXD311     (.I(1'b0), .T(!never), .IO(PXD3[11]), .O(ipxd3[11]));
  OBUF     i_PX_DCLK3   (.I(ipx_dclk3),        .O(PX_DCLK3));
  IBUF     i_PX_BPF3    (.I(PX_BPF3),          .O(ipx_bpf3));
  IBUF     i_PX_HACT3   (.I(PX_HACT3),         .O(ipx_hact3));
  IBUF     i_PX_VACT3   (.I(PX_VACT3),         .O(ipx_vact3));
  OBUF     i_PX_MRST3   (.I(ipx_mrst3),        .O(PX_MRST3));
  OBUF     i_PX_ARO3    (.I(ipx_aro3),         .O(PX_ARO3));
  OBUF     i_PX_ARST3   (.I(ipx_arst3),        .O(PX_ARST3));
//  IOBUF    i_PX_SCL3    (.I(1'b0), .T(!never), .O(), .IO(PX_SCL3));
//  IOBUF    i_PX_SDA3    (.I(1'b0), .T(!never), .O(), .IO(PX_SDA3));
  IOBUF    i_SENSPGM3   (.I(1'b0), .T(!never), .O(), .IO(SENSPGM3));

  wire [7:0] dcm1_status;
	wire dcm1_locked;
	reg rst_dcm_1=1'b0;
	
	reg [3:0] i2c_cnt=2'b0;
	reg [7:0] i2c_reg_addr=8'b0;	
	wire	[ 7:0]	ia=i2c_reg_addr;		// internal address bus - before latches
	
	reg [31:0] nu_cnt=8'b0;
		
	wire pre_sclk0;		
		
  BUFG i_pclk (.I(idclk), .O(pclk)); 
	
	wire sclk0a,sclk90a,sclk180a,sclk270a;
	
	BUFG i_sclk0 	 (.I(pre_sclk0), 	 .O(sclk0));
//	BUFG i_sclk90  (.I(pre_sclk90), 	 .O(sclk90));
//	BUFG i_sclk180 (.I(pre_sclk180), .O(sclk180));
//	BUFG i_sclk270 (.I(pre_sclk270), .O(sclk270));
 
	reg  [31:0] idi=32'hffffffff;// this is smth like system bus
 
	wire da_dcm1;
	reg dcm1_en;
	reg dcm1_incdec;
	reg [1:0] dcm1_ph90=0;
	
	reg [7:0] dcm1_cnt=0;
	reg [7:0] dcm1_reg=0;
 
  always @ (negedge pclk) begin
    dcm1_en     <= da_dcm1 & (idi[1]!=idi[0]);
    dcm1_incdec <= da_dcm1 & idi[0]; 

		if (rst_dcm_1)     dcm1_cnt <= 0;
		else if (dcm1_en)	 
			if (dcm1_incdec) dcm1_cnt <= dcm1_cnt + 1;
			else             dcm1_cnt <= dcm1_cnt - 1;
  end

	always @ (negedge sclk0) begin
		dcm1_reg[7:0] <= dcm1_cnt[7:0];
	end

  wire  dcm1_clk=!pclk; // DCM is triggered by posedge
	 
	DCM #(.CLKOUT_PHASE_SHIFT("VARIABLE")) i_dcm_359_1(
    .CLKIN (pclk),
    .CLKFB (sclk0),
    .RST (rst_dcm_1), .PSEN (dcm1_en),.PSINCDEC (dcm1_incdec), .PSCLK (dcm1_clk),.DSSEN (1'b0),
    .CLK0 (pre_sclk0),
    .CLK90 (sclk90),
		.CLK180 (sclk180),
    .CLK270 (sclk270),
    .STATUS (dcm1_status[7:0]),
    .LOCKED (dcm1_locked)
		);

	wire da_dcm_s2;
	reg dcm_s2_en;
	reg dcm_s2_incdec;
	wire dclk2_locked;
	reg [7:0] dcm_s2_reg=0;
	reg [7:0] dcm_s2_cnt=0;
	reg [7:0] dclk2_cnt=0;
 
  always @ (negedge pclk) begin
    dcm_s2_en     <= da_dcm_s2 & (idi[1]!=idi[0]);
    dcm_s2_incdec <= da_dcm_s2 & idi[0];
	
		if (rst_dcm_1)       dcm_s2_cnt <= 0;
		else if (dcm_s2_en)
			if (dcm_s2_incdec) dcm_s2_cnt <= dcm_s2_cnt + 1;
			else               dcm_s2_cnt <= dcm_s2_cnt - 1;
  end

	always @ (negedge pclk) begin
		dcm_s2_reg[7:0] <= dcm_s2_cnt[7:0];
	end
	
	wire dcm_s2_clk=!pclk;	
	
	wire pre_ipx_dclk2;
	
	BUFG i_ipx_dclk2 	 (.I(pre_ipx_dclk2), 	 .O(ipx_dclk2));
	
	DCM #(.CLKOUT_PHASE_SHIFT("VARIABLE")) i_sensor2(
    //.CLKIN (sclk0),
		.CLKIN (pclk),
    //.CLKFB (ipx_bpf2),
		.CLKFB (ipx_dclk2),
    .RST (rst_dcm_1), .PSEN (dcm_s2_en),.PSINCDEC (dcm_s2_incdec), .PSCLK(dcm_s2_clk),.DSSEN(0),
    .CLK0 (pre_ipx_dclk2),
		.LOCKED (dclk2_locked)
		);		

  always @ (negedge ipx_dclk2) 
	begin
    dclk2_cnt <= dclk2_cnt + 1;
  end	

	wire da_dcm_s3;
	reg dcm_s3_en;
	reg dcm_s3_incdec;
	wire dclk3_locked;
	reg [7:0] dcm_s3_reg=0;
	reg [7:0] dcm_s3_cnt=0;
	reg [7:0] dclk3_cnt=0;
 
  always @ (negedge pclk) begin
    dcm_s3_en     <= da_dcm_s3 & (idi[1]!=idi[0]);
    dcm_s3_incdec <= da_dcm_s3 & idi[0];
	
		if (rst_dcm_1)       dcm_s3_cnt <= 0;
		else if (dcm_s3_en)
			if (dcm_s3_incdec) dcm_s3_cnt <= dcm_s3_cnt + 1;
			else               dcm_s3_cnt <= dcm_s3_cnt - 1;
  end

	always @ (negedge pclk) begin
		dcm_s3_reg[7:0] <= dcm_s3_cnt[7:0];
	end
	
	wire dcm_s3_clk=!pclk;	

	wire pre_ipx_dclk3;
	
	BUFG i_ipx_dclk3 	 (.I(pre_ipx_dclk3), 	 .O(ipx_dclk3));
		
	DCM #(.CLKOUT_PHASE_SHIFT("VARIABLE")) i_sensor3(
    //.CLKIN (sclk0),
		.CLKIN (pclk),
    //.CLKFB (ipx_bpf2),
		.CLKFB (ipx_dclk3),
    .RST (rst_dcm_1), .PSEN (dcm_s3_en),.PSINCDEC (dcm_s3_incdec), .PSCLK(dcm_s3_clk),.DSSEN(0),
    .CLK0 (pre_ipx_dclk3),
		.LOCKED (dclk3_locked)
		);

  always @ (negedge ipx_dclk3) 
	begin
    dclk3_cnt <= dclk3_cnt + 1;
  end	
	
	reg [4:0] adcmrst_d;
	
	always @ (negedge pclk) begin
		adcmrst_d[0] <= rst_dcm_1;
		adcmrst_d[1] <= adcmrst_d[0];
		adcmrst_d[2] <= adcmrst_d[1];
		adcmrst_d[3] <= adcmrst_d[2];
		adcmrst_d[4] <= adcmrst_d[3];
	end		

	wire [7:0]	dcm2_status;
	wire 				dcm2_locked;
	wire [1:0] 	phsel;	

 dcm333 i_dcm_359_2(
									 .sclk(sclk0),       						// in clock pad - 120MHz
                   .SDCLK(SDCLK),   							// out positive clock to SDRAM
                   .SDNCLK(SDNCLK),  							// out negative clock to SDRAM
									 .sdcl_fb(sdcl_fb),							// in feedback clock
                   .xclk(xclk),    								// 60MHz for compressor
									 .phsel(phsel[1:0]),   					// additional phase control for SDRAM CLK 
                   .dcm_rst(adcmrst_d[4]),// || dcm_rst_2), 	// reset DCM phase
                   .dcm_incdec(dcm_incdec),  			// variable phase control to adjust SDCLK so read DQS is aligned with sclk90/sclk270
									 .dcm_en(dcm_en),
                   .dcm_clk(!sclk0),//.dcm_clk(dcm_clk),
                   .dcm_done(dcm2_done),
                   .locked(dcm2_locked),  					// dcm locked
                   .status(dcm2_status[7:0])   		// dcm status (bit 1 - dcm clkin stopped)
);		

wire [4:0] chen_out;

reg [8:0] testd=0;
reg [11:0] testh_cnt=0;
reg testh=1;
reg testv=1, testv_d, testv_dd;

reg ttch=0;

  always @(negedge pclk) begin
    data12bits <=  USE5MPIX;
    data10bits <= ~USE5MPIX;
  end
		
	reg [5:0] ch_mux=6'b00001;
	
	reg Frame_Ready=0, Frame2_Ready=0;
	reg Frame_Ready_d=0, Frame2_Ready_d=0;	
	reg [15:0] aror_cnt=0;
	
	reg [31:0] Trig_delay=32'h00000015;
	reg        trig_delay_en=0;
	reg [31:0] trig_delay_cnt=0;
	reg aror_d;

  //always @ (negedge pclk) begin
	always @ (negedge sclk0) begin

    mrstr <= imrst;
		
		if (!aror) aror_cnt[15:0] <= aror_cnt[15:0] + 1;
    arstr <= iarst;
		
		if (trig_delay_cnt==Trig_delay)         trig_delay_en <= 0;
		else if ((!iaro & aror)|(iaro & !aror)) trig_delay_en <= 1;
		
		if ((!iaro & aror)|(iaro & !aror)) aror_d <= iaro;
		
		if (trig_delay_en) trig_delay_cnt <= trig_delay_cnt + 1;
		else 							 trig_delay_cnt <= 0;
		
		//aror  <=  (Trig_delay[31])?(trig_delay_cnt[30:0]==Trig_delay[30:0]?aror_d:aror ):(iaro);
		//aror2 <= (!Trig_delay[31])?(trig_delay_cnt[30:0]==Trig_delay[30:0]?aror_d:aror2):(iaro);
		aror <= iaro;
		if (trig_delay_cnt==Trig_delay) aror2 <= aror_d;
		
  end
	
	wire        i2c_slave_en;   // enable slave bus (may be disabled during fast/non-standard-timing communications with master)
	wire        i2c_scl;        // SCL from master
	wire        i2c_scl_en;     // enable sclk output to external slave device
	wire        i2c_sdami;      // SDA from master input
	wire        i2c_sdamo;      // SDA to master output
	wire        i2c_sdamen;     // SDA to master output enable
	wire        i2c_sdasi;      // SDA from external slave input
	wire        i2c_sdaso;      // SDA to external slave output
	wire        i2c_sdasen;     // SDA to external slave output enable
	wire [15:0] i2c_sr;         // 16-bit shift register output (skipped ACKN)
	wire        i2c_wra_stb;    // single-cycle strobe when sr contains register+slave address to read/write
	wire        i2c_wra_ackn;   // acknowledge address (active high input)
	wire        i2c_wrd_stb;    // single-cycle strobe when sr[7:0] contains byte to write (st[16:9] may have previous/MS byte)
	wire        i2c_wrd_ackn;   // acknowledge for the byte/word written
	wire        i2c_rd_req;     // request read byte (ackn from master), strobe after SCL goes high
	reg   [7:0] i2c_rdat;       // 8-bit data to send to master
	wire        i2c_rd_stb;     // rdat is updated
	wire        i2c_start;      // start strobe (just in case)
	wire        i2c_stop;       // stop strobe (some commands will be triggered now (i.e. connecting/disconnecting bridge to external slave)
	wire [31:0] i2c_do;
	
	
	wire        ext_i2c_disable;
	reg  [10:0] regfila; // register file address (10 bits - address, lsb - byte hi/lo)
	wire [ 7:0] sa_int = 8'h10; // slave addr 10.. 17
	wire [15:0] regfil_do; // output from register file;
	reg  [ 2:0] regfil_ro; // readout strobes for regfil Block RAM

	assign i2c_slave_en=!ext_i2c_disable;
	assign i2c_rd_stb = regfil_ro[2];

  wire          xfpgatdo;    // TDO read from an external FPGA
  wire          xfpgastat;   // Multiplexed read data from xjtag (senspgmin/xfpgadone/xfpgatdo)
	wire	[31:0]	bdo;		// 32-bit data from SDRAM channel3	
	wire	[31:0]	dsdo;		// 32-bit data from SDRAM descriptor memory
	wire 	[11:0]  io_pins;  // status of 12 i/o pins (from J2 connector)
  wire  [19:0]  pio_usec;
  wire  [31:0]  pio_sec;
	wire	[31:0]	dma_d0;
	wire	[31:0]	dma_d1;
	wire  [31:0]  hist_do; // histogram data out, actully only [17:0];	
	wire 	[31:0] 	rd_regs; // read 0x10-0x14

	reg [4:0] i2c_brd_address 					=5'b00010; // ability to change board address // wakes up with 0x10xx
	
	parameter [7:0] TEST_BRAM					  =8'h01;
	parameter [7:0] I2C_SDRAM_ADDRESS	  =8'h03;
	parameter [7:0] DA_DCM1_ADDRESS			=8'h07;
	parameter [7:0] DA_DCM2_ADDRESS			=8'h08;
	parameter [7:0] DA_DCM_S2_ADDRESS		=8'h09;
	parameter [7:0] DA_DCM_S3_ADDRESS		=8'h0a;		
	parameter [7:0]	RA_DCM_1						=8'h10;
	parameter [7:0] DA_DSWE_ADDRESS			=8'h40;// 40...5f
	parameter [7:0] WNR_ADDRESS					=8'h31;
	parameter [7:0] DA_NEXT_CH3_ADDRESS	=8'h32;
	parameter [7:0] DA_MEM_ADDRESS			=8'h33;
	parameter [7:0] DA_INIT_CH3_ADDRESS	=8'h34;
	parameter [7:0] SET_MUX							=8'h35;
	parameter [7:0] SET_BA							=8'h36;
	parameter [7:0] SET_FAKE						=8'h37;
	parameter [7:0] SET_DELAY						=8'h38;
	parameter [7:0] MEASURE						  =8'h39;
	parameter [7:0] LATCH					  	  =8'h3a;
	parameter [7:0] SET_TRIG_DELAY			=8'h3b;
	parameter [7:0] ADCMRST_ADDRESS			=8'h1b;

	wire   i2c_slava_ackn=(i2c_sr[7:3] == i2c_brd_address[4:0]); // will ackn. (using 7 MSBs from sr)
	
	assign i2c_wra_ackn   = 1'b1; // may be used no acknowledge only subset of addresses
	assign i2c_wrd_ackn   = 1'b1; // may be used no acknowledge only subset of data values

	reg i2c_wrd_stb_d=0;
	reg i2c_wra_stb_d=0;
	
	always @ (negedge pclk) begin
		i2c_wrd_stb_d <= i2c_wrd_stb;
		i2c_wra_stb_d <= i2c_wra_stb;
	end
		
	wire da_mem=			(i2c_reg_addr[7:0]==DA_MEM_ADDRESS[7:0]) 			& ((i2c_rd_req 	& (i2c_cnt[1:0]==0)));	
	
	wire da_test_bram=(i2c_reg_addr[7:0]==TEST_BRAM[7:0]);
	
	assign da_dcm1=		(i2c_reg_addr[7:0]==DA_DCM1_ADDRESS[7:0]) 		& (i2c_wrd_stb_d & (i2c_cnt[1:0]==0));
	assign da_dcm_s2=	(i2c_reg_addr[7:0]==DA_DCM_S2_ADDRESS[7:0]) 	& (i2c_wrd_stb_d & (i2c_cnt[1:0]==0));
	assign da_dcm_s3=	(i2c_reg_addr[7:0]==DA_DCM_S3_ADDRESS[7:0]) 	& (i2c_wrd_stb_d & (i2c_cnt[1:0]==0));	
	
	wire da_dcm2=			(i2c_reg_addr[7:0]==DA_DCM2_ADDRESS[7:0]) 		& (i2c_wrd_stb_d & (i2c_cnt[1:0]==0));
	wire da_dswe=			(i2c_reg_addr[7:5]==DA_DSWE_ADDRESS[7:5]) 		& (i2c_wrd_stb_d & (i2c_cnt[1:0]==0));
	wire da_init_ch3=	(i2c_reg_addr[7:0]==DA_INIT_CH3_ADDRESS[7:0])	& (i2c_wrd_stb_d & (i2c_cnt[1:0]==0));
	wire da_next_ch3=	(i2c_reg_addr[7:0]==DA_NEXT_CH3_ADDRESS[7:0])	& (i2c_wrd_stb_d & (i2c_cnt[1:0]==0));
	wire da_wnr=		 	(i2c_reg_addr[7:0]==WNR_ADDRESS[7:0]) 				& (i2c_wrd_stb_d & (i2c_cnt[1:0]==0));
	wire da_set_mux=	(i2c_reg_addr[7:0]==SET_MUX[7:0]) 						& (i2c_wrd_stb_d & (i2c_cnt[1:0]==0));
	wire da_set_ba=		(i2c_reg_addr[7:0]==SET_BA[7:0]) 					    & (i2c_wrd_stb_d & (i2c_cnt[1:0]==0));
	wire da_set_fake=	(i2c_reg_addr[7:0]==SET_FAKE[7:0]) 						& (i2c_wrd_stb_d & (i2c_cnt[1:0]==0));
	wire da_set_delay=(i2c_reg_addr[7:0]==SET_DELAY[7:0]) 					& (i2c_wrd_stb_d & (i2c_cnt[1:0]==0));
	wire da_measure=  (i2c_reg_addr[7:0]==MEASURE[7:0]) 						& (i2c_wrd_stb_d & (i2c_cnt[1:0]==0));
	wire da_latch_phase=(i2c_reg_addr[7:0]==LATCH[7:0]) 						& (i2c_wrd_stb_d & (i2c_cnt[1:0]==0));
	wire da_trig_delay=(i2c_reg_addr[7:0]==SET_TRIG_DELAY[7:0]) 		& (i2c_wrd_stb_d & (i2c_cnt[1:0]==0));
	
	always @ (negedge pclk) begin
		if (da_set_ba) i2c_brd_address[4:0]<=idi[4:0];
	end

	wire adcmrst=1'b0;//(i2c_brd_addr== ADCMRST_ADDRESS[7:0]);	

	wire dcm_1_sel=(i2c_reg_addr==RA_DCM_1) & (i2c_cnt[1:0]==0) & i2c_wrd_stb_d;
	//assign dcm_sel=da_next_ch3; // reg0x32
	
	reg rst_dcm_manual=0;
	reg rst_dcm_manual_d=0;
	reg pre0_rst_dcm_1=0;
	reg pre1_rst_dcm_1=0;
	reg pre2_rst_dcm_1=0;
	reg pre3_rst_dcm_1=0;			
	
	reg rst_lock=0;
	reg rst_lock_d=0;
	reg pre_rst_lock=0;
	reg dcm1_locked_d=0;
	reg dcm1_locked_dd=0;
	
	reg locked_cnt_en=0;
	reg [11:0] locked_cnt=0;
	
	SRLC16_1 #(.INIT(16'h000f)) INITIAL_DCM_RST (.Q(rst_from_srl16),.A0(1),.A1(1),.A2(1),.A3(1),.CLK(pclk),.D(0));
	
	always @ (negedge pclk) begin
		dcm1_locked_d  <= dcm1_locked;
		dcm1_locked_dd <= dcm1_locked_d;	
		
		locked_cnt <= locked_cnt + 1;

		rst_lock <= (locked_cnt[11:4]==8'hff);//&(!dcm1_locked_dd);
		
		rst_lock_d <= rst_lock & !dcm1_locked_dd;//dcm1_status[2] | (!dcm1_locked & dcm1_locked_d);//pre_rst_lock;//!dcm1_locked;
	
		pre3_rst_dcm_1 <= rst_from_srl16 | // initial reset
								rst_lock_d     | // reset from counter in case LOCKED is low for too long
		                  dcm1_status[1] | // reset from status output, which is clkin stop indicator
								(rst_dcm_manual & !rst_dcm_manual_d); // manual reset
		pre2_rst_dcm_1 <= pre3_rst_dcm_1;
		pre1_rst_dcm_1 <= pre2_rst_dcm_1;
		//pre0_rst_dcm_1 <= pre1_rst_dcm_1;
		rst_dcm_1 <= pre3_rst_dcm_1 | pre2_rst_dcm_1 | pre1_rst_dcm_1;// | pre0_rst_dcm_1;
	end

		
	always @ (negedge pclk) begin
		if (i2c_wra_stb) i2c_reg_addr <= i2c_sr[7:0];
		if (dcm_1_sel) rst_dcm_manual <= i2c_sr[0];
		rst_dcm_manual_d <= rst_dcm_manual;
	end
	
	always @ (negedge pclk) 
	begin				
		if (i2c_wrd_stb & (i2c_cnt[1:0]==1)) idi[31:16] <= i2c_sr;
		if (i2c_wrd_stb & (i2c_cnt[1:0]==3)) idi[15: 0] <= i2c_sr;

		if (i2c_start)                       i2c_cnt <= 4'b0;
		else if (i2c_wrd_stb | i2c_rd_stb)   i2c_cnt <= i2c_cnt + 1;

    regfil_ro[2:0] <= {regfil_ro[1:0],i2c_rd_req};
		
    if      (i2c_wra_stb)                regfila[10:0] <= {i2c_sr[10:9], i2c_sr[7:0], 1'b0}; // i2c_sr[8] is direction 1 - read, 2 - write
    else if (i2c_wrd_stb | i2c_rd_stb)   regfila[10:0] <= regfila[10:0] + 1;
  end

	wire [15:0] ch3a_out;
	wire [1:0] dcm_err;
	reg [9:0] sens_a1=0;
	
	reg [31:0] active_reg=0,status_reg=0;
	
	reg [15:0] ch0_test_reg=0;	
	reg [31:0] i2c_do_r=0;

	always @ (negedge pclk) begin
		if (regfil_ro[1]) 
			case (i2c_cnt[1:0])
				2'b00: i2c_rdat <= i2c_do[31:24];
				2'b01: i2c_rdat <= i2c_do[23:16];
				2'b10: i2c_rdat <= i2c_do[15:8];
				2'b11: i2c_rdat <= i2c_do[7:0];
			endcase	
	end
		
//	always @ (negedge pclk) begin
//		if (regfil_ro[1]) 
//			case (i2c_cnt[1:0])
//				2'b00: i2c_rdat <= i2c_do[31:24];
//				2'b01: i2c_rdat <= i2c_do[23:16];
//				2'b10: i2c_rdat <= i2c_do[15:8];
//				2'b11: i2c_rdat <= i2c_do[7:0];
//			endcase	
//	end

  FDE i_ext_i2c_disable (.Q(ext_i2c_disable),.C(!pclk),.CE(i2c_wrd_stb && (regfila[10:0] == 11'h03)), .D(i2c_sr[0]));
  i2csbr i_i2csbr (.clk(!pclk),                 // global clock (30-150MHz?)
                   .slave_en(i2c_slave_en),     // enable slave bus (may be disabled while fast/non-standard communications with master)
                   .scl(i2c_scl),               // SCL from master
                   .scl_en(i2c_scl_en),         // enable sclk output to external slave device
                   .sdami(i2c_sdami),           // SDA from master input
                   .sdamo(i2c_sdamo),           // SDA to master output
                   .sdamen(i2c_sdamen),         // SDA to master output enable
                   .sdasi(i2c_sdasi),           // SDA from external slave input
                   .sdaso(i2c_sdaso),           // SDA to external slave output
                   .sdasen(i2c_sdasen),         // SDA to external slave output enable
                   .sr(i2c_sr[15:0]),           // 16-bit shift register output (skipped ACKN)
                   .slava_ackn(i2c_slava_ackn), // will ackn. (using 7 MSBs from sr)
                   .wra_stb(i2c_wra_stb),       // single-cycle strobe when sr contains register+slave address to read/write
                   .wra_ackn(i2c_wra_ackn),     // acknowledge address (active high input)
                   .wrd_stb(i2c_wrd_stb),       // single-cycle strobe when sr[7:0] contains byte to write (st[16:9] may have previous/MS byte)
                   .wrd_ackn(i2c_wrd_ackn),     // acknowledge for the byte/word written
                   .rd_req(i2c_rd_req),         // request read byte (ackn from master), strobe after SCL goes high
                   .rdat(i2c_rdat[7:0]),        // 8-bit data to send to master
                   .rd_stb(i2c_rd_stb),         // rdat is updated
                   .start(i2c_start),           // start strobe (just in case)
                   .stop(i2c_stop));            // stop strobe (some commands will be triggered now (i.e. connecting/disconnecting bridge to external slave)

  PULLUP i_sdam_pu (.O(SDA0));
  PULLUP i_sclm_pu (.O(SCL0));
	
  PULLUP i_sdas_pu1 (.O(PX_SDA1));
  PULLUP i_scls_pu1 (.O(PX_SCL1));
	PULLUP i_sdas_pu2 (.O(PX_SDA2));
  PULLUP i_scls_pu2 (.O(PX_SCL2));
  PULLUP i_sdas_pu3 (.O(PX_SDA3));
  PULLUP i_scls_pu3 (.O(PX_SCL3));	
	
	wire i2c_scl_en1,i2c_scl_en2,i2c_scl_en3;
	wire i2c_sdasen1,i2c_sdasen2,i2c_sdasen3;
	                
  IOBUF    i_sclm     (.I(1'b0),    	.T(1'b1),        .O(i2c_scl), 	.IO(SCL0));
  IOBUF    i_sdam     (.I(i2c_sdamo), .T(~i2c_sdamen), .O(i2c_sdami), .IO(SDA0));

	assign i2c_scl_en1 = i2c_scl_en & ch_mux[0];
	assign i2c_sdasen1 = i2c_sdasen & ch_mux[0];
	
	IOBUF    i_scls1     (.I(i2c_scl), 	.T(~i2c_scl_en1), .O(),        		.IO(PX_SCL1));
  IOBUF    i_sdas1     (.I(i2c_sdaso),.T(~i2c_sdasen1), .O(i2c_sdasi1), .IO(PX_SDA1));
	
	assign i2c_scl_en2 = i2c_scl_en & ch_mux[1];
	assign i2c_sdasen2 = i2c_sdasen & ch_mux[1];
	IOBUF    i_scls2     (.I(i2c_scl), 	.T(~i2c_scl_en2), .O(),        		.IO(PX_SCL2));
  IOBUF    i_sdas2     (.I(i2c_sdaso),.T(~i2c_sdasen2), .O(i2c_sdasi2), .IO(PX_SDA2));

	assign i2c_scl_en3 = i2c_scl_en & ch_mux[2];
	assign i2c_sdasen3 = i2c_sdasen & ch_mux[2];
	IOBUF    i_scls3     (.I(i2c_scl), 	.T(~i2c_scl_en3), .O(),        		.IO(PX_SCL3));
  IOBUF    i_sdas3     (.I(i2c_sdaso),.T(~i2c_sdasen3), .O(i2c_sdasi3), .IO(PX_SDA3));
	
	assign i2c_sdasi = 		ch_mux[0]?
													i2c_sdasi1     // ch_mux[2:0]=001|011|101|111
													:
													(ch_mux[1]?
														i2c_sdasi2   // ch_mux[2:0]=010|110
														:
														(ch_mux[2]?
															i2c_sdasi3 // ch_mux[2:0]=001
															:
															1'b0));    // ch_mux[2:0]=000


	// check RAM
	reg [9:0] bram_cnt_A=0;
	reg [9:0] bram_cnt_B=0;
	reg i2c_rd_stb_d;
	
	always @ (negedge pclk)
	begin
			i2c_rd_stb_d <= i2c_rd_stb;
	
			if (i2c_cnt[0]) begin
				if (i2c_wrd_stb & da_test_bram)                bram_cnt_A <= bram_cnt_A + 1;				
			  if (i2c_rd_stb & (i2c_reg_addr[7:3]=='b01101)) bram_cnt_B <= bram_cnt_B + 1;
			end
	end
	
  RAMB16_S18_S18  TEST_RAMB16_S18_S18 (
      .DOA(),                            // Port A 16-bit Data Output
      .DOPA(),                           // Port A 2-bit Parity Output
			.ADDRA(bram_cnt_A[9:0]),           // Port A 10-bit Address Input
      .CLKA(!pclk),                      // Port A Clock
      .DIA(i2c_sr[15:0]),                // Port A 16-bit Data Input
      .DIPA(2'h0),                       // Port A 2-bit parity Input
      .ENA(1'b1),                        // Port A RAM Enable Input
      .SSRA(1'b0),                       // Port A Synchronous Set/Reset Input
      .WEA(i2c_wrd_stb & da_test_bram & i2c_cnt[0]), // Port A Write Enable Input
      .DOB(regfil_do[15:0]),             // Port B 16-bit Data Output
      .DOPB(),                           // Port B 2-bit Parity Output
      .ADDRB(bram_cnt_B[9:0]),           // Port B 10-bit Address Input
      .CLKB(!pclk),                      // Port B Clock
      .DIB(16'h0),                       // Port B 16-bit Data Input
      .DIPB(2'h0),                       // Port-B 2-bit parity Input
      .ENB(1'b1),                        // Port B RAM Enable Input
      .SSRB(1'b0),                       // Port B Synchronous Set/Reset Input
      .WEB(1'b0)                         // Port B Write Enable Input
   );

	wire sdwe_p,sdcas_p,sdras_p;
	wire sddqt_manual;	
	reg  sddqt_manual_1;

	always @ (negedge sclk0) begin	
		if (!sdwe_p & !sdcas_p & sdras_p)      sddqt_manual_1=1'b0; // start of write
		else if (!sdwe_p & sdcas_p & !sdras_p) sddqt_manual_1=1'b1; // end of write (precharge)
	end

	assign sddqt_manual=(!sdwe_p & !sdcas_p & sdras_p)?1'b0:sddqt_manual_1;

	// IO pads and related FFs
	
	sddrio16 i_SDDd(.c0(sclk0),
									//.c90(sclk90),
									.c270(sclk270),
									//.d(sddo_p[31:0]),
									.d({sddo_p[31:1],1'b1}), // that was for check
									.t(pretrist & sddqt_manual),.q(sddi_r[31:0]),.dq(SDD[15:0]));

	wire [1:0] sddm_pp;
	
	assign sddm_pp[1]= sddm_p[1] & sddqt_manual;
	assign sddm_pp[0]= sddm_p[0] & sddqt_manual;

	sddrdm        i_SDUDM  (.c0(sclk0),/*.c90(sclk90),*/.c270(sclk270),.d(sddm_pp[1:0]),.dq(SDUDM));
	sddrdm        i_SDLDM  (.c0(sclk0),/*.c90(sclk90),*/.c270(sclk270),.d(sddm_pp[1:0]),.dq(SDLDM));
	sdo15_2       i_SDA    (.c(sclk0),.d({sdba_p[1:0],sda_p[12:0]}),.q(SDA[14:0]));		 
	
	sdo1_2        i_SDRAS  (.c(sclk0),.d(sdras_p),.q(SDRAS));
	sdo1_2        i_SDCAS  (.c(sclk0),.d(sdcas_p),.q(SDCAS));
	sdo1_2        i_SDWE   (.c(sclk0),.d(sdwe_p),. q(SDWE ));

	// temporary change behaviour of dqs2 to fix pinout problem - will influence adjustment goal
  dqs2          i_sddqs(.c0(sclk0),/*.c90(sclk90),*/.c270(sclk270),
//     dqs2 i_sddqs(.c0(sclk0),.c0comb(sclk270),.c90(sclk90),.c270(sclk270),
	                      .t       (sddqt && sddqt_manual),       // 1/2 cycle before cmd "write" sent out to the SDRAM, sync to sclk180
	                      .UDQS    (UDQS),         // UDQS I/O pin
	                      .LDQS    (LDQS),         // LDQS I/O pin
	                      .udqsr90 (udqsr90),      // data from SDRAM interface pin UDQS strobed at rising sclk90
	                      .ldqsr90 (ldqsr90),      // data from SDRAM interface pin LDQS strobed at rising sclk90
	                      .udqsr270(udqsr270),     // data from SDRAM interface pin UDQS strobed at rising sclk270
	                      .ldqsr270(ldqsr270)      // data from SDRAM interface pin UDQS strobed at rising sclk270
//       ,.qtmp(qtmp[3:0]) // temporary
	              );

// assumed CL=2.5
 sdram_phase i_sdram_phase(
//***********************  .wclk(cwr),       // global CPU WE pulse
                           .pre_wcmd(da_dcm2),       // decoded address - enables wclk
                           .wd(idi[3:0]),         // CPU write data [3:0]
                                          //       0 - nop, just reset status data
                                          //       1 - increase phase shift
                                          //       2 - decrease phase shift
                                          //       3 - reset phase shift to default (preprogrammed in FPGA configuration)
                           .ph_err(dcm_err[1:0]),     // [1:0] 0 - no data (SDRAM reads) since last change (wclk*wcmd)
                                       //       1 - clock is too late
                                       //       2 - clock is too early
                                       //       3 - OK (some measurements show too late, some - too early)
                           .sclk0(sclk0),      // global clock, phase 0
//                           .sclk90(sclk90),     // global clock, phase 0
                           .sclk270(sclk270),    // global clock, phase 0
                           .enrd180(sd_dqsrd),    // read enable, latency 2 from the command, sync with sclk0 falling edge
                           .udqsr90(udqsr90),    // data from SDRAM interface pin UDQS strobed at rising sclk90
                           .ldqsr90(ldqsr90),    // data from SDRAM interface pin LDQS strobed at rising sclk90
                           .udqsr270(udqsr270),   // data from SDRAM interface pin UDQS strobed at rising sclk270
                           .ldqsr270(ldqsr270),   // data from SDRAM interface pin UDQS strobed at rising sclk270
                           .dcm_rst(dcm_rst),    // set DCM phase to default
                           .dcm_clk(dcm_clk),//.dcm_clk(dcm_clk),    // clock for changing DCM phase (now == sclk0)
                           .dcm_en(dcm_en),     // enable inc/dec of the DCM phase
                           .dcm_incdec(dcm_incdec), // 1 - increment, 0 - decrement DCM phase
                           .phase90sel(phsel[1:0])  // add phase 0 - 0, 1 - 90, 2 - 180, 3 - 270
                     );

reg     ipx_bpf1_dl=0,   ipx_bpf2_dl=0,   ipx_bpf3_dl=0;

wire    ipx_vact1_dl,  ipx_vact2_dl,  ipx_vact3_dl;
wire    ipx_hact1_dl,  ipx_hact2_dl,  ipx_hact3_dl;
wire [11:0] ipxd1_dl,      ipxd2_dl,      ipxd3_dl;

// Other modules
	wire [9:0] fpn_a;
	reg [15:0] px_d1,px_d1_d, px_d2,px_d2_d;
	reg px_hact1, px_hact1_d, px_hact1_d2, px_hact2, px_hact2_d, px_hact2_d2;
	reg px_vact1, px_vact1_d, px_vact2, px_vact2_d;
	
	wire [15:0] sens_d1, sens_d2;

	reg stch2=0;
	reg stch4=0;

// for test
always @ (negedge sclk0) begin
	if (stch2)       nu_cnt[ 7:0] <= nu_cnt[ 7:0] + 1;
	//if (sens_wpage1) nu_cnt[15:8] <= nu_cnt[15:8] + 1;
end 

wire ch2weo;
wire ch4weo;

reg ch0_vact_imt=0, ch1_vact_imt=0;
reg ch0_hact_imt=0, ch1_hact_imt=0;
reg [15:0] ch0_vact_imt_cnt=0, ch1_vact_imt_cnt=0;
reg [15:0] ch0_hact_imt_cnt=0, ch1_hact_imt_cnt=0;
reg ch0_wpage_imt=0, ch1_wpage_imt=0;
reg [9:0] ch0a=0, ch1a=0;

reg ch2_en_a=0;
reg [11:0] ch2_a_cnt=0;
reg stch2_first=0;
reg stch4_first=0;
reg [15:0] ch2_hact_imt_cnt=0;
reg [15:0] ch4_hact_imt_cnt=0;
reg [15:0] px_d1a=0,px_d1a_d=0;
reg [15:0] px_d2a=0,px_d2a_d=0;

reg [15:0] ch0_N_fl=4;
reg [15:0] ch0_N_fp=1619;
reg [15:0] ch2_N_fl=4;
reg [15:0] ch2_N_fp=1619;
reg [15:0] pre_N_l=4;
reg [15:0] pre_N_p=1619;

reg [31:0] MAX_DELAY=32'h00000fff;

	mcontr i_mcontr(
                   .clk0(sclk0),	  // SDRAM clocks (hope to get 120MHz)
//                 .mclk(cwr),    // modified CPU global clock - write and read with a7==1 (for block memory R/W)
//descriptor memory and channel 3
                   .ia(ia[4:0]),       // internal 4-bit address bus (fast - directly from I/O pads)
                   .as(ia[4:0]),//.as(as[3:0]),		   // 4 bit address to select descriptor address and data word (3 bits)
                   .am(ia[4:0]),//.am(am[3:0]),       // switching between ia (async read) and as (sync write)
                   .dscs(da_dswe),		// WE for descriptor memory
//
                   .mwnr(da_wnr),//.mwnr(wnr),	         	// CPU write, not read (valid @ negedge clk0)
                   .init_ch3(da_init_ch3),   // write to init cnhannel 3 (will reset address r/w)
                   .next_ch3(da_next_ch3),   // advance to the next channel3 page, reset address
									 .ch3a_out(ch3a_out),
                   .menrw(da_mem),           // enable CPU read/write  32-bit word from buffer(valid @ mweoe)

						 .di(idi[31:0]),				// // 32-bit data from CPU (descriptor and channel 3)
//                 .d_late(d_late[31:0]),
					    .dsdo(dsdo[31:0]),			// [31:0] data out from descriptor memory
					    .do(bdo[31:0]),				// 32-bit data to CPU (channel 3)
							//.do_2_we(do_2_we),  // testing manual write and read

						 .rdy(sr_memrdy),				// ready to r/w next 8x32 page (channel 3) CPU<->SDRAM
						 .wrempty(sr_wrempty), 		// CPU write buffer empty (valid only in write mode). May reprogram channel3

							////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
				      .ch0clk(sclk0),//pclk),				// clock for channel0
							.ch0we(px_hact1_d & !Frame_Ready),				// channel 0 (sensor->SDRAM) WE
							.ch0a (ch0a[9:0]),			// [7:0] channel 0 address (MSB - block #)
							.ch0di(sens_d1),
					    .stch0(ch0_wpage_imt),			// start channel 0 transfer (up to 2 w/o confirmation)
					    .ch0rdy(sr_ch0rdy),				// channel 0 ready -- not used
							////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//			            .ch1clk(pclk),				// clock for channel1
//					    .ch1a(fpn_a[9:0]),			// [7:0] channel 1 (SDRAM -> sensor calibration data) address (MSB - block #)
//					    .ch1do(fpn_d[15:0]),		// [15:0] channel 1 data out
//					    .stch1(fpn_rpage),			// start channel 1 transfer (up to 2 w/o confirmation)
//					    .ch1rdy(sr_ch1rdy),				// channel 1 ready -- not used
				      .ch1clk(sclk0),//pclk),				// clock for channel0
							.ch1we(px_hact2_d & !Frame_Ready),				// channel 0 (sensor->SDRAM) WE
							.ch1a (ch1a[9:0]),			// [7:0] channel 0 address (MSB - block #)
							.ch1di(sens_d2),
					    .stch1(ch1_wpage_imt),			// start channel 0 transfer (up to 2 w/o confirmation)
					    .ch1rdy(sr_ch1rdy),				// channel 0 ready -- not used
							////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			        .ch2clk(!sclk0),//xclk),				// clock for channech2frm_rdyl2 
							.ch2a(ch2a[11:0]),
							.ch2do(ch2do[15:0]),			// [7:0] channel 2 data out // 2008-02-22
							.stch2(stch2 | stch2_first),				// start channel 2 transfer
							.ench2(1),                   // enable read from channel 2 buffer to compressor				
							.ch2weo(ch2weo),
//			        .ch2clk(!sclk0),//xclk),				// clock for channech2frm_rdyl2 
//							.ch2a(0),
//							.ch2do(),			// [7:0] channel 2 data out // 2008-02-22
//							.stch2(0),				// start channel 2 transfer
//							.ench2(1),                   // enable read from channel 2 buffer to compressor			
//							.ch2weo(),
							////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			        .ch4clk(!sclk0),//xclk),		 // clock for channel 4 
							.ch4a(ch4a[11:0]),
							.ch4do(ch4do[15:0]),			   // [7:0] channel 2 data out // 2008-02-22
							.stch4(stch4 | stch4_first),				// start channel 2 transfer
							.ench4(1),                   // enable read from channel 2 buffer to compressor					
							.ch4weo(ch4weo),
//			        .ch4clk(!sclk0),//xclk),		 // clock for channel 4 
//							.ch4a(0),
//							.ch4do(),			   // [7:0] channel 2 data out // 2008-02-22
//							.stch4(0),				// start channel 2 transfer
//							.ench4(1),                   // enable read from channel 2 buffer to compressor					
//							.ch4weo(),							
							////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
							.chen_out(chen_out[4:0]),
							////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
					    .nextFrame(sr_nextFrame[4:0]),// (level) generated before actual block is filled - needed to be combined with the pulse from buffer control
					    .sddi(sddi_r[31:0]),		// [15:0] data from SDRAM  (1 cycle delayed)
					    .sddo(sddo_p[31:0]),		// [15:0] data to SDRAM		(1 cycle ahead)
					    .sda(sda_p[12:0]),			// [11:0] address to SDRAM (1 cycle ahead)
					    .sdba(sdba_p[1:0]),			// [ 1:0] bank address to SDRAM (1 cycle ahead)
                   .sdwe(sdwe_p),				// WE  command bit to SDRAM (1 cycle ahead)
					    .sdras(sdras_p),				// CAS command bit to SDRAM (1 cycle ahead)
					    .sdcas(sdcas_p),				// CAS command bit to SDRAM
						 .trist(pretrist),			// tristate data to SDRAM   (1 cycle ahead)
                   .predqt(sddqt),           // fristate DQ outputs (one extra for FF in output buffer)
                   .dmask(sddm_p[1:0]),      // [1:0] - now both the same (even number of words written)
                   .dqs_re(sd_dqsrd));       // enable read from DQS i/o-s for phase adjustments  (latency 2 from the SDRAM RD command)

reg trig=0, trig_d=0;
reg trig_rst=0;

always @ (negedge sclk0) begin
	trig <= aror;
	trig_d <= trig;
	trig_rst <= !trig & trig_d;
end

reg da_set_mux_d=0;

reg rst_reg=1;

always @ (negedge sclk0) begin
	if ((chen_out[0] & chen_out[2])) rst_reg <= 0;
	else                             rst_reg <= 1;
end

wire rst=(rst_reg)|(da_set_mux_d)|(trig_rst);

always @ (negedge sclk0) da_set_mux_d <= da_set_mux;

always @ (negedge sclk0) begin
	if (rst)
		ch0a[9:0] <= 0;
	else
		if      ((ch0a[7:0]==255 | px_d1a_d[15:0]==ch0_N_fp) & (!Frame_Ready | px_hact1_d)) ch0a[9:0] <= {ch0a[9:8]+2'b01,8'b0};
		else if (px_hact1_d & !Frame_Ready)                                                 ch0a[9:0] <= ch0a[9:0] + 1;
end

always @ (negedge sclk0) begin
	if (rst)
		ch1a[9:0] <= 0;
	else
		if      ((ch1a[7:0]==255 | px_d2a_d[15:0]==ch0_N_fp) & (!Frame2_Ready | px_hact2_d)) ch1a[9:0] <= {ch1a[9:8]+2'b01,8'b0};
		else if (px_hact2_d & !Frame2_Ready)                                                 ch1a[9:0] <= ch1a[9:0] + 1;
end

reg px_hact1_dd=0,  px_hact2_dd=0;
reg px_vact1_dd=0,  px_vact2_dd=0;
reg px_vact1_ddd=0, px_vact2_ddd=0;
reg [31:0] ctrl_reg=1;
reg px_vact_go=0;
reg px_vact_go_d=0;

reg EnV1=0,EnV2=0,EnV3=0;
reg EnV1_d=0;	
reg EnV3_d=0;

reg ipx_vact2_d=0;
reg ipx_vact2_dd=0;
reg ipx_hact2_d=0;
reg [11:0] ipxd2_d=0;

reg EnV2_enable=1;

reg ch2_hact_imt=0;
reg ch2_vact_imt=0;
reg [15:0] ch2weo_cnt=0;
reg [15:0] ch2_line_cnt=0;
reg ch2_buf_rdy=0;
reg [7:0] stch2_cnt=0;
reg Line_Ready=0;
reg [15:0] ch2_vact_imt_cnt=0;
reg ch2_hact_imt_d=0;
reg ch2_vact_imt_d=0;
reg Line_Ready_d=0;

reg ch4_hact_imt=0;
reg ch4_vact_imt=0;
reg [15:0] ch4weo_cnt=0;
reg [15:0] ch4_line_cnt=0;
reg ch4_buf_rdy=0;
reg [7:0] stch4_cnt=0;
reg Line2_Ready=0;
reg [15:0] ch4_vact_imt_cnt=0;
reg ch4_hact_imt_d=0;
reg ch4_vact_imt_d=0;
reg Line2_Ready_d=0;

//reg [2:0] sphase=3'b111;
reg [8:0] sphase=9'b000000111;

always @ (negedge sclk0) begin
	ipx_vact2_d <= ch_mux[4]?ipx_vact2_dl:ipx_vact1_dl;
	ipx_hact2_d <= ch_mux[4]?ipx_hact2_dl:ipx_hact1_dl;
	ipxd2_d     <= ch_mux[4]?    ipxd2_dl:    ipxd1_dl;
end	
	
reg ipx_vact1_dl_d=0;	
reg ipx_vact2_dl_d=0;
	
always @ (negedge sclk0) begin
	
	ipx_vact1_dl_d <= ipx_vact1_dl;
	ipx_vact2_dl_d <= ipx_vact2_dl;
	
	if (da_latch_phase) sphase[8:0] <= idi[8:0];
	
	if (da_set_mux) ch_mux[5:0] <= idi[5:0];
	if (da_set_mux) ctrl_reg[0] <= idi[6];
	
	if (da_set_mux)                          trig_aro1 <= !idi[7];
	else if (ipx_vact1_dl & !ipx_vact1_dl_d) trig_aro1 <= 1;

	if (da_set_mux)                          trig_aro2 <= !idi[7];
	else if (ipx_vact2_dl & !ipx_vact2_dl_d) trig_aro2 <= 1;
	
	if (da_trig_delay) Trig_delay <= idi[31:0];
	
end	

assign USE5MPIX=  ctrl_reg[0];

reg Frame_Locked=0, Frame2_Locked=0;
	
reg delay_en=0;
reg delay_en1=0;
reg delay_en2=0;
reg [31:0] delay_cnt=0;
reg [31:0] delay_cnt2=0;	
	
always @ (negedge sclk0) begin	
	if (rst) begin
		EnV2 <= 1;
		EnV1 <= 0;
		EnV3 <= 0;
		ipx_vact2_dd <= 0;
		Frame_Locked <= 0;
		Frame2_Locked <= 0;
		
		EnV1_d <= 0;
		EnV3_d <= 0;
		delay_en <= 0;
		delay_cnt <= 0;
		delay_en2 <= 0;
		delay_cnt2 <= 0;
	end	
	else begin
	
		EnV1_d <= EnV1;
		EnV3_d <= EnV3;
	
		if (!ipx_vact2_d & ipx_vact2_dd) EnV2 <= 0;
		
		if (!ipx_vact2_d & ipx_vact2_dd & EnV2) delay_en <= 1;
		else if (EnV1 & !EnV1_d)                delay_en <= 0;
		
		if (delay_en) delay_cnt <= delay_cnt + 1;
		else          delay_cnt <= 0;
		
		//if (!ipx_vact2_d & ipx_vact2_dd & EnV2) EnV1 <= 1;
		if (delay_cnt==MAX_DELAY & delay_en)   EnV1 <= 1;
		else if (!Frame_Ready & Frame_Ready_d) EnV1 <= 0;
		
		if (!Frame_Ready & Frame_Ready_d) delay_en2 <= 1;
		else if (EnV3 & !EnV3_d)          delay_en2 <= 0;		
		
		if (delay_en2) delay_cnt2 <= delay_cnt2 + 1;
		else           delay_cnt2 <= 0;		
		
		if (delay_cnt2==MAX_DELAY & delay_en2)   EnV3 <= 1;
		
		ipx_vact2_dd <= ipx_vact2_d;
		if (Frame_Ready & !Frame_Ready_d) Frame_Locked <= 1;
		if (Frame2_Ready & !Frame2_Ready_d) Frame2_Locked <= 1;
	end					
end

reg Frame_En=0;

always @ (negedge sclk0) begin
	if (rst) begin
		px_d1[15:0]   <= 0;
		px_d1_d[15:0] <= 0;
		px_d1a[15:0]  <= 0;
		px_d1a_d[15:0]<= 0;
		px_hact1      <= 0;
		px_hact1_d    <= 0;
		px_hact1_dd   <= 0;
		//px_vact1      <= 0;
		//px_vact1_d    <= 0;
		//px_vact1_dd   <= 0;
	end	
	else begin	 
		px_d1[15:0] <= ch_mux[3]?{ch0_vact_imt_cnt[9:0],2'b0}:ch_mux[4]?ipxd1_dl[11:0]:ipxd2_dl[11:0];
		px_d1_d[15:0] <= px_d1[15:0]; // delay sensor data by 1 tact to have time to catch HACT goes 0

		//px_d1a[15:0] <= fake_pix_cnt[15:0];
		px_d1a[15:0] <= ch0_hact_imt_cnt[15:0];
		px_d1a_d[15:0] <= px_d1a[15:0];

		px_hact1    <= ((ipx_hact2_dl & !ch_mux[4])|(ipx_hact1_dl & ch_mux[4])) & !Frame_Locked & Frame_En;
		px_hact1_d  <= px_hact1 & !Frame_Ready_d;	// delay by 1 tact
		px_hact1_dd <= px_hact1_d;		            // delay by 2 tacts

	end	
	
	px_vact1    <= ((ipx_vact2_dl & !ch_mux[4])|(ipx_vact1_dl & ch_mux[4])) & !Frame_Locked;
	px_vact1_d  <= px_vact1 & !Frame_Ready_d;	// delay by 1 tact
	px_vact1_dd <= px_vact1_d;			          // delay by 2 tacts	
	
end 

always @ (negedge sclk0) begin
	if (rst)                        Frame_En <= 0;
	else if ( px_vact1&!px_vact1_d) Frame_En <= 1;
	else if (!px_vact1& px_vact1_d) Frame_En <= 0;
end

//assign	sens_d1[15:0] = ch_mux[3]?{ch0_vact_imt_cnt[9:0],2'b0}:ch_mux[4]?ipxd2_dl[11:0]:ipxd1_dl[11:0];
assign	sens_d1[15:0] = px_d1_d[15:0];

reg Frame2_En=0;

always @ (negedge sclk0) begin
	if (rst) begin
		px_d2[15:0]   <= 0;
		px_d2_d[15:0] <= 0;
		px_d2a[15:0]  <= 0;
		px_d2a_d[15:0]<= 0;
		px_hact2      <= 0;
		px_hact2_d    <= 0;
		px_hact2_dd   <= 0;
//		px_vact2      <= 0;
//		px_vact2_d    <= 0;
//		px_vact2_dd   <= 0;
	end	
	else begin	 
		px_d2[15:0] <= ch_mux[3]?{ch1_vact_imt_cnt[9:0],2'b0}:ch_mux[4]?ipxd1_dl[11:0]:ipxd2_dl[11:0];
		px_d2_d[15:0] <= px_d2[15:0]; // delay sensor data by 1 tact to have time to catch HACT goes 0

		//px_d1a[15:0] <= fake_pix_cnt[15:0];
		px_d2a[15:0] <= ch1_hact_imt_cnt[15:0];
		px_d2a_d[15:0] <= px_d2a[15:0];

		px_hact2    <= ((ipx_hact3_dl & !ch_mux[4])|(ipx_hact2_dl & ch_mux[4])) & !Frame2_Locked & Frame2_En;
		px_hact2_d  <= px_hact2 & !Frame2_Ready_d;	// delay by 1 tact
		px_hact2_dd <= px_hact2_d;		              // delay by 2 tacts
	end	
	
	px_vact2    <= ((ipx_vact3_dl & !ch_mux[4])|(ipx_vact2_dl & ch_mux[4])) & !Frame2_Locked;
	px_vact2_d  <= px_vact2 & !Frame2_Ready_d;	// delay by 1 tact
	px_vact2_dd <= px_vact2_d;			            // delay by 2 tacts	
	
end 

always @ (negedge sclk0) begin
	if (rst)                        Frame2_En <= 0;
	else if ( px_vact2&!px_vact2_d) Frame2_En <= 1;
	else if (!px_vact2& px_vact2_d) Frame2_En <= 0;
end
//assign	sens_d1[15:0] = ch_mux[3]?{ch0_vact_imt_cnt[9:0],2'b0}:ch_mux[4]?ipxd2_dl[11:0]:ipxd1_dl[11:0];
assign	sens_d2[15:0] = px_d2_d[15:0]; 

////////////////////////////////////////////////////////////////////
always @ (negedge sclk0) begin
	if (rst) begin
		ch0_vact_imt <= 0;
		ch0_vact_imt_cnt <= 0;
		ch0_hact_imt <= 0;
		ch0_hact_imt_cnt <= 0;
		ch0_wpage_imt <= 0;
	end
	else begin
		if (px_vact1 & !px_vact1_d)                  ch0_vact_imt <= 1;
		else if (ch0_vact_imt_cnt==ch0_N_fl)         ch0_vact_imt <= 0;

		if (!ch0_vact_imt)                           ch0_vact_imt_cnt <= 0;
		else if (ch0_hact_imt_cnt==ch0_N_fp)         ch0_vact_imt_cnt <= ch0_vact_imt_cnt + 1;

		if ((px_hact1 & !px_hact1_d) & ch0_vact_imt) ch0_hact_imt <= 1;
		else if (ch0_hact_imt_cnt==ch0_N_fp)         ch0_hact_imt <= 0;

		if (ch0_hact_imt)                            ch0_hact_imt_cnt <= ch0_hact_imt_cnt + 1;
		else							                           ch0_hact_imt_cnt <= 0;

		if  (ch0_hact_imt_cnt[7:0]==255 | ch0_hact_imt_cnt==ch0_N_fp) ch0_wpage_imt <= 1;
		else                                                          ch0_wpage_imt <= 0;
	end				
end
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////
always @ (negedge sclk0) begin
	if (rst) begin
		ch1_vact_imt <= 0;
		ch1_vact_imt_cnt <= 0;
		ch1_hact_imt <= 0;
		ch1_hact_imt_cnt <= 0;
		ch1_wpage_imt <= 0;
	end
	else begin
		if (px_vact2 & !px_vact2_d)                  ch1_vact_imt <= 1;
		else if (ch1_vact_imt_cnt==ch0_N_fl)         ch1_vact_imt <= 0;

		if (!ch1_vact_imt)                           ch1_vact_imt_cnt <= 0;
		else if (ch1_hact_imt_cnt==ch0_N_fp)         ch1_vact_imt_cnt <= ch1_vact_imt_cnt + 1;

		if ((px_hact2 & !px_hact2_d) & ch1_vact_imt) ch1_hact_imt <= 1;
		else if (ch1_hact_imt_cnt==ch0_N_fp)         ch1_hact_imt <= 0;

		if (ch1_hact_imt)                            ch1_hact_imt_cnt <= ch1_hact_imt_cnt + 1;
		else							                     ch1_hact_imt_cnt <= 0;

		if  (ch1_hact_imt_cnt[7:0]==255 | ch1_hact_imt_cnt==ch0_N_fp) ch1_wpage_imt <= 1;
		else                                                          ch1_wpage_imt <= 0;
	end				
end
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
reg [31:0] stch2_delay_cnt=0;
reg stch2_cnt_en=0;

always @ (negedge sclk0) begin
	if (rst) begin
		stch2_first <= 0;
		ch2weo_cnt <= 0;
		stch2 <= 0;
		Line_Ready <= 0; 
		Line_Ready_d <= 0;
		ch2_hact_imt <= 0;
		ch2_hact_imt_cnt <= 0;
		ch2_hact_imt_d <= 0;
		ch2_vact_imt <= 0;
		ch2_vact_imt_d <= 0;
		ch2_vact_imt_cnt <= 0;	
	end
	else begin	
		
		//if (!ipx_vact2_d & ipx_vact2_dd & !Frame_Locked) stch2_first <= 1;
		if ((EnV1 & !EnV1_d & Frame_Ready)|(EnV1 & (Frame_Ready & !Frame_Ready_d))) stch2_first <= 1;
		else                stch2_first <= 0;
	
		if (ch2weo)            ch2weo_cnt <= ch2weo_cnt + 1;
		else if (ch2_hact_imt) ch2weo_cnt <= 0;

		if ((ch2weo_cnt[6:0]==127 & !Line_Ready & Frame_Ready)|(Line_Ready_d & !Line_Ready & Frame_Ready))
			stch2 <= 1;	
		else
			stch2 <= 0;					

		if (ch2weo & ch2weo_cnt==ch2_N_fp[15:1]+1) Line_Ready <= 1; // divided by 2
		else if (!ch2_hact_imt & ch2a[7:0]==255)   Line_Ready <= 0; 

		Line_Ready_d <= Line_Ready;

		if (ch2_hact_imt_cnt==ch0_N_fp)            ch2_hact_imt <= 0;
		else if (Line_Ready & !Line_Ready_d)       ch2_hact_imt <= 1;	

		if   (ch2_hact_imt)     ch2_hact_imt_cnt <= ch2_hact_imt_cnt + 1;
		else                    ch2_hact_imt_cnt <= 0;

		ch2_hact_imt_d <= ch2_hact_imt;
	
		if	(ch2_vact_imt_cnt==ch0_N_fl) ch2_vact_imt <= 0;		
		else if (ch2weo & Frame_Ready)   ch2_vact_imt <= 1;	
		
		ch2_vact_imt_d <= ch2_vact_imt;
	
		if      (!ch2_vact_imt)              ch2_vact_imt_cnt <= 0;		
		else if (ch2_hact_imt_cnt==ch0_N_fp) ch2_vact_imt_cnt <= ch2_vact_imt_cnt + 1;		
	end	
	
	if (ch2_hact_imt | (Line_Ready_d & Line_Ready)) ch2a <= ch2a + 1; // don't reset (addr on the other side is not reseted)
		
end

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
reg [31:0] stch4_delay_cnt=0;
reg stch4_cnt_en=0;

always @ (negedge sclk0) begin
	if (rst) begin
		stch4_first <= 0;
		ch4weo_cnt <= 0;
		stch4 <= 0;
		Line2_Ready <= 0; 
		Line2_Ready_d <= 0;
		ch4_hact_imt <= 0;
		ch4_hact_imt_cnt <= 0;
		ch4_hact_imt_d <= 0;
		ch4_vact_imt <= 0;
		ch4_vact_imt_d <= 0;
		ch4_vact_imt_cnt <= 0;	
	end
	else begin	
		
		//if (!ipx_vact2_d & ipx_vact2_dd & !Frame_Locked) stch2_first <= 1;
		if ((EnV3 & !EnV3_d & Frame2_Ready)|(EnV3 & (Frame2_Ready & !Frame2_Ready_d))) stch4_first <= 1;
		else                                                                           stch4_first <= 0;
	
		if (ch4weo)            ch4weo_cnt <= ch4weo_cnt + 1;
		else if (ch4_hact_imt) ch4weo_cnt <= 0;

		if ((ch4weo_cnt[6:0]==127 & !Line2_Ready & Frame2_Ready)|(Line2_Ready_d & !Line2_Ready & Frame2_Ready))
			stch4 <= 1;	
		else
			stch4 <= 0;					

		if (ch4weo & ch4weo_cnt==ch2_N_fp[15:1]+1) Line2_Ready <= 1; // divided by 2
		else if (!ch4_hact_imt & ch4a[7:0]==255)   Line2_Ready <= 0; 

		Line2_Ready_d <= Line2_Ready;

		if (ch4_hact_imt_cnt==ch0_N_fp)        ch4_hact_imt <= 0;
		else if (Line2_Ready & !Line2_Ready_d) ch4_hact_imt <= 1;	

		if   (ch4_hact_imt) ch4_hact_imt_cnt <= ch4_hact_imt_cnt + 1;
		else                ch4_hact_imt_cnt <= 0;

		ch4_hact_imt_d <= ch4_hact_imt;
	
		if	(ch4_vact_imt_cnt==ch0_N_fl) ch4_vact_imt <= 0;		
		else if (ch4weo & Frame2_Ready)   ch4_vact_imt <= 1;	
		
		ch4_vact_imt_d <= ch4_vact_imt;
	
		if      (!ch4_vact_imt)              ch4_vact_imt_cnt <= 0;		
		else if (ch4_hact_imt_cnt==ch0_N_fp) ch4_vact_imt_cnt <= ch4_vact_imt_cnt + 1;		
	end	
	
	if (ch4_hact_imt | (Line2_Ready_d & Line2_Ready)) ch4a <= ch4a + 1; // don't reset (addr on the other side is not reseted)
		
end

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
always @ (negedge sclk0) 
	begin
		if (da_set_fake) 
			begin
				pre_N_l <= idi[31:16];
				pre_N_p <= idi[15:0];
			end

// changes are applied when nothing works
		if (rst)
			begin
				ch0_N_fl <= pre_N_l;
				ch0_N_fp <= pre_N_p;
				ch2_N_fl <= pre_N_l;
				ch2_N_fp <= pre_N_p;
			end		

	if (da_set_delay) MAX_DELAY <= idi[31:0];	
end

always @ (negedge sclk0) begin
	if (rst) begin
		Frame_Ready <= 0;
		Frame_Ready_d <= 0;
	end
	else begin
		if (!px_vact1 & px_vact1_d & Frame_En)    Frame_Ready <= 1;
		else	if (!ch2_vact_imt & ch2_vact_imt_d) Frame_Ready <= 0;
		Frame_Ready_d <= Frame_Ready;
	end
end

always @ (negedge sclk0) begin
	if (rst) begin
		Frame2_Ready <= 0;
		Frame2_Ready_d <= 0;
	end
	else begin
		if (!px_vact2 & px_vact2_d & Frame2_En)   Frame2_Ready <= 1;
		else	if (!ch4_vact_imt & ch4_vact_imt_d) Frame2_Ready <= 0;
		Frame2_Ready_d <= Frame2_Ready;
	end
end
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// calculate the delay between frames
reg ch1_ch2_delay_en=0;
reg [31:0] ch1_ch2_delay_cnt=0;
reg [31:0] ch1_ch2_delay_fixed=0;

always @ (negedge sclk0) begin
if (rst)
	begin
		ch1_ch2_delay_en    <= 0;
		ch1_ch2_delay_cnt   <= 0;
		ch1_ch2_delay_fixed <= 0;
	end
else 
	begin
		if ((ipx_vact2_d & !ipx_vact2_dd & EnV2) & (px_vact1 & !px_vact1_d))  ch1_ch2_delay_en <= 0;
		else if ((ipx_vact2_d & !ipx_vact2_dd & EnV2)|(px_vact1 & !px_vact1_d)) ch1_ch2_delay_en <= !ch1_ch2_delay_en;
		
		if (px_vact1 & !px_vact1_d & !ch1_ch2_delay_en)
			ch1_ch2_delay_cnt <= 32'h80000000;
		else if (ch1_ch2_delay_en) 
			ch1_ch2_delay_cnt <= ch1_ch2_delay_cnt + 1;
		
		ch1_ch2_delay_fixed <= ch1_ch2_delay_cnt;
	end			
end

// for test
always @ (negedge sclk0) begin
	if (!sdras_p | sdcas_p | sdwe_p) begin
		active_reg[15:0] <= {1'b0,sdba_p[1:0],sda_p[12:0]};
		active_reg[31:16] <= active_reg[31:16] + 1;
	end	
	
	if (!sdras_p | !sdcas_p) begin
		status_reg[14:0]<= {sdba_p[1:0],sda_p[12:0]};
		status_reg[15] <=	sdwe_p;
		status_reg[16] <= sdcas_p;
		status_reg[17] <= sdras_p;
	end
end

// cite from sdseq
//  FD_1 i_precmd_0	(.D(mancmd[15] && ~(prewrite | preprech)),         .C(clk0),.Q(precmd[0]));	//WE
//  FD_1 i_precmd_1	(.D(mancmd[16] && ~(prerefr | preread | prewrite)),.C(clk0),.Q(precmd[1]));	//CAS
//  FD_1 i_precmd_2	(.D(mancmd[17] && ~(prerefr | preact | preprech)), .C(clk0),.Q(precmd[2]));	//RAS
 
////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////

reg [3:0] charge=0;
reg [3:0] charge_en=0;

reg ch1_hact=0, ch1_hact_d=0;
reg ch1_vact=0, ch1_vact_d=0;

reg ch2_hact=0, ch2_hact_d=0;
reg ch2_vact=0, ch2_vact_d=0;

reg [15:0] ch1_hact_probe=0;
reg [15:0] ch1_vact_probe=0;

reg [15:0] ch2_hact_probe=0;
reg [15:0] ch2_vact_probe=0;

reg ipx_bpf_direct=0;
reg ipx_vact_direct=0;
reg ipx_hact_direct=0;
reg [11:0] ipxd_direct=0;

reg ipx_bpf_direct_d=0;
reg ipx_vact_direct_d=0;
reg ipx_hact_direct_d=0;
reg [11:0] ipxd_direct_d=0;

reg    ipx_vact1_dl1_pos,  ipx_vact2_dl1_pos,  ipx_vact3_dl1_pos;
reg    ipx_hact1_dl1_pos,  ipx_hact2_dl1_pos,  ipx_hact3_dl1_pos;

reg    ipx_vact1_dl1_neg,  ipx_vact2_dl1_neg,  ipx_vact3_dl1_neg;
reg    ipx_hact1_dl1_neg,  ipx_hact2_dl1_neg,  ipx_hact3_dl1_neg;

reg    ipx_vact1_dl2_neg,  ipx_vact2_dl2_neg,  ipx_vact3_dl2_neg;
reg    ipx_hact1_dl2_neg,  ipx_hact2_dl2_neg,  ipx_hact3_dl2_neg;

reg [11:0] ipxd1_dl1_neg,      ipxd2_dl1_neg,      ipxd3_dl1_neg;
reg [11:0] ipxd1_dl2_neg,      ipxd2_dl2_neg,      ipxd3_dl2_neg;
reg [11:0] ipxd1_dl3_neg,      ipxd2_dl3_neg,      ipxd3_dl3_neg;

always @ (negedge sclk0) 
begin
	ipx_vact1_dl1_neg <= ipx_vact1; ipx_hact1_dl1_neg <= ipx_hact1;
	ipx_vact2_dl1_neg <= ipx_vact2; ipx_hact2_dl1_neg <= ipx_hact2;
	ipx_vact3_dl1_neg <= ipx_vact3; ipx_hact3_dl1_neg <= ipx_hact3;		
end

always @ (posedge sclk0) 
begin
	ipx_vact1_dl1_pos <= ipx_vact1; ipx_hact1_dl1_pos <= ipx_hact1;
	ipx_vact2_dl1_pos <= ipx_vact2; ipx_hact2_dl1_pos <= ipx_hact2;
	ipx_vact3_dl1_pos <= ipx_vact3; ipx_hact3_dl1_pos <= ipx_hact3;	
end

always @ (negedge sclk0)
begin
	ipxd1_dl1_neg <=     ipxd1;
	ipxd2_dl1_neg <=     ipxd2;
	ipxd3_dl1_neg <=     ipxd3;
	ipxd1_dl2_neg <=     ipxd1_dl1_neg;
	ipxd2_dl2_neg <=     ipxd2_dl1_neg;
	ipxd3_dl2_neg <=     ipxd3_dl1_neg;
	ipxd1_dl3_neg <=     ipxd1_dl2_neg; 
	ipxd2_dl3_neg <=     ipxd2_dl2_neg;
	ipxd3_dl3_neg <=     ipxd3_dl2_neg;	

	ipx_vact1_dl2_neg <= sphase[0]?ipx_vact1_dl1_neg:ipx_vact1_dl1_pos; 
	ipx_hact1_dl2_neg <= sphase[0]?ipx_hact1_dl1_neg:ipx_hact1_dl1_pos;

	ipx_vact2_dl2_neg <= sphase[1]?ipx_vact2_dl1_neg:ipx_vact2_dl1_pos;
	ipx_hact2_dl2_neg <= sphase[1]?ipx_hact2_dl1_neg:ipx_hact2_dl1_pos;

	ipx_vact3_dl2_neg <= sphase[2]?ipx_vact3_dl1_neg:ipx_vact3_dl1_pos;
	ipx_hact3_dl2_neg <= sphase[2]?ipx_hact3_dl1_neg:ipx_hact3_dl1_pos;
end

assign ipx_vact1_dl=ipx_vact1_dl2_neg;
assign ipx_hact1_dl=ipx_hact1_dl2_neg;

assign ipx_vact2_dl=ipx_vact2_dl2_neg;
assign ipx_hact2_dl=ipx_hact2_dl2_neg;

assign ipx_vact3_dl=ipx_vact3_dl2_neg;
assign ipx_hact3_dl=ipx_hact3_dl2_neg;

assign     ipxd1_dl= sphase[4]?ipxd1_dl1_neg:(sphase[3]?ipxd1_dl3_neg:ipxd1_dl2_neg);
assign     ipxd2_dl= sphase[6]?ipxd2_dl1_neg:(sphase[5]?ipxd2_dl3_neg:ipxd2_dl2_neg);
assign     ipxd3_dl= sphase[8]?ipxd3_dl1_neg:(sphase[7]?ipxd3_dl3_neg:ipxd3_dl2_neg);

reg [5:0] ch_mux_d=0;
reg [5:0] ch_mux_dd=0;
reg ch_mux_enable=1;

always @ (negedge sclk0) 
begin
	if ((ch_mux[5:0]!=ch_mux_d[5:0]) & !ipx_vact_direct_d)
			ch_mux_d[5:0] <= ch_mux[5:0];
			
	if       (!ch_mux_dd[0] & ch_mux_d[0] & !ipx_vact1_dl)
		  ch_mux_dd[5:0] <= ch_mux_d[5:0];
	else if  (!ch_mux_dd[1] & ch_mux_d[1] & !ipx_vact2_dl)
	    ch_mux_dd[5:0] <= ch_mux_d[5:0];
	else if  (!ch_mux_dd[2] & ch_mux_d[2] & !ipx_vact3_dl)
	    ch_mux_dd[5:0] <= ch_mux_d[5:0];			
//			ch_mux <= pre2_ch_mux;

	if ((ch_mux[5:0]!=ch_mux_d[5:0]) & !ipx_vact_direct_d) ch_mux_enable <= 0;
	else if (ch_mux_dd[2:0]==ch_mux_d[2:0])                ch_mux_enable <= 1;
end

always @ (negedge sclk0) 
begin
	if (ch_mux_enable) begin
		if      (ch_mux_d[0]) ipx_bpf_direct <= ipx_bpf1_dl;
		else if (ch_mux_d[1]) ipx_bpf_direct <= ipx_bpf2_dl;
		else if (ch_mux_d[2]) ipx_bpf_direct <= ipx_bpf3_dl;
		else                  ipx_bpf_direct <= 0;
		
		if      (ch_mux_d[0]) ipx_vact_direct <= ipx_vact1_dl;
		else if (ch_mux_d[1]) ipx_vact_direct <= ipx_vact2_dl;
		else if (ch_mux_d[2]) ipx_vact_direct <= ipx_vact3_dl;
		else                  ipx_vact_direct <= 0;
		
		if      (ch_mux_d[0]) ipx_hact_direct <= ipx_hact1_dl;
		else if (ch_mux_d[1]) ipx_hact_direct <= ipx_hact2_dl;
		else if (ch_mux_d[2]) ipx_hact_direct <= ipx_hact3_dl;		
		else                  ipx_hact_direct <= 0;
	
		if      (ch_mux_d[0]) ipxd_direct <= ipxd1_dl;
		else if (ch_mux_d[1]) ipxd_direct <= ipxd2_dl;
		else if (ch_mux_d[2]) ipxd_direct <= ipxd3_dl;
		else                  ipxd_direct <= 0;
	end
	else
	begin
		ipx_bpf_direct <= 0;
		ipx_vact_direct <= 0;
		ipx_hact_direct <= 0;
		ipxd_direct <= 0;	
	end
end

always @ (negedge sclk0) 
begin			
	ibpf <= 0;
		
	ipx_vact_direct_d <= ipx_vact_direct;
	ipx_hact_direct_d <= ipx_hact_direct;
	ipxd_direct_d <= ipxd_direct;
	
	if (!ch_mux_d[5])
		begin
			ivact <= ipx_vact_direct_d;
			ihact <= ipx_hact_direct_d;
			pxdr[11:0] <= ipxd_direct_d;
		end
	else
	if (EnV3)
			begin
				ivact      <= ch4_vact_imt;
				ihact      <= ch4_hact_imt_d;
				pxdr[11:0] <= ch4do[11:0];								
			end			
	else if (EnV1)
			begin
				ivact      <= ch2_vact_imt;
				ihact      <= ch2_hact_imt_d;
				pxdr[11:0] <= ch2do[11:0];								
			end		
	else if (EnV2)
			begin
				ivact      <= ipx_vact2_d;
				ihact      <= ipx_hact2_d;
				pxdr[11:0] <= ipxd2_d[11:0];								
			end		
end

reg [15:0] vact_cnt=0, vact_reg=0;		
reg [15:0] hact_cnt=0, hact_reg=0;

reg [3:0] hact_errors=0;
//////////////////////////////////////////////////////////////////////////////////////////////////////
//for tests//
always @ (negedge sclk0) 
begin			
	// count number of lines in a frame
	if (ipx_vact_direct_d & (ipx_hact_direct_d & !ihact)) vact_cnt <= vact_cnt + 1; // catch 'ihact' fronts
	else if	(!ipx_vact_direct_d)                       vact_cnt <= 0;
	
	if (!ipx_vact_direct_d & ivact) vact_reg <= vact_cnt; // latch counter data
	
	// count number of pixels in a line
	if (ipx_hact_direct_d) hact_cnt <= hact_cnt + 1;
	else	               hact_cnt <= 0;
	
	if (!ipx_hact_direct_d & ihact) hact_reg <= hact_cnt;	// latch counter data
	
	i2c_do_r <= i2c_do;
		
end		
	
reg [5:0] ch_mux_reg=0;	
reg [7:0] snap_status=0;
reg [15:0] ch2weo_cnt_fixed=0;
	
always @ (negedge sclk0)
begin
	ch_mux_reg <= ch_mux;
	if (rst)              snap_status[0] <= 0;
	else if (stch2_first) snap_status[0] <= 1;

	if (rst)                               snap_status[1] <= 0;
	else if (!Frame_Ready_d & Frame_Ready) snap_status[1] <= 1;

	if (rst)                               snap_status[2] <= 0;
	else if (Frame_Ready_d & !Frame_Ready) snap_status[2] <= 1;
	
	if (rst)                              snap_status[3] <= 0;
	else if (!ipx_vact2_d & ipx_vact2_dd) snap_status[3] <= 1;
  
	if (ch2weo_cnt[15:0]==33) ch2weo_cnt_fixed[15:0] <= ch2weo_cnt[15:0];
	
end	

reg [15:0] sclk0_cnt=0;
always @ (negedge sclk0)
begin
	sclk0_cnt <= sclk0_cnt + 1;
end

reg [15:0] pclk_cnt=0;
always @ (negedge pclk)
begin
	pclk_cnt <= pclk_cnt + 1;
end

reg [3:0] dcm1_locked_rise_cnt=0;
reg [3:0] dcm1_locked_fall_cnt=0;

always @ (negedge pclk)
begin
	if (!dcm1_locked_d &  dcm1_locked_dd) dcm1_locked_fall_cnt <= dcm1_locked_fall_cnt + 1;
	if ( dcm1_locked_d & !dcm1_locked_dd) dcm1_locked_rise_cnt <= dcm1_locked_rise_cnt + 1;
end
	
	assign i2c_do[31:0] = //32'habcd6789; 
	(ia[6])?
		(
			(ia[5])?
				(
					(ia[4])?
						//{14'b0,xfpgatdo,xfpgastat,4'b0,io_pins[11:0]}                     // 0x70..0x7f 
						ch1_ch2_delay_fixed[31:0]
						:   
            ia[3]?
							{regfil_do[15:8],regfil_do[7:0],regfil_do[15:8],regfil_do[7:0]} // 0x68..0x6f
							:
							//{dcm1_locked,7'b0,sclk0_cnt[7:0],dclk2_locked,7'b0,dclk2_cnt[7:0]}
							ia[1]?
								ia[0]?{0}:{0}
								:
								ia[0]?{0}:{0}	            // 0x60..0x67
        ):(
					ia[2]?
						(
							ia[1]? ia[0]?{0}:{0} :
							ia[0]?
								//{16'b0,ch2do}//pio_sec[31:0] // 8'b010x x1x1
								//{16'b0,ch0_test_reg}
								{0}
								:		
                //{12'h0,pio_usec[19:0]} // 8'b010xx1x0
								{0}//active_reg// 8'b010xx1x0
						):
					      ia[0]? 
									status_reg[31:0]:	// 8'b010xx0x1
									//{1'b0,nu_cnt[15:8],nu_cnt[7:0]}//hist_do[31:0] // 8'b010xx0x0 
									{0}//hist_do[31:0] // 8'b010xx0x0 
						)
			):(
				ia[5]?
						(
							ia[4]?
								(0):		// 3x // 8'b0011xxxx
								//(ia[2]?bdo_d[4]:(ia[1]?(ia[0]?bdo_d[3]:bdo_d[2]):(ia[0]?bdo_d[1]:bdo_d[0])))
								//:
								(0)//({dsdo[31:0]})	// 2x // 8'b0010xxxx
						):(
							ia[4]? 
								0//rd_regs[31:0]		
							:
                (
									ia[1]?
										//ia[0]?{16'b0,ch2weo_cnt_fixed[15:0]}:{27'b0,chen_out[4:0]}
										ia[0]?{pclk_cnt[15:0],sclk0_cnt[15:0]}:{0}
										:
										ia[0]?{dcm1_locked_fall_cnt[3:0],20'b0,dcm1_status[2],dcm1_status[1],dcm1_status[0],dcm1_locked,dcm1_locked_rise_cnt[3:0]}:MODELREV
								)
							)
						); //00/02/04/06/08/0a/0c/0e	

////////////////////////////////////////////////////////////////////
			
endmodule
