/*
** -----------------------------------------------------------------------------**
** sensorpads353.v
**
** I/O pads related circuitry for the sensor board connector
**
** Copyright (C) 2002-2006 Elphel, Inc.
**
** -----------------------------------------------------------------------------**
**  This file is part of X353
**  X353 is free software - hardware description language (HDL) code.
** 
**  This program is free software: you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation, either version 3 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program.  If not, see <http://www.gnu.org/licenses/>.
** -----------------------------------------------------------------------------**
**
*/



module	sensorpads (clk,
                     clk2, // twice pixel clock
                     vact, hact, bpf, pxd, mrst, arst, aro,
                     dclkmode, // 0 - DCLK is clock to sensor, 1 - combined sync from sensor (like 10347)
                     pxd14,       // use {vact,hact} as 2 LSB in 14-bit data
                     latehact,   // register hact N/4 Tpclk later
                     dclk, ivact, vacts, ihact, ibpf, sens_clk, ipxd, imrst, iarst, iaro, idclk,
							dlyhact,skipline,fillfactory, ystart, yclock,
							cnvctl,cnven,
                     senspgm,    // SENSPGM I/O pin
                     senspgmin,  // state of the SENSPGM I/O pin (read)
                     xpgmen,     // enable programming mode for an external FPGA
                     xfpgaprog,  // PROG_B to be sent to an external FPGA
                     xfpgadone,  // state of the MRST pin ("DONE" pin on an external FPGA)
                     xfpgatck,   // TCK to be sent to an external FPGA
                     xfpgatms,   // TMS to be sent to an external FPGA
                     xfpgatdi,   // TDI to be sent to an external FPGA
                     xfpgatdo,    // TDO read from an external FPGA
                     pherr       //[1:0] phase error (sync to posedge pclk) {too_early, too_late}
                     );
	input				clk,clk2;
	inout				vact;
	inout				hact; //output in fillfactory mode
	inout				bpf;  // output in fillfactory mode
	inout	  [11:0]	pxd; //actually only 2 LSBs are inouts
//	output			mrst;
	inout          mrst;
	output			arst;
	output			aro;
   input          dclkmode;
   input          pxd14;     // use {vact,hact} as 2 LSB in 14-bit data
   input   [1:0]  latehact;  // register hact, vact, bpf N/4 Tpclk later than data
	inout	         dclk;
	output			ivact;
	output			vacts;	// 1 cycle long
	output			ihact;
	output			ibpf;
   output         sens_clk; // ibpf before FD
	output  [15:0]	ipxd;
	input				imrst;
	input				iarst;
	input				iaro;
	input				idclk;
	input				dlyhact;	// delay hact pulse by 1
   input          skipline; // skip first scan line from tghe sensor
	input				fillfactory; // fillfactory sensor mode (will also delay hact by 7)
	input				ystart;		 // start of frame (fillfactory sensor mode) - 3 cycles long;
	input				yclock;      // start of line  (fillfactory sensor mode) - 1 cycle long;
	input    [1:0] cnvctl;      // 2-bits for converter control (use the same pins as lower bits in 12-bit sensors)
	input          cnven;       // 0 - use 12-bit sensor data, 1 - use converter output pins

   inout          senspgm;    // SENSPGM I/O pin
   output         senspgmin;  // state of the SENSPGM I/O pin (read)
   input          xpgmen;     // enable programming mode for external FPGA
   input          xfpgaprog;  // PROG_B to be sent to an external FPGA
   output         xfpgadone;  // state of the MRST pin ("DONE" pin on external FPGA)
   input          xfpgatck;   // TCK to be sent to external FPGA
   input          xfpgatms;   // TMS to be sent to external FPGA
   input          xfpgatdi;   // TDI to be sent to external FPGA
   output         xfpgatdo;   // TDO read from external FPGA
   output   [1:0] pherr;

   reg   [1:0] fs_d;
//  reg             vacts347;
  reg             hact347;
  wire				ihact;
  reg	   			ivact, ibpf;
  reg      [15:0]	ipxd_pre;
  reg      [15:0]	ipxd;
  wire     [15:0]	ipxde; //early (by Tpclk) ipxd
  reg      [15:0]	ipxded; //pclk2x later than ipxde
  wire     [ 1:0] ipxdel; // registered 1/4 Tpclk later than @posedge pclk
  reg      [ 1:0] ipxdeld;// ipxdel[1:0] delayed by Tpclk2x
  wire            iibpf, iibpfl; // early (by Tpclk) ibpf and registered 1/4 later
  reg             iibpfd, iibpfld; // iibpf, iibpfl delayed by Tpclk2x

  reg      [ 1:0]	ipxdes02;
  reg      [ 1:0]	ipxdes13;
  reg             iibpfs02, iibpfs13;


  wire      [1:0] cnvctl;
  wire            cnven;
//  wire      [3:0] iipxd;

  wire				mrst, arst,aro,dclk;
  wire				vactd,vacts;
  reg             ihact0; // to be delayed
  wire            ihact1; // to be delayed
  wire            fs_composite0; //  extract vact/hact from it (10347 board)
  reg             fs_composite;  //  extra stage to match delays
   wire           ihact_unmasked;
   reg            line_en0, line_en;
   always @ (posedge clk) begin
     if (vacts) line_en0 <= ~skipline;
     else if (ihact_unmasked) line_en0 <= 1'b1;
     if (!ihact_unmasked) line_en <= line_en0;
   end
   SRL16 i_ihact1 (.Q(ihact1), .A0(dlyhact && fillfactory), .A1(fillfactory), .A2(fillfactory), .A3(1'b0), .CLK(clk), .D(ihact0));	//
   assign ihact_unmasked=(dlyhact || fillfactory )?ihact1:ihact0;
   assign ihact=line_en & ihact_unmasked;

   FD	  i_vactd (.C(clk), .D(ivact), .Q(vactd));
wire pxd6_qr2; // pxd[6] (pxd[2]) registered at negedge pclk2
wire pclk_clone; // output of the register clocked at posedge clk2 that reproduces global pclk
wire pclk_clone_prev; // registered by posedge clk2 pclk_clone
wire pclk_clone_err; // ==1 when pclk_clone is out of phase with pclk
//wire pclk_clone_neg; // pclk_clone delayed to negedge clk2


wire epxd; // enable pxd registers
//reg  epxdn; // delayed by Tpclk/4

reg  [2:0]  pxd6_nel; // pxd6 at nominal, early and late clocks

reg  [1:0]  pxd6_qr2_prev; // pxd_qr2 delayed by 1,2 of clk2

reg  [1:0]  pxd6_qr2_pn;  // pxd_qr2_prev valid @posedge clk (shortest 3/4 of a period ahead)
reg  [1:0]  pherr;

assign epxd=pclk_clone_prev;
  FD	  i_pclk_clone      (.C(clk2), .D(~(pclk_clone | (pclk_clone_prev & pclk_clone_err))),.Q(pclk_clone));
  FD	  i_pclk_clone_prev (.C(clk2), .D(pclk_clone),                                        .Q(pclk_clone_prev));
  FD	  i_pclk_clone_err  (.C(clk),  .D(pclk_clone),                                        .Q(pclk_clone_err));
//  FD_1  i_pclk_clone_neg  (.C(clk2), .D(pclk_clone),                                        .Q(pclk_clone_neg));
  FD_1  i_epxdl           (.C(clk2), .D(epxd),                                              .Q(epxdl));
  always @ (posedge clk) begin
    pxd6_nel[2:0] <= {ipxd_pre[6],pxd6_qr2_pn[1:0]};
    pherr[1:0] <= {pxd6_nel[2]^pxd6_nel[1],pxd6_nel[2]^pxd6_nel[0]};
  end

  always @ (negedge clk2) begin
    pxd6_qr2_prev[1:0] <= {pxd6_qr2_prev[0],pxd6_qr2};
//    if (pclk_clone_prev) pxd6_qr2_pn[1:0] <= pxd6_qr2_prev[1:0];
    if (epxdl) pxd6_qr2_pn[1:0] <= pxd6_qr2_prev[1:0];
  end
  FD	  i_vacts (.C(clk), .D(dclkmode? (!fs_composite && fs_d[0] && !fs_d[1]): (ivact && !vactd)), .Q(vacts));
  ipadqffc	i_ipxde2	(.c2(clk2),.e(1'b1), .q(),         .qr(ipxde[ 2]), .qr2(),
                     .d(xpgmen?xfpgatdi:cnvctl[0]),.dq(pxd[0]),.t(~(cnven | xpgmen)));
  ipadqffc	i_ipxde3	(.c2(clk2),.e(1'b1), .q(xfpgatdo), .qr(ipxde[ 3]), .qr2(),
                     .d(cnvctl[1]),.dq(pxd[1]),.t(~cnven | xpgmen));
  ipadqffc i_ipxde6	(.c2(clk2),.e(1'b1), .q(),         .qr(ipxde[ 6]), .qr2(pxd6_qr2),
                     .d(1'b0),     .dq(pxd[2]),.t(1'b1));  //
	ipadqffc	i_ipxde7	(.c2(clk2),.e(1'b1), .q(), .qr(ipxde[ 7]), .qr2(), .d(1'b0),  .dq(pxd[ 3]),.t(1'b1));
	ipadqffc	i_ipxde8	(.c2(clk2),.e(1'b1), .q(), .qr(ipxde[ 8]), .qr2(), .d(1'b0),  .dq(pxd[ 4]),.t(1'b1));
	ipadqffc	i_ipxde9	(.c2(clk2),.e(1'b1), .q(), .qr(ipxde[ 9]), .qr2(), .d(1'b0),  .dq(pxd[ 5]),.t(1'b1));
	ipadqffc	i_ipxde10(.c2(clk2),.e(1'b1), .q(), .qr(ipxde[10]), .qr2(), .d(1'b0),  .dq(pxd[ 6]),.t(1'b1));
	ipadqffc	i_ipxde11(.c2(clk2),.e(1'b1), .q(), .qr(ipxde[11]), .qr2(), .d(1'b0),  .dq(pxd[ 7]),.t(1'b1));
	ipadqffc	i_ipxde12(.c2(clk2),.e(1'b1), .q(), .qr(ipxde[12]), .qr2(), .d(1'b0),  .dq(pxd[ 8]),.t(1'b1));
	ipadqffc	i_ipxde13(.c2(clk2),.e(1'b1), .q(), .qr(ipxde[13]), .qr2(), .d(1'b0),  .dq(pxd[ 9]),.t(1'b1));
	ipadqffc	i_ipxde14(.c2(clk2),.e(1'b1), .q(), .qr(ipxde[14]), .qr2(), .d(1'b0),  .dq(pxd[10]),.t(1'b1));
	ipadqffc	i_ipxde15(.c2(clk2),.e(1'b1), .q(), .qr(ipxde[15]), .qr2(), .d(1'b0),  .dq(pxd[11]),.t(1'b1));

sensio1 i_ipxde0 (.c2(clk2),       // pclk2x
                   .e (1'b1),      // inverted pclk, enable for FFs clocked at pclk2x to make them as if clocked at pclk
                   .d(1'b0),        // data to be sent out (in fillfactory mode)
                   .t(1'b1),        // tristate (all but fillfactory)
                   .q(ipxde[0]),    // pad data registered @ posedge c2 (enabled by e)
                   .dq(hact),       // pad
                   .qp(),           // non-registered data read from the pad
                   .q2(ipxdel[0])); // pad data registered at negedge c2
    
sensio1 i_ipxde1  (.c2(clk2),       // pclk2x
                   .e (1'b1),      // inverted pclk, enable for FFs clocked at pclk2x to make them as if clocked at pclk
                   .d(ystart),      // data to be sent out (in fillfactory mode)
                   .t(!fillfactory),// tristate (all but fillfactory)
                   .q(ipxde[1]),    // pad data registered @ posedge c2 (enabled by e)
                   .dq(vact),       // pad
                   .qp(),           // non-registered data read from the pad
                   .q2(ipxdel[1])); // pad data registered at negedge c2

sensio1 i_iibpf   (.c2(clk2),       // pclk2x
                   .e (1'b1),       // inverted pclk, enable for FFs clocked at pclk2x to make them as if clocked at pclk
                   .d(yclock),      // data to be sent out (in fillfactory mode)
                   .t(!fillfactory),// tristate (all but fillfactory)
                   .q(iibpf),       // pad data registered @ posedge c2 (enabled by e)
                   .dq(bpf),        // pad
                   .qp(sens_clk),   // non-registered data read from the pad
                   .q2(iibpfl));    // pad data registered at negedge c2
                   
//  reg      [ 1:0] ipxde_r; // ipxde[1:0] delayed by Tpclk2x
//  reg      [ 1:0] ipxdel_r;// ipxdel[1:0] delayed by Tpclk2x
//  reg             iibpf_r, iibpfl_r; // iibpf, iibpfl delayed by Tpclk2x
//  reg      [ 1:0]	ipxdes02;
//  reg      [ 1:0]	ipxdes13;
//  reg             iibpfs02, iibpfs13;
                   
                   
   always @ (posedge clk2) begin
     ipxded[15:0] <= ipxde[15:0];
     iibpfd       <= iibpf;
     if (epxd) ipxdes02[1:0] <= latehact[1]?ipxde[1:0]:ipxded[1:0];
     if (epxd) iibpfs02      <= latehact[1]?iibpf:     iibpfd;
   end
   always @ (negedge clk2) begin
     ipxdeld[1:0] <= ipxdel[1:0];
     iibpfld      <= iibpfl;
     if (epxdl) ipxdes13[1:0] <= latehact[1]?ipxdel[1:0]:ipxdeld[1:0];
     if (epxdl) iibpfs13      <= latehact[1]?iibpfl:     iibpfld;
   end
   always @ (posedge clk) begin
     ipxd_pre[ 5:0] <= cnven?6'b0:{ipxded[3:2],pxd14?ipxded[1:0]:2'h0,2'h0};
     ipxd_pre[15:6] <= ipxded[15:6];
     ipxd[15:0] <= ipxd_pre[15:0];
     ihact0 <= dclkmode? hact347 :  (latehact[0]?ipxdes13[0]:ipxdes02[0]);
     ivact  <= latehact[0]? ipxdes13[1]: ipxdes02[1];
     ibpf   <= latehact[0]? iibpfs02:    iibpfs13;
   end
   IOBUF	 i_mrst	(.I(imrst), .IO(mrst), .T(xpgmen), .O(xfpgadone));
   OBUF	 i_arst	(.I(xpgmen? xfpgatms : iarst), .O(arst));
   OBUF	 i_aro	   (.I(xpgmen? xfpgatck : iaro),  .O(aro ));
//NET  "i_sensorpads/fs_composite" TIG; # did not work
//   IOBUF	 i_dclk	(.I(idclk), .IO(dclk), .T(dclkmode), .O(fs_composite));
   IOBUF	 i_dclk	(.I(idclk), .IO(dclk), .T(dclkmode), .O(fs_composite0));
   PULLUP i_PU_mrst   (.O(mrst));
   PULLUP i_PU_senspgm(.O(senspgm));
// to reduce noise on pulled-up senspgm it is possible to force it high when xpgmen goes from high to low
   reg [1:0] xpgmen_d;
   wire      force_senspgm;
   always @ (posedge clk) xpgmen_d[1:0] <= {xpgmen_d[0],xpgmen};
// will latch the state of the senspgm pin after xpgmen is over - that will protect from driving senspgm pin
// high if it is grounded on the sensor board
   FDCE i_force_senspgm (.Q(force_senspgm), .D(senspgmin), .C(clk), .CE(xpgmen_d[1:0]==2'b10), .CLR(xpgmen));
   IOBUF	 i_senspgm	 (.I(xpgmen?(~xfpgaprog):force_senspgm), .IO(senspgm), .T(~(xpgmen || force_senspgm)), .O(senspgmin));
// 347 mode
   always @ (posedge clk) begin
    fs_composite<=fs_composite0; // extra delay to match increased delay in data path
    fs_d[1:0] <= {fs_d[0],fs_composite};
    hact347      <= fs_d[0] && (fs_composite || fs_d[1]);
   end
endmodule

module sensio1(c2,  // pclk2x
               e,   // inverted pclk, enable for FFs clocked at pclk2x to make them as if clocked at pclk
               d,   // data to be sent out (in fillfactory mode)
               t,   // tristate (all but fillfactory)
               q,   // pad data registered @ posedge c2 (enabled by e)
               dq,  // pad
               qp,  // non-registered data read from the pad
               q2); // pad data registered at negedge c2
    input c2,e;
    input d;
    input t;
    output q,q2;
    output qp;
    inout dq;
    sensio0 i_dq (.c2(c2),.e(e),.d(d),.t(t),.q(q),.dq(dq),.qp(qp),.q2(q2));
// s-ynthesis attribute KEEP_HIERARCHY of i_dq is "TRUE"
endmodule

module sensio0(c2,e,d,t,q,dq,qp,q2);
    input c2,e;
    input d;
    input t;
    output q,qp,q2;
    inout dq;

    wire dr,tr,qp;

    IOBUF i_dq (.I(dr), .T(tr),.O(qp), .IO(dq));
    FDE   i_dr (.C(c2), .CE(e), .D(d), .Q(dr));
    FDE   i_tr (.C(c2), .CE(e), .D(t), .Q(tr));
    FDE   i_q  (.C(c2), .CE(e), .D(qp),.Q(q));
    FDE_1 i_q2 (.C(c2), .CE(e), .D(qp),.Q(q2));
// synthesis attribute IOB of i_dr is "TRUE"
// synthesis attribute IOB of i_tr is "TRUE"
// synthesis attribute IOB of i_q  is "TRUE"
// synthesis attribute IOB of i_q2 is "TRUE"
endmodule


// making I/O module with a second trigger to detect correct phase. Same data is registered at twice clock
// (and on the other edge) at is nominally +/-(Tpclk/4) that is  +/-2.6ns @ 96MHz
// e is supposed to be equal to !pclk, just not global - then data can be registered @ posedge pclk
module ipadqffc (c2,e,q,qr,qr2,d,dq,t);  //
    input c2;
    input e;
    output q;
    output qr,qr2;
    input d;
	 inout dq;
	 input t;
	ipadqffc0 i_q (.c2(c2),.e(e),.q(q),.qr(qr),.qr2(qr2),.d(d),.dq(dq),.t(t));
// s-ynthesis attribute KEEP_HIERARCHY of i_q is "TRUE"
endmodule
module ipadqffc0(c2,e,q,qr,qr2,d,dq,t);
    input c2;
    input e;
    output q;
    output qr,qr2;
    input d;
	 inout dq;
	 input t;
  IOBUF i_dq  (.I(d), .T(t),   .O(q), .IO(dq));
  FDE	  i_qr  (.C(c2),.CE(e),  .D(q), .Q(qr));
  FDE_1 i_qr2 (.C(c2),.CE(e),  .D(q), .Q(qr2));
// synthesis attribute IOB of i_qr is "TRUE"
// synthesis attribute IOB of i_qr2 is "TRUE"
// synthesis attribute NODELAY of i_dq is "TRUE"

endmodule
