//fpga_io.h
#ifndef _FPGA_IO_H
#define _FPGA_IO_H

int fpga_io_ioctl(struct inode *inode, struct file *file,  unsigned int cmd, unsigned long arg);

int fpga_io_open(void);
int fpga_io_close (void);

/* added read/write/lseek to read and write FPGA registers
 * write fromat (6 bytes per write):
 * byte 0 - address LSB
 * byte 1 - address MSB (high bit - 0 - ~=writereg, 1 -~=writereg4)
 * byte 2 - data LSB
 * ...
 * byte 5 - data MSB
 *
 * for reading registers - use lseek to 16-bit address (high bit readreg/readreg4)
 * byte address (for write - 32-bit address)
 * read data - 4 bytes (LSB first), reading again will not autoincrement registers
 *
 */
/*!
 * Changing again:
 * 1. Lseek selects a 32-bit register, Bit 13 selects between CSP0 and CSP4 (needed for buffer reads that increment buffer address - i.e. histogram)
 * 2. Writes and Reads always start from register LSB (i.e. multiple one byte reads will each return LSB of the same register), addresses are not autoincremented
 * 3. Writes with less than muptiple of 4 bytes will fill higher bytes with zeros, register addresses are not auto incremented
 */
ssize_t fpga_io_write  (struct file * file, const char * buf, size_t count, loff_t *off);
loff_t  fpga_io_lseek  (struct file * file, loff_t offset, int orig);
ssize_t fpga_io_read   (struct file * file, char * buf, size_t count, loff_t *off);

ssize_t fpga_io_table_write  (struct file * file, const char * buf, size_t count, loff_t *off);
loff_t  fpga_io_table_lseek  (struct file * file, loff_t offset, int orig);


///extern void fpga_table_write(unsigned long fpga_addr, unsigned long *data, int length); - was only in hist353.c
/// write FPGA tables/read histograms
//extern void fpga_table_write_nice (int addr, int len, unsigned long * data);
//extern void fpga_hist_read_nice   (int addr, int len, unsigned long * data);
void fpga_table_write_nice (int addr, int len, unsigned long * data);
void fpga_hist_read_nice   (int addr, int len, unsigned long * data);
#endif
