#!/bin/sh
# 
# getfiles.sh
#
# Script for copying and importing mtd files into nandboot
#
# -----------------------------------------------------------------------
# (C) Copyright 2007, Axis Communications AB, LUND, SWEDEN
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 52 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# -----------------------------------------------------------------------

MTD_IMPORT_DIR=import-mtd
CRISV32_IMPORT_DIR=import-crisv32
MTD_NANDBOOT_DIR=mtd
CRISV32_NANDBOOT_DIR=crisv32

if [ -z "$AXIS_KERNEL_DIR" ]; then
  echo "\$AXIS_KERNEL_DIR not set. Did you forget to do . ./init_env?"
  exit 1;
fi

echo ""
echo "This script will copy files from $AXIS_KERNEL_DIR"
echo "into the ${MTD_IMPORT_DIR}/ and ${CRISV32_IMPORT_DIR}/ subdirectories."
echo -n "Are you sure this is what you want to do? "

read ANSWER
if [ "$ANSWER" != yes ]; then
  echo "Ok, exiting..."
  exit 0;
fi

echo ""
echo "Copying files from kernel in $AXIS_KERNEL_DIR..."

mkdir -p -v "$MTD_IMPORT_DIR" && \
cp -v "$AXIS_KERNEL_DIR"/include/mtd/mtd-abi.h $MTD_IMPORT_DIR/mtd-abi.h && \
cp -v "$AXIS_KERNEL_DIR"/include/linux/mtd/mtd.h $MTD_IMPORT_DIR/mtd.h && \
cp -v "$AXIS_KERNEL_DIR"/include/linux/mtd/nand.h $MTD_IMPORT_DIR/nand.h && \
cp -v "$AXIS_KERNEL_DIR"/drivers/mtd/nand/nand_base.c $MTD_IMPORT_DIR/nand_base.c && \
cp -v "$AXIS_KERNEL_DIR"/drivers/mtd/nand/nand_bbt.c $MTD_IMPORT_DIR/nand_bbt.c && \
cp -v "$AXIS_KERNEL_DIR"/drivers/mtd/nand/nand_ecc.c $MTD_IMPORT_DIR/nand_ecc.c && \
cp -v "$AXIS_KERNEL_DIR"/drivers/mtd/nand/nand_ids.c $MTD_IMPORT_DIR/nand_ids.c &&
mkdir -p -v "$CRISV32_IMPORT_DIR" && \
cp -v "$AXIS_KERNEL_DIR"/arch/cris/arch-v32/drivers/nandflash.c $CRISV32_IMPORT_DIR/nandflash.c &&
cp -v "$AXIS_KERNEL_DIR"/arch/cris/arch-v32/lib/nand_init.S "$CRISV32_IMPORT_DIR"/nand_init.S &&
cp -v "$AXIS_KERNEL_DIR"/arch/cris/arch-v32/boot/rescue/head.S "$CRISV32_IMPORT_DIR"/head.S

if [ $? -ne 0 ]; then
  echo "Couldn't copy all files, bailing out."
  exit 2;
fi

echo ""
echo "Now, specify kernel version to be used as part of the import tag"
echo "For instance, if you say IR2_6_19-3 here, the import tag will be"
echo "import--mtd_IR2_6_19-3."
echo -n "Specify import tag suffix: "

read IMPORT_SUFFIX

CVSROOT=:ext:dev-cvs:/usr/local/cvs/linux
VENDOR_TAG=import_branch--linux_kernel
IMPORT_TAG=import--linux_kernel-$IMPORT_SUFFIX

CVS_MTD_CMD="cvs -d $CVSROOT import -ko os/nandboot/"$MTD_NANDBOOT_DIR" $VENDOR_TAG $IMPORT_TAG"
CVS_CRISV32_CMD="cvs -d $CVSROOT import -ko os/nandboot/"$CRISV32_NANDBOOT_DIR" $VENDOR_TAG $IMPORT_TAG"

echo ""
echo "cvs commands are:"
echo ""
echo "$CVS_MTD_CMD"
echo ""
echo "and"
echo ""
echo "$CVS_CRISV32_CMD"
echo ""

echo "Please cd to $MTD_IMPORT_DIR and $CRISV32_IMPORT_DIR, respectively"
echo "and use copy+paste to execute the relevant command above. Thank you."
