#
# Shell functions to be used by the scripts in /etc/init.d/ to help out with
# network interfaces.
#
# (C) Copyright 2005-2007, Axis Communications AB, LUND, SWEDEN
# This file is released under the GPL v2.

RC_GOT_NET=y

# set media (atm it only works for ethernet devices)
net_set_media() {
	if [ ! -x /bin/ethtool ]; then
		return 0
	fi

	_net_iface=$1

	if [ -z "$MEDIA" ]; then
		information "media: missing, using default"
		return 0
	fi

	[ -f /bin/eth_ctrl ] && /bin/eth_ctrl

	case "$MEDIA" in
		auto)
			ethtool_set_options="autoneg on"
			;;
		10baseT-HD)
			ethtool_set_options="autoneg off speed 10 duplex half"
			;;
		10baseT-FD)
			ethtool_set_options="autoneg off speed 10 duplex full"
			;;
		100baseTX-HD)
			ethtool_set_options="autoneg off speed 100 duplex half"
			;;
		100baseTX-FD)
			ethtool_set_options="autoneg off speed 100 duplex full"
			;;
		*)
			warning "$0: Unknown media: $MEDIA"
			return 1
			;;
	esac

	if /bin/ethtool -s $_net_iface $ethtool_set_options; then
		information "media: $MEDIA"
		return 0
	else
		warning "$0: ethtool failed!"
		return 1
	fi
}

net_set_hwaddr() {
	[ "$1" ] || error "$0:net_set_hwaddr: device name argument missing"
	[ "$MAC" ] || error "$0: MAC is missing!"
	hwaddr=$(ip link show $1 | sed -n -e 's!^[[:space:]]\+link/ether[[:space:]]\+\([:[:xdigit:]]\+\)[[:space:]]\+.*!\1!p' | tr 'a-f' 'A-F')
	
	# try to search MAC in kernel commandline
	CMDLINE=`cat /proc/cmdline`
	M=""
	for l in ${CMDLINE} ; do
		a=`echo $l | grep 'mac='`
		if [ "$a" != "" ] ; then
			M=$a
		fi
	done
	if [ "$M" != "" ] ; then
		MAC=`echo $M | sed -e 's/://g' | sed -e 's/mac=//'`
		M=`bootblocktool -x SERNO`
		if [ "$M" != "" ] ; then
			information "skip save MAC in bootblock"
		else
			information "save MAC in bootblock"
			bootblocktool -w SERNO=$MAC
		fi
		MAC=`echo $MAC | sed -e 's/\([0-9A-F]\{2\}\)/\1:/g;s/:$//'`
		
	fi
	if [ "$MAC" != none ] && [ "$MAC" != "$hwaddr" ] && [ "$M" = "" ] ; then
		ip link set $1 address $MAC &&
			information "hardware address: $MAC"
	fi
	if [ "$M" != "" ] ; then
		ip link set $1 address $MAC &&
			information "hardware address from kernel command line: $MAC"
	fi
}

net_wait_for_link() {
	if [ ! -x /bin/ethtool ]; then
		return 0
	fi

	_net_iface=$1

	_mac_type=$2

	if [ "$_mac_type" = "802.3" ]; then
		err=0
		while :; do
			if ethtool $_net_iface | \
			   grep -q 'Speed:[[:space:]]\+[[:digit:]]\+Mb/s'; then
				return 0
			else
				err=$(($err + 1))
				[ $err -lt 24 ] || return 1
			fi
			usleep 200000
		done
	elif [ "$_mac_type" = "802.11" ]; then
		err=0
		while :; do
			STATUS=$(iwconfig $_net_iface)
			STATUS="${STATUS#*ESSID:\"}"
			UPLINK="${STATUS%%\"*}"
			if [ -z "$UPLINK" ]; then
				err=$(($err + 1))
				[ $err -lt 10 ] || return 1
				usleep 500000
			else
				return 0
			fi
		done
	fi

	return 1
}

# address management

# NOTE: May be oversimplistic
net_addr_family() {
	[ "$1" ] || error "$0:net_addr_family: ip address argument missing"
	if expr $1 : '\(\(\([0-9]\{1,3\}\.\)\{3\}\)[0-9]\{1,3\}\)' >/dev/null; then
		_af=4
	elif [ $(expr index $1 ':') -gt 0 ]; then
		_af=6
	else
		_af=0
	fi
	return $_af
}

net_route_exists() {
	[ $# -lt 3 ] && error "$0:net_route_exists: missing some argument(s)"
	_dest=$1
	[ "$_dest" = default ] && _dest="0.0.0.0/0"
	[ $(ip -f inet route list $_dest via $2 dev $3) ]
}

net_flush_ipv6_addresses() {
	[ "$1" ] || error "$0:net_flush_ipv6_addresses: device name argument missing"
	ip -f inet6 address flush dev $1 2>/dev/null
}
