<?php

/*
Web-control for launcher program.
Abstract:
	launcher (/etc/init.d/launcher.php) is subsystem what provide user-friendly possibility to add alunch of some programs after all initscripts at the system.
	Program can be launched at background mode (at default the run sequence is program after program); with output redirect to some file (by default to /dev/null).
Interface:
	/launcher.php: return current configuration XML file for launcher (just return file /etc/launcher.conf.xml)
	/launcher.php?cmd="add|remove|update"&program="name_of_program"[&log="log_file_name"][&background="true|false"]
		- cmd="add" - add program to launch
		- cmd="remove" - remove program from launch if record present
		- cmd="update" - update program record - i.e. change parameters 'log' and 'background' for it
		- opt=wait - only for tests to prevent race conditions
		- program="name_of_program" - name of program to launch; for example: program="ls"
		- log="log_file_name" - log file for program output; for example: log='/var/state/ls.log' 
		- background="true|false" - run program in background (by default == false). If program will run in background, launcher run this program
			and dont wait for program exit - instead, launcher will run next program from the list.

*/

$conf_file='/etc/launcher.conf.xml';

function _result($cmd, $message, $result) {
	$rez = "<?xml version='1.0' standalone='yes'?>\n<launcher cmd='".$cmd."' result='".$result."'>".$message."</launcher>\n";
	header("Content-Type: text/xml");
	header("Content-Length: ".strlen($rez)."\n");
	header("Pragma: no-cache\n");
	printf($rez);

	flush();
	exit();
}

$cmd = $_GET['cmd'];

if($_GET['opt'] == 'wait')
	while(!is_file($conf_file))
		sleep(1);
if(!is_file($conf_file))
	_result($cmd, "launcher not installed properly (missing $conf_file). Skip", 'FAIL');

$cfg = file_get_contents($conf_file);

$cmd_add = ($cmd == 'add');
$cmd_remove = ($cmd == 'remove');
$cmd_update = ($cmd == 'update');

if($cmd_add || $cmd_remove || $cmd_update) {
	$xml = new SimpleXMLElement($cfg);
	$program = '';
	$log = '';
	$background = 'false';
	if($_GET['program'])
		$program = $_GET['program'];
	if($_GET['log'])
		$log = $_GET['log'];
	if($_GET['background'])
		$background = $_GET['background'];

	if($program == '')
		_result($cmd, "parameter \"program\" must be specified", 'FAIL');

	$l = $xml->launch;
	$c = count($xml->launch);
	$index = -1;
	for($i = 0; $i < $c; $i++) {
		$x = $l[$i];
		if($x['program'] == $program) {
			$index = $i;
			break;
		}
	}
	if($cmd_add) {
		if($index != -1) {
			_result($cmd, "record for program '".$program."' already exist", 'FAIL');
		} else {
			$x = "launch program='".$program."' log='".$log."' background='".$background."'";
			$xml->addChild($x);
		}
	}
	if($cmd_update) {
		if($index == -1) {
			_result($cmd, "record for program '".$program."' does not exist", 'FAIL');
		} else {
			unset($xml->launch[$index]);
			$x = "launch program='".$program."' log='".$log."' background='".$background."'";
			$xml->addChild($x);
		}
	}
	if($cmd_remove) {
		if($index == -1) {
			_result($cmd, "record for program '".$program."' does not exist", 'FAIL');
		} else {
			unset($xml->launch[$index]);
		}
	}

	// save new XML...
	$xml = $xml->asXML();
	$f = fopen($conf_file, 'w');
	fwrite($f, $xml);
	fclose($f);
//	_result($cmd, '', 'OK');
	_result($cmd, $c, 'OK');
	exit();
}

// just return configuration XML
$rez = $cfg;
header("Content-Type: text/xml");
header("Content-Length: ".strlen($rez)."\n");
header("Pragma: no-cache\n");
printf($rez);

flush();

/*
$xml = @new SimpleXMLElement(file_get_contents($conf_file));
$c = count($xml->launch);
if($c == 0) {
	echo "launcher: nothing to launch\n";
	exit();
}
// <launch program="[patch]program_name" [log="log_file_name"] [background="true|false"]/>
for($i = 0; $i < $c; $i++) {
	$l = $xml->launch[$i];
	$program = $l['program'];
	if($program == "")
		continue;
	$cmd = $program;
	$cmd .= ' 2>&1 ';
//	$cmd .= ' > ';
	$log = ' /dev/null ';
	if($l['log'] != "")
		$log = $l['log'];
	// do fork for background
	if($l['background'] == "true") {
		$cmd = '('.$cmd.' &) > '.$log;
		echo 'launcher: run command: '.$cmd."\n";
		system($cmd);
	} else {
//	$cmd = "(/etc/init.d/test.php 2>&1 &) > /var/state/ls.log";
		$cmd .= ' > '.$log;
		echo 'launcher: run command: '.$cmd."\n";
		exec($cmd);
	}
}

echo "launcher: finish\n";
*/
?>
