#!/usr/local/sbin/php -q
<?php
/*!
*! PHP script
*! FILE NAME  : test_10359.php
*! DESCRIPTION: The script is for tesing 10359 board with 5MPix sensors.
*!              1. Initializes 10359 board.
*!              2. Checks 10353-10359 I2C connection.
*!              3. Detects and programs sensors (up to 3).
*!              4. Adjusts clock phase (+/-40*5 steps) for each sensor until MD5 sum of a frame is correct. 
*! AUTHOR     : spectr, dzhimiev
*! Copyright (C) 2008 Elphel, Inc
*! -----------------------------------------------------------------------------**
*!
*!  This program is free software: you can redistribute it and/or modify
*!  it under the terms of the GNU General Public License as published by
*!  the Free Software Foundation, either version 3 of the License, or
*!  (at your option) any later version.
*!
*!  This program is distributed in the hope that it will be useful,
*!  but WITHOUT ANY WARRANTY; without even the implied warranty of
*!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*!  GNU General Public License for more details.
*!
*!  You should have received a copy of the GNU General Public License
*!  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*! -----------------------------------------------------------------------------**
*!
*!  $Log: test_inner_10359.php,v $
*!  Revision 1.3  2009/03/03 18:56:59  dzhimiev
*!  1. added a 10-second delay before starting the test to avoid it during setting the sensor's frequency and the sensor's initialization
*!
*!  Revision 1.2  2009/02/26 14:04:45  dzhimiev
*!  1. pause after channel switch - more stable work
*!  2. test fails if one of the phases fails to be adjusted
*!
*!  Revision 1.1  2009/02/05 20:00:00  dzhimiev
*!  files for 10359 board test - ported from 7.x to 8.x software version
*!
*!
*/

require 'i2c.inc';

// result of the test, what will be return by test_10359.php on request
$res_fn = "/etc/test/board_10359.res";
// program name for launcher
$self_fn="test_inner_10359.php";

$bdetect_log="/var/log/bdetect";

//------------------------------------------------------------------------------
// general flow functions

function pr($text) {
	echo "<pre>$text</pre>\n";
	flush();
}

# result - stored in a file
function res_update($name, $result, $description) {
	global $res_fn;

	$file = fopen($res_fn, "a+");
	if($result != "")
		fwrite($file, "<${name} result=\"${result}\">".$description."</${name}>\n");
	else
		fwrite($file, "<${name}>".$description."</${name}>\n");
	fclose($file);

	pr("<${name} result=\"${result}\">".$description."</${name}>");
}

function _finish($res) {
	res_update("finish", $res, "");
	//exec("sync");
	exit(0);
}

$sensor_type = '5MPx';

function apply_def_settings($pass) {
	$DEF_QUALITY = 96;
	$DEF_WOI_WIDTH = 10000;
	$DEF_WOI_HEIGHT = 10000;
	// sample gamma == 1.0
	$DEF_GAMMA = 6554624;
	$DEF_COLOR = 2;	// color mode - "color, YCbCr 4:2:0, 3x3 pixels"

	// copy predefined settings with autocampars.php
	$conf_xml = simplexml_load_file("/etc/autocampars.xml");
	foreach($conf_xml->paramSets->children() as $paramSet) {
		$numSet = (integer)$paramSet->attributes()->number;
		if($numSet == 0) {
			if($pass == 1 || $pass == 2 || $pass == 0) {
				$paramSet->QUALITY = $DEF_QUALITY;
				$paramSet->WOI_WIDTH = $DEF_WOI_WIDTH;
				$paramSet->WOI_HEIGHT = $DEF_WOI_HEIGHT;
				$paramSet->GTAB_R = $DEF_GTAB_R;
				$paramSet->GTAB_G = $DEF_GTAB_G;
				$paramSet->GTAB_B = $DEF_GTAB_B;
				$paramSet->GTAB_GB = $DEF_GTAB_GB;
				$paramSet->SENSOR_RUN = 2;
				$paramSet->COMPRESSOR_RUN = 2;
				$paramSet->GAINR = 0x20000;
				$paramSet->GAING = 0x20000;
				$paramSet->GAINB = 0x20000;
				$paramSet->GAINGB = 0x20000;
				$paramSet->RSCALE = 0x10000;
				$paramSet->GSCALE = 0x10000;
				$paramSet->BSCALE = 0x10000;
				$paramSet->DGAINR = 32768;
				$paramSet->DGAING = 32768;
				$paramSet->DGAINGB = 32768;
				$paramSet->DGAINB = 32768;
				$paramSet->COLOR = 1;
				$paramSet->AUTOEXP_ON = 0;
				$paramSet->GAIN_MIN = 0x10000;
				$paramSet->GAIN_MAX = 0x10000;
				$paramSet->ANA_GAIN_ENABLE = 0;
				// daemons
				$paramSet->DAEMON_EN = 0;
				$paramSet->DAEMON_EN_AUTOEXPOSURE = 0;
				$paramSet->DAEMON_EN_STREAMER = 0;
				$paramSet->DAEMON_EN_CCAMFTP = 0;
				$paramSet->DAEMON_EN_CAMOGM = 0;
				$paramSet->DAEMON_EN_AUTOCAMPARS = 0;
			}
			if($pass == 0) { // for SDRAM test
		        	$paramSet->SENSOR_RUN = 0;
		        	$paramSet->COMPRESSOR_RUN = 0;
			}
	    }
	}
	$z = $conf_xml->asXml();
	$f = fopen('/tmp/cd359_def.xml', 'w+');
	fwrite($f, $z);
	fclose($f);
	exec('wget http://127.0.0.1/autocampars.php?load=/tmp/cd359_def.xml');
}

function sensor_init($init_pars){
	for ($i=0;$i<256;$i++){
		i2c_send(16,0,0x4800+$i,$init_pars[$i],0);	
		$readout=i2c_receive(16,0,0x4800+$i,0);
	}
}

function send_i2c_4($a,$d) {
	$i2c  = fopen('/dev/xi2c16','w');
	fseek ($i2c, 2*$a) ;
	$res=fwrite($i2c, chr (floor($d/(256*256*256))).chr (($d - 256*256*256*floor($d/(256*256*256)))/(256*256)).chr (($d - 256*256*floor($d/(256*256)))/256).chr ($d - 256*floor($d/(256))) );
	fclose($i2c);
	return $res;
} // end of send_i2c()

function receive_i2c_4($width,$bus,$a,$raw=0) {
   $w=($width==16)?2:1;
   $i2c_fn='/dev/xi2c'.($raw?'raw':(($w==2)?'16':'8')).(($bus==0)?'':'_aux');
   $i2c  = fopen($i2c_fn, 'r');
	 fseek ($i2c, 4, SEEK_END);
   fseek ($i2c, $w*$a);
	 
   $data = fread($i2c, 2*$w);
   fclose($i2c);
   if (strlen($data)<2*$w) return -1;
   $v=unpack(($w==1)?'C*':'N*',$data);
	 //print_r($v);
	 //printf("0x%x\n",$v[1]);
	 return $v[1];
} // end of receive_i2c()


function _sensor_id() {
	global $sensor_type;

	$s_id = i2c_receive(16, 0, 0x4800, 0);
	$sensor_id = sprintf("0x%x",$s_id);
	if($sensor_id == '0xffffffff') {
		$sensor_type = '3MPx';
		$s_id = i2c_receive(16, 0, 0x5D00, 0);
		$sensor_id = sprintf("0x%x",$s_id);
	}
	return $sensor_id;
}

function _sensor_pattern_set() {
	global $sensor_type;

	if($sensor_type == '5MPx') {
//		i2c_send(16, 0, 0x48A0, 0x19, 0);
		// use other test pattern
		i2c_send(16, 0, 0x48A0, 0x41, 0);
		return;
	}
	if($sensor_type == '3MPx') {
		$addr = 0x5D00;
		i2c_send(16, 0, $addr + 0x32, 0xAAAA, 0);
		$ctrl = i2c_receive(16, 0, $addr + 0x07, 0);
		$ctrl |= 0x40;
		i2c_send(16, 0, $addr + 0x07, $ctrl, 0);
		return;
	}
}

function _sensor_pattern_unset() {
	global $sensor_type;

	if($sensor_type == '5MPx') {
		i2c_send(16, 0, 0x48A0, 0x00, 0);
		return;
	}
	if($sensor_type == '3MPx') {
		$addr = 0x5D00;
		$ctrl = i2c_receive(16, 0, $addr + 0x07, 0);
		$ctrl &= 0xFFBF;
		i2c_send(16, 0, $addr + 0x07, $ctrl, 0);
		return;
	}
}

function md5sum_find($xml_md5sum, $sensor_id, $fpga_rev) {
	$rez->sensor = false;
	$rez->fpga = false;
	$rez->md5sum = "";

	for($i = 0; $i < count($xml_md5sum->md5sum); $i++) {
		$md5 = $xml_md5sum->md5sum[$i];
		if($md5['sensor_id'] == $sensor_id)
			$rez->sensor = true;
		if($md5['fpga_rev'] == $fpga_rev)
			$rez->fpga = true;
		if($md5['sensor_id'] == $sensor_id && $md5['fpga_rev'] == $fpga_rev) {
			$rez->md5sum = $md5;
			break;
		}
	}
	return $rez;
}

function phase_adjust($channel,$md5){
  $phase_shift=1;
  $md5_new = "";
  $md5_old = "";
  for($i=0;$i<120;$i++){
    if ($i==40) $phase_shift=2;
    file_get_contents('http://127.0.0.1/359/phases_adjust.php?dcm='.($channel+1).'&phase_shift='.$phase_shift.'&n=5');
    //exec("wget http://127.0.0.1:8081/noexif/next/wait/img -O /dev/null");
    exec("wget http://127.0.0.1:8081/noexif/next/wait/img -O /tmp/spc.jpeg");
    if(is_file("/tmp/spc.jpeg")) {
      $md5_new = substr(exec("md5sum /tmp/spc.jpeg"), 0, 32);
      unlink("/tmp/spc.jpeg");
    }      
    if (($md5_new==$md5)&&($md5_old==$md5)) return true;
    $md5_old=$md5_new;
  }
  return false;
}
//------------------------------------------------------------------------------
$flag = FALSE;

// at the stage processing we must do this:
// if test passed:
// - update results record
// - increase stage number
// - reboot
// if test fail:
// - update results record
// - finish

if(is_file($res_fn))
  unlink($res_fn);
if(is_file($bdetect_log)) {
  if (file_get_contents("/var/log/bdetect")=="10359") $flag = true;
  res_update("BOARD_ID", "ok", file_get_contents("/var/log/bdetect"));
}

if(!$flag) {
  res_update("BOARD_ID", "fail", '');
  _finish("fail");
}

if(test_main() == false) {
  res_update("test", "fail", "main test failed");
  _finish("fail");
} else {
  res_update("test", "ok  ", "main test passed");
  _finish("ok");
}

function test_main() {
  $MD_LENGTH = 32;
  $spc_xml_file = "/etc/test/spc.xml";
  $md5sums = "";

  //return false;
  if(file_exists($spc_xml_file)) {
	  $md5sums = new SimpleXMLElement(file_get_contents($spc_xml_file));
  } else {
	  $md5sums = "<?xml version='1.0' standalone='yes'?><10359_test>\n";
	  // sample of records: <md5sum sensor_id='0x1801' fpga_rev='0x353301e'>df5bf0c75fd10b1f9de3847076e32be2</md5sum>
	  $md5sums .= "</sensor_pattern_test>";
	  $md5sums = new SimpleXMLElement($md5sums);
  }

// FPGA revision...-------------------------------------------------------------------------------------------------------------
  $data = elphel_fpga_read(0x13);
  $fpga_rev = sprintf("0x%x",$data);

// sensors present
  $sensors=array(false,false,false);
  $phases=array(false,false,false);
  sleep(10);
// 10353-10359 I2C connection check---------------------------------------------------------------------------------------------
  if (i2c_receive(16, 0, 0x800, 0)==0x359) {
    res_update("FIRMWARE", "ok",'');
  } else {
    res_update("FIRMWARE", "fail", "An old firmware revision or i2c doesn't work, got ".i2c_receive(16, 0, 0x800, 0));
    _finish("fail");
  }

// Detect sensor on channel 0 --------------------------------------------------------------------------------------------------
  if (i2c_receive(16, 0, 0x4800, 0)==0x1801) {
    res_update("SENSOR_ch0_id", "ok", "0x1801");
    $sensors[0]=true;
  } else {
    res_update("SENSOR_ch0_id", "fail", "10338 is not connected to channel 0 or I2C bus between 10359 and 10338 doesn't work. Got ".i2c_receive(16, 0, 0x4800, 0));
    _finish("fail");
  }

// Detect sensors on channel 1 and 2 -------------------------------------------------------------------------------------------
  for($i=1;$i<3;$i++){
    // Switch to channel i
    send_i2c_4(0x835,2*$i);
    // Detect sensor on channel i
    if (i2c_receive(16, 0, 0x4800, 0)==0x1801){
      res_update("SENSOR_ch".$i.'_id', "ok", "0x1801");
      $sensors[$i]=true;
    } else
    res_update("SENSOR_ch".$i.'_id', "fail", "10338 is not connected to channel ".$i." or I2C bus between 10359 and 10338 doesn't work. 	Got ".i2c_receive(16, 0, 0x4800, 0));
  }
//// back to channel 0
  send_i2c_4(0x835,1);

//  send_i2c_4(0x0835,0x00000001); //printf("\nReading parameters from sensor <font size=\"6\">1</font>\n");
  exec("fpcf -i2cw16 48a0 0041");
  sleep(1);
  apply_def_settings(0);
  sleep(2);
  apply_def_settings(1);
  for($i = 0; $i < 6; $i++) exec('wget http://127.0.0.1:8081/noexif/next/wait/img -O /dev/null');
  sleep(3);
  apply_def_settings(2);
  for($i = 0; $i < 6; $i++) exec('wget http://127.0.0.1:8081/noexif/next/wait/img -O /dev/null');

// read sensor parameters
  for ($i=0;$i<256;$i++){
	  $init_pars3[$i]=i2c_receive(16,0,0x4800+$i,0);
  }

  send_i2c_4(0x0835,0x00000002); //printf("\nInitializing sensor <font size=\"6\">2</font>\n");
  sensor_init($init_pars3);
  exec("fpcf -i2cw16 48a0 0041");
  sleep(1);
  apply_def_settings(0);
  sleep(2);
  apply_def_settings(1);
  for($i = 0; $i < 6; $i++) exec('wget http://127.0.0.1:8081/noexif/next/wait/img -O /dev/null');
  sleep(3);
  apply_def_settings(2);
  for($i = 0; $i < 6; $i++) exec('wget http://127.0.0.1:8081/noexif/next/wait/img -O /dev/null');

  send_i2c_4(0x0835,0x00000004);
  sensor_init($init_pars3);
  exec("fpcf -i2cw16 48a0 0041");
  sleep(1);
  apply_def_settings(0);
  sleep(2);
  apply_def_settings(1);
  for($i = 0; $i < 6; $i++) exec('wget http://127.0.0.1:8081/noexif/next/wait/img -O /dev/null');
  sleep(3);
  apply_def_settings(2);
  for($i = 0; $i < 6; $i++) exec('wget http://127.0.0.1:8081/noexif/next/wait/img -O /dev/null');
//end of init//


//// back to channel 0
send_i2c_4(0x835,1);

  $sensor_id = _sensor_id();

  //--------------------
  //get test pattern reference MD5 sum
  $md5sum = "";

  $rez = md5sum_find($md5sums, $sensor_id, $fpga_rev);
  if($rez->md5sum == "") {
    res_update('FPGA_compress', 'unknown', 'control MD5SUM for sensor ID '.$sensor_id.' and FPGA revision '.$fpga_rev.'not found');
    _finish("fail");
  } else
    $md5sum = $rez->md5sum;

  elphel_compressor_run();

  //Adjust phases for detected channels-------------------------------------------------------------------------------------------
  while(list($key, $value) = each($sensors)) {
    if ($value) {
      switch($key) {
	case 0  : $chn=1; break;
	case 1  : $chn=2; break;
	case 2  : $chn=4; break;
	default : $chn=0;
      }   
      send_i2c_4(0x835,$chn);
      _sensor_pattern_set();
      if (phase_adjust($key,$md5sum)) {res_update('SENSOR_ch'.$key.'_phase', 'ok', ''); $phases[$key]=true;}
      else                            res_update('SENSOR_ch'.$key.'_phase', 'fail', 'Failed to adjust correct phase');
      _sensor_pattern_unset();   
    }
  }

  //// back to channel 0
  send_i2c_4(0x835,1);

// do some other tests, and update subtest results
// res_update("some_subtest_name", "fail", "just test number X");
  if ($phases[2] && $phases[1] && $phases[0])
    return true;
  else 
    return false;
}

/*
//SDRAM test -------------------------------------------------------------------------------------------------------------------

//$row_adr=0x3ff;
//ddr_init();

//$written_pages=array(array(),array(),array(),array(),array());

//for($i=$row_adr;$i<$row_adr+5;$i++) {
//  $written_pages[$row_adr-$i]=ddr_page_write($i);
//  res_update('Page_'.$i, 'ok', $written_pages[$row_adr-$i][0]);
//}
////for($i=$row_adr;$i<$row_adr+5;$i++) ddr_page_read($i);
*/
?>
