#!/usr/bin/php -q
<?php
//apt-get install php5.cli
/*!
*! PHP script
*! FILE NAME  : prep_kdevelop.php
*! DESCRIPTION: Creates files to use KDevelop with Elphel camera project
*! AUTHOR     : Andrey Filippov
*! Copyright (C) 2008 Elphel, Inc
*! -----------------------------------------------------------------------------**
*!
*!  This program is free software: you can redistribute it and/or modify
*!  it under the terms of the GNU General Public License as published by
*!  the Free Software Foundation, either version 3 of the License, or
*!  (at your option) any later version.
*!
*!  This program is distributed in the hope that it will be useful,
*!  but WITHOUT ANY WARRANTY; without even the implied warranty of
*!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*!  GNU General Public License for more details.
*!
*!  You should have received a copy of the GNU General Public License
*!  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*! -----------------------------------------------------------------------------**
*!
*!  $Log: prep_kdevelop.php,v $
*!  Revision 1.1.1.1  2008/11/27 20:04:00  elphel
*!
*!
*!  Revision 1.4  2008/09/19 04:37:27  elphel
*!  snapshot
*!
*!  Revision 1.3  2008/09/15 16:47:35  elphel
*!  "compile" driver file now runs make in os/linux-2.6 (builds kernel)
*!
*!  Revision 1.2  2008/09/14 19:18:15  elphel
*!  removed duplicates from ctags
*!
*!  Revision 1.1  2008/09/03 18:32:19  elphel
*!  added prep_kdevelop.php
*!
*/
$makefile_header= '#Dummy Makefile.am for Elphel camera KDevelop project';
$HOME_DIR=        trim(shell_exec('cd ~ ; pwd'));
$PWD=             trim(shell_exec('pwd'));
$TAG_FILE=        "$PWD/tags";
$TAG_EXCLUDE_FILE="$PWD/ctags_excludes";
//$KDEVELOP_RC_FILE='~/.kde/share/config/kdeveloprc'; //'~/' does not work with file_exists :-(
$KDEVELOP_RC_FILE="$HOME_DIR/.kde/share/config/kdeveloprc";
$PROJECT_KDEVELOP='elphel353.kdevelop';
$DOXYFILE=        'Doxyfile';
$REVISION=        trim(shell_exec('grep "JIMAGEID =" Makefile | sed -n -e "s/JIMAGEID = //p"'));
$HWID=            trim(shell_exec('grep "HWID =" Makefile | sed -n -e "s/HWID = //p"'));
$DOXYGEN_EXTENSIONS=array ('c',  'cc',  'cxx','cpp', 'c++','d',  'java','ii', 'ixx',
                           'ipp','i++', 'inl','h',   'hh', 'hxx','hpp', 'h++','idl',
                           'odl','cs',  'php','php3','inc','m',  'mm',  'dox','py',
                           'C',  'CC',  'C++','II',  'I++','H',  'HH',  'H++','CS',
                           'PHP','PHP3','M',  'MM',  'PY');

/// To create dummy Makefile.am,for files to be included in the project. Seems TODO, ..., errors only work in C-files
$PROJECT_SOURCES =array ('c', 'cc',  'cxx','cpp', 'c++', 'C',  'CC',  'C++');
$PROJECT_HEADERS =array ('h', 'H',);
$PROJECT_SCRIPTS =array ('php', 'php3', 'PHP', 'PHP3');


/// Check PHP version >= 5.0.0 (strrchr multi-character needle )
  if (phpversion() < '5.0.0') {
    echo "You need PHP version of at least 5.0.0 to run $argv[0]\n";
    exit (1);
  }
  if ((array_search("--verbose",$argv)===false) && (array_search("-v",$argv)===false)) {
    echo "You may use --verbose or -v command line option to show information about created Makefile.am files\n";
  }
  if ((array_search("--strict",$argv)===false) && (array_search("-s",$argv)===false)) {
    echo "You may use --strict or -s command line option to stop if any of the required KDevelop-related programs is missing\n";
  }
  if (array_search("--force",$argv)===false) {
    echo "You may use --force command line option to force overwriting existent files (created by $argv[0])\n";
  }
  if (array_search("--clean",$argv)===false) {
    echo "You may use --clean command line option to delete most files created by $argv[0]\n";
  }

  echo "\nThis script ($argv[0]) prepares Elphel cameras software to be used with KDevelop\n";
  echo "integrated development environment. You may need to install additional packages for that.\n";
  echo "Looking for required programs...\n";
  $all_installed  = true;
  $all_installed &= test_kdevelop_depend('kdevelop','kdevelop');
  $all_installed &= test_kdevelop_depend('ctags',   'ctags');
  $all_installed &= test_kdevelop_depend('doxygen', 'doxygen');
  $all_installed &= test_kdevelop_depend('dot',     'graphviz');
  if ($all_installed) echo "All required programs are found in your system\n";
  else if ((array_search("--strict",$argv)!==false) || (array_search("-s",$argv)!==false)) {
    echo "Some needed programs are missing, and you required 'strict'.\n";
    exit (1);
  }

  $list=file('src.list');
  array_walk($list,'walk_src_nolinks');
  $list=array_filter  ($list); // remove empty lines
  sort ($list);
/// Arrange src.list file entries as a directory tree
  $files=array();
  foreach ($list as $line) {
    $path=explode ('/',$line);
    if ($path[0]=='.') $path=array($path[1]);
    $pointer=&$files;
    for ($i=0; $i<(count($path)-1); $i++) {
      if (!is_array($pointer[$path[$i]])) $pointer[$path[$i]]=array();
      $pointer=&$pointer[$path[$i]];
    }
    $pointer[]=$path[(count($path)-1)];
  }
/// Now write Makefile.am files
   make_Makefile_am ($files,'.');
/// Create file templates in templates directory
   create_templates();
/// create elphel353.kdevelop file
   if (file_exists($PROJECT_KDEVELOP) && (array_search("--force",$argv)===false)) {
     echo "$PROJECT_KDEVELOP already exists, skipping initialization.\n";
     echo "Please delete $PWD/$PROJECT_KDEVELOP and rerun $argv[0] if you would like $PROJECT_KDEVELOP to be regenerated.\n";
     echo "You may also rerun $argv[0] with --force command line option.\n\n";
   } else {
     file_put_contents  ($PROJECT_KDEVELOP, create_elphel353_kdevelop());
     echo "Created $PWD/$PROJECT_KDEVELOP\n\n";
   }
/// Creating CTAGS file (may need editing, it is currently manual - all included but other (non CRIS) architectures and
/// all applications but PHP and non-elphel PHP extensions
   if (array_search("--clean",$argv)===false) {
     if (shell_exec ("which ctags")) {
       if (file_exists($TAG_EXCLUDE_FILE) && (array_search("--force",$argv)===false)) {
         echo "$TAG_EXCLUDE_FILE already exists, skipping initialization.\n";
         echo "Please delete $TAG_EXCLUDE_FILE and rerun $argv[0] if you would like $TAG_EXCLUDE_FILE to be regenerated.\n";
         echo "You may also rerun $argv[0] with --force command line option.\n\n";
       } else {
         file_put_contents  ($TAG_EXCLUDE_FILE, create_elphel353_ctags_exclusions());
         echo "Created CTAGS exclusions file $TAG_EXCLUDE_FILE\n\n";
       }
       echo "Creating CTAGS file $TAG_FILE (you may regenerate it from KDevelop: CTags tab, Regenarate button))\n\n";
       shell_exec ("ctags -R --c++-types=+px --excmd=pattern --exclude=Makefile --exclude=. --exclude=@$PWD/ctags_excludes -f $TAG_FILE  $PWD");
/// Modifying $KDEVELOP_RC_FILE to include reference to the generated ctags file
       if (file_exists($KDEVELOP_RC_FILE)) {
         $kdeveloprc=file_get_contents($KDEVELOP_RC_FILE);
         $kdevelop_rc_file=fopen($KDEVELOP_RC_FILE,"a");
          if (strpos($kdeveloprc,"[CTAGS]")===false) {
           fwrite($kdevelop_rc_file, "[CTAGS]\n");
           fwrite($kdevelop_rc_file, "JumpToFirst=false\n");
           fwrite($kdevelop_rc_file, "ShowDeclaration=true\n");
           fwrite($kdevelop_rc_file, "ShowDefinition=true\n");
           fwrite($kdevelop_rc_file, "ShowLookup=true\n");
           fwrite($kdevelop_rc_file, "ctags binary=".trim(shell_exec("which ctags"))."\n");
           echo "$KDEVELOP_RC_FILE is modified to include CTAGS executable\n\n";
         }
         fclose($kdevelop_rc_file);
       } else echo "$KDEVELOP_RC_FILE is not found - it can be created by opening/closing KDevelop\n"; 
     }
   } else {
     echo "Deleting CTAGS file $TAG_FILE\n";
     @unlink($TAG_FILE);
     echo "Deleting CTAGS exclusions file $TAG_EXCLUDE_FILE\n";
     @unlink($TAG_EXCLUDE_FILE);
   }
///preparing Doxyfile
   if (array_search("--clean",$argv)===false) {
     if (file_exists($DOXYFILE) && (array_search("--force",$argv)===false)) {
       echo "$DOXYFILE already exists, skipping initialization.\n";
       echo "Please delete $PWD/$DOXYFILE and rerun $argv[0] if you would like $DOXYFILE to be regenerated.\n";
       echo "You may also rerun $argv[0] with --force command line option.\n\n";
     } else {
       file_put_contents  ($DOXYFILE, create_doxyfile());
       echo "Created $PWD/$DOXYFILE\n\n";
     }
   } else {
     echo "Deleting $PWD/$DOXYFILE\n\n";
     @unlink ("$PWD/$DOXYFILE");
   }
   echo "\n";
   echo "#############################################################################################################################\n";
   echo "# You may open project in KDevelop by clicking on the $PROJECT_KDEVELOP in file manager                                     #\n";
   echo "# In the KDevelop, 'build'    means run top 'make' (rebuild flash image)                                                    #\n";
   echo "#                  'compile' means run 'make install' in the directory of the current source file - works with applications #\n";
   echo "# You may create documentation with 'Build'->Build API documentation.                                                       #\n";
   echo "#############################################################################################################################\n\n";

  exit (0);

  
/// Functions
/// Iterate through directory structure and create Makefile.am files to show src.list files as project files
/// in KDevelop.
/// If Makefile.am already exists and does not have $header string it will not be modified, otherwise lines
/// INCLUDES = 
/// METASOURCES = AUTO
/// bin_PROGRAMS = dummy
/// dummy_SOURCES = 
/// SUBDIRS = 
/// will be (re)generated
/// (it is probably possible to carefully modify existent Makefile.am)

function make_Makefile_am ($tree,$path) { /// recursive
  global $makefile_header;
  global $PROJECT_SOURCES, $PROJECT_HEADERS, $PROJECT_SCRIPTS;
  global $argv;
  $subdirs=array();
  $files=array(
                'sources'=> array(),
                'headers'=> array(),
                'scripts'=> array(),
                'other'  => array()
              );
  foreach ($tree as $name=>$entry) {
    if (is_array($entry)) $subdirs[]=$name; /// directories
//    else $files[]=$entry;                   /// regular files
    else { // sort
      if (($pos=strrpos($entry,'.'))) {
        $ext=substr($entry,$pos+1);
        if      ((array_search ( $ext,$PROJECT_SOURCES)) !==false) $files['sources'][]=$entry;
        else if ((array_search ( $ext,$PROJECT_HEADERS)) !==false) $files['headers'][]=$entry;
        else if ((array_search ( $ext,$PROJECT_SCRIPTS)) !==false) $files['scripts'][]=$entry;
        else $files['other'][]=$entry;                   /// other extensions
      } else $files['other'][]=$entry;                   /// no extension
    }
  }
  if (array_search("--clean",$argv)===false) {
    $makefile=$makefile_header."\n".
            "INCLUDES = \n".
            "METASOURCES = AUTO\n";
    if (count($subdirs)){
      $makefile.='SUBDIRS = ';
      for ($i=0; $i<count($subdirs)-1; $i++) $makefile.=$subdirs[$i]." \\\n";
      $makefile.=$subdirs[count($subdirs)-1]."\n";
    }
    if (count($files['sources'])){
      $esc_path=trim(str_replace('-' , '_', str_replace('.' , '_', str_replace('/' , '_', $path ))),"_");
      $makefile.="bin_PROGRAMS = {$esc_path}\n{$esc_path}_SOURCES = ";
      for ($i=0; $i<count($files['sources'])-1; $i++) $makefile.=$files['sources'][$i]." \\\n";
      $makefile.=$files['sources'][count($files['sources'])-1]."\n";
    }
    if (count($files['headers'])){
//    $makefile.="include_HEADERS = ";
      $makefile.="noinst_HEADERS = ";
      for ($i=0; $i<count($files['headers'])-1; $i++) $makefile.=$files['headers'][$i]." \\\n";
      $makefile.=$files['headers'][count($files['headers'])-1]."\n";
    }
    if (count($files['scripts'])){
      $makefile.="bin_SCRIPTS = ";
      for ($i=0; $i<count($files['scripts'])-1; $i++) $makefile.=$files['scripts'][$i]." \\\n";
      $makefile.=$files['scripts'][count($files['scripts'])-1]."\n";
    }
    if (count($files['other'])){
      $makefile.="bin_DATA = ";
      for ($i=0; $i<count($files['other'])-1; $i++) $makefile.=$files['other'][$i]." \\\n";
      $makefile.=$files['other'][count($files['other'])-1]."\n";
    }
  }
///Does Makefile already exist?
  $makefile_path=$path."/Makefile.am";
  $old_makefile="";
  if (file_exists ($makefile_path)) $old_makefile=file_get_contents($makefile_path);
  if (($old_makefile!="") && (strpos($old_makefile,$makefile_header)===false)) {
    if ((array_search("--verbose",$argv)!==false) || (array_search("-v",$argv)!==false)) {
      echo "\"Real\" Makefile.am ($makefile_path) exists, using to ${makefile_path}.elphel\n";
    }
    $makefile_path.=".elphel";
  }
  if (array_search("--clean",$argv)===false) {
    if ((array_search("--verbose",$argv)!==false) || (array_search("-v",$argv)!==false)) {
      echo "Writing to $makefile_path\n";
    }
    file_put_contents  ( $makefile_path, $makefile);
  } else {
    if ((array_search("--verbose",$argv)!==false) || (array_search("-v",$argv)!==false)) {
      echo "Deleting $makefile_path\n";
    }
    @unlink ($makefile_path);
  }
/// recurse subdirectories
  foreach ($subdirs as $subdir)  make_Makefile_am ($tree[$subdir],$path.'/'.$subdir);
}

function walk_src_nolinks(&$line) {
    $pos=strpos($line,'#');
    if ($pos!==false) $line=substr($line,0,$pos);
    $line=ltrim($line);
    if (strpos($line,'===')!==false) $line="";
    if (strpos($line,'->')!==false)  $line=""; // remove links
    $pos=strpos($line,' ');
    if ($pos!==false) $line=substr($line,0,$pos);
    $line=rtrim($line);
    if (($line!='') && (strpos($line,'/')===false)) $line='./'.$line;
}

function create_templates() {
  global $argv;
  $fool_cvs='$Log';
  if (file_exists ("templates")) {
    echo "File templates directory already exists. If you would like to regenerate all file templates - \n";
    echo "delete $PWD/templates directory and re-run $argv[0]\n";
    echo "You may also rerun $argv[0] with --force command line option.\n\n";
  } else {
    echo "Creating $PWD/templates directory\n";
    mkdir ("templates");
  }
  $template_c= <<<TEMPLATE_C
/*!
*! FILE NAME  : \$MODULE\$.c
*! DESCRIPTION: 
*! AUTHOR     : \$AUTHOR\$
*! Copyright (C) \$YEAR\$ Elphel, Inc
*! -----------------------------------------------------------------------------**
*!
*!  This program is free software: you can redistribute it and/or modify
*!  it under the terms of the GNU General Public License as published by
*!  the Free Software Foundation, either version 3 of the License, or
*!  (at your option) any later version.
*!
*!  This program is distributed in the hope that it will be useful,
*!  but WITHOUT ANY WARRANTY; without even the implied warranty of
*!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*!  GNU General Public License for more details.
*!
*!  You should have received a copy of the GNU General Public License
*!  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*! -----------------------------------------------------------------------------**
*!  $fool_cvs:\$MODULE\$.c,v $
*/

TEMPLATE_C;

  $template_cpp= <<<TEMPLATE_CPP
/*!
*! FILE NAME  : \$MODULE\$.cpp
*! DESCRIPTION: 
*! AUTHOR     : \$AUTHOR\$
*! Copyright (C) \$YEAR\$ Elphel, Inc
*! -----------------------------------------------------------------------------**
*!
*!  This program is free software: you can redistribute it and/or modify
*!  it under the terms of the GNU General Public License as published by
*!  the Free Software Foundation, either version 3 of the License, or
*!  (at your option) any later version.
*!
*!  This program is distributed in the hope that it will be useful,
*!  but WITHOUT ANY WARRANTY; without even the implied warranty of
*!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*!  GNU General Public License for more details.
*!
*!  You should have received a copy of the GNU General Public License
*!  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*! -----------------------------------------------------------------------------**
*!  $fool_cvs:\$MODULE\$.cpp,v $
*/

TEMPLATE_CPP;

  $template_h= <<<TEMPLATE_H
/*!
*! FILE NAME  : \$MODULE\$.h
*! DESCRIPTION: 
*! AUTHOR     : \$AUTHOR\$
*! Copyright (C) \$YEAR\$ Elphel, Inc
*! -----------------------------------------------------------------------------**
*!
*!  This program is free software: you can redistribute it and/or modify
*!  it under the terms of the GNU General Public License as published by
*!  the Free Software Foundation, either version 3 of the License, or
*!  (at your option) any later version.
*!
*!  This program is distributed in the hope that it will be useful,
*!  but WITHOUT ANY WARRANTY; without even the implied warranty of
*!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*!  GNU General Public License for more details.
*!
*!  You should have received a copy of the GNU General Public License
*!  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*! -----------------------------------------------------------------------------**
*!  $fool_cvs:\$MODULE\$.h,v $
*/

TEMPLATE_H;

  $template_php= <<<TEMPLATE_PHP
#!/usr/local/sbin/php -q
<?php
/*!
*! PHP script
*! FILE NAME  : \$MODULE\$.php
*! DESCRIPTION: 
*! AUTHOR     : \$AUTHOR\$
*! Copyright (C) \$YEAR\$ Elphel, Inc
*! -----------------------------------------------------------------------------**
*!
*!  This program is free software: you can redistribute it and/or modify
*!  it under the terms of the GNU General Public License as published by
*!  the Free Software Foundation, either version 3 of the License, or
*!  (at your option) any later version.
*!
*!  This program is distributed in the hope that it will be useful,
*!  but WITHOUT ANY WARRANTY; without even the implied warranty of
*!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*!  GNU General Public License for more details.
*!
*!  You should have received a copy of the GNU General Public License
*!  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*! -----------------------------------------------------------------------------**
*!  $fool_cvs:\$MODULE\$.php,v $
*/
?>

TEMPLATE_PHP;

  $template_v= <<<TEMPLATE_V
/*!
*! Verilog HDL
*! FILE NAME  : \$MODULE\$.v
*! DESCRIPTION: 
*! AUTHOR     : \$AUTHOR\$
*! Copyright (C) \$YEAR\$ Elphel, Inc
*! -----------------------------------------------------------------------------**
*!
*!  This program is free software: you can redistribute it and/or modify
*!  it under the terms of the GNU General Public License as published by
*!  the Free Software Foundation, either version 3 of the License, or
*!  (at your option) any later version.
*!
*!  This program is distributed in the hope that it will be useful,
*!  but WITHOUT ANY WARRANTY; without even the implied warranty of
*!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*!  GNU General Public License for more details.
*!
*!  You should have received a copy of the GNU General Public License
*!  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*! -----------------------------------------------------------------------------**
*!  $fool_cvs:\$MODULE\$.v,v $
*/

TEMPLATE_V;

  $template_tf= <<<TEMPLATE_TF
/*!
*! Verilog HDL test fixture
*! FILE NAME  : \$MODULE\$.tf
*! DESCRIPTION: 
*! AUTHOR     : \$AUTHOR\$
*! Copyright (C) \$YEAR\$ Elphel, Inc
*! -----------------------------------------------------------------------------**
*!
*!  This program is free software: you can redistribute it and/or modify
*!  it under the terms of the GNU General Public License as published by
*!  the Free Software Foundation, either version 3 of the License, or
*!  (at your option) any later version.
*!
*!  This program is distributed in the hope that it will be useful,
*!  but WITHOUT ANY WARRANTY; without even the implied warranty of
*!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*!  GNU General Public License for more details.
*!
*!  You should have received a copy of the GNU General Public License
*!  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*! -----------------------------------------------------------------------------**
*!  $fool_cvs:\$MODULE\$.tf,v $
*/

TEMPLATE_TF;

  $template_ucf= <<<TEMPLATE_UCF
##
## HDL user constraints file
## FILE NAME  : \$MODULE\$.ucf
## DESCRIPTION: 
## AUTHOR     : \$AUTHOR\$
## Copyright (C) \$YEAR\$ Elphel, Inc
## -----------------------------------------------------------------------------
##
##  This program is free software: you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation, either version 3 of the License, or
##  (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program.  If not, see <http://www.gnu.org/licenses/>.
## -----------------------------------------------------------------------------
##  $fool_cvs:\$MODULE\$.ucf,v $
##

TEMPLATE_UCF;

  write_template('templates/c',  $template_c);
  write_template('templates/cpp',$template_cpp);
  write_template('templates/h',  $template_h);
  write_template('templates/php',$template_php);
  write_template('templates/v',  $template_v);
  write_template('templates/tf', $template_tf);
  write_template('templates/ucf',$template_ucf);
}

function write_template($path,$content) {
  global $argv;
  if (array_search("--clean",$argv)===false) {
    if (file_exists ($path) && (array_search("--force",$argv)===false)) {
      echo "File template $path already exists, skipping generation.\n";
    } else {
      echo "Generating file template $path\n";
      file_put_contents  ( $path, $content);
    }
  } else {
      echo "Deleting template file $path\n";
      @unlink( $path);
  }
}
function create_elphel353_kdevelop() {
  global $HWID,$REVISION,$PWD,$argv;

  return <<<KDEVELOP
<?xml version = '1.0'?>
<kdevelop>
  <general>
    <author>Elphel, Inc.</author>
    <email>support@elphel.com</email>
    <version>$REVISION</version>
    <projectmanagement>KDevAutoProject</projectmanagement>
    <primarylanguage>C++</primarylanguage>
    <ignoreparts/>
    <projectname>elphel$HWID</projectname>
    <projectdirectory>.</projectdirectory>
    <absoluteprojectpath>false</absoluteprojectpath>
    <description/>
    <defaultencoding/>
    <versioncontrol>kdevcvsservice</versioncontrol>
  </general>
  <kdevautoproject>
    <general>
      <activetarget>apps/web/imgsrv/dummy</activetarget>
      <useconfiguration>default</useconfiguration>
    </general>
    <configurations>
      <optimized>
        <builddir>optimized</builddir>
        <ccompiler>GccOptions</ccompiler>
        <cxxcompiler>GppOptions</cxxcompiler>
        <f77compiler>G77Options</f77compiler>
        <cxxflags>-O2 -g0</cxxflags>
      </optimized>
      <debug>
        <configargs>--enable-debug=full</configargs>
        <builddir/>
        <ccompiler>kdevgccoptions</ccompiler>
        <cxxcompiler>kdevgppoptions</cxxcompiler>
        <f77compiler>kdevpgf77options</f77compiler>
        <cxxflags>-O0 -g3</cxxflags>
        <envvars/>
        <topsourcedir/>
        <cppflags/>
        <ldflags/>
        <ccompilerbinary>gcc-cris</ccompilerbinary>
        <cxxcompilerbinary/>
        <f77compilerbinary/>
        <cflags/>
        <f77flags/>
      </debug>
      <default>
        <envvars/>
      </default>
    </configurations>
    <make>
      <envvars/>
      <abortonerror>true</abortonerror>
      <runmultiplejobs>false</runmultiplejobs>
      <numberofjobs>1</numberofjobs>
      <dontact>false</dontact>
      <makebin>DDD=`command pwd` ; while [ ! -f init_env ] ; do cd .. ; done ; . ./init_env ; if [ `echo \$DDD | grep "/os/"` ] ; then  cd packages/os/linux-2.6-R1_4_2 ; make install ; else cd \$DDD ; if [ -f init_env ] ; then make ; else make install ; fi ; fi ; echo ignoring</makebin>
      <prio>0</prio>
    </make>
    <run>
      <mainprogram></mainprogram>
      <programargs/>
      <globaldebugarguments/>
      <globalcwd></globalcwd>
      <useglobalprogram>false</useglobalprogram>
      <terminal>false</terminal>
      <autocompile>false</autocompile>
      <autoinstall>false</autoinstall>
      <autokdesu>false</autokdesu>
      <envvars/>
    </run>
  </kdevautoproject>
  <kdevdebugger>
    <general>
      <dbgshell>libtool</dbgshell>
      <gdbpath/>
      <configGdbScript/>
      <runShellScript/>
      <runGdbScript/>
      <breakonloadinglibs>true</breakonloadinglibs>
      <separatetty>false</separatetty>
      <floatingtoolbar>false</floatingtoolbar>
    </general>
    <display>
      <staticmembers>false</staticmembers>
      <demanglenames>true</demanglenames>
      <outputradix>10</outputradix>
    </display>
  </kdevdebugger>
  <kdevdoctreeview>
    <ignoretocs>
      <toc>ada</toc>
      <toc>ada_bugs_gcc</toc>
      <toc>bash</toc>
      <toc>bash_bugs</toc>
      <toc>clanlib</toc>
      <toc>fortran_bugs_gcc</toc>
      <toc>gnome1</toc>
      <toc>gnustep</toc>
      <toc>gtk</toc>
      <toc>gtk_bugs</toc>
      <toc>haskell</toc>
      <toc>haskell_bugs_ghc</toc>
      <toc>java_bugs_gcc</toc>
      <toc>java_bugs_sun</toc>
      <toc>kde2book</toc>
      <toc>opengl</toc>
      <toc>pascal_bugs_fp</toc>
      <toc>php</toc>
      <toc>php_bugs</toc>
      <toc>perl</toc>
      <toc>perl_bugs</toc>
      <toc>python</toc>
      <toc>python_bugs</toc>
      <toc>qt-kdev3</toc>
      <toc>ruby</toc>
      <toc>ruby_bugs</toc>
      <toc>sdl</toc>
      <toc>sw</toc>
      <toc>w3c-dom-level2-html</toc>
      <toc>w3c-svg</toc>
      <toc>w3c-uaag10</toc>
      <toc>wxwidgets_bugs</toc>
    </ignoretocs>
    <ignoreqt_xml>
      <toc>Guide to the Qt Translation Tools</toc>
      <toc>Qt Assistant Manual</toc>
      <toc>Qt Designer Manual</toc>
      <toc>Qt Reference Documentation</toc>
      <toc>qmake User Guide</toc>
    </ignoreqt_xml>
    <ignoredoxygen>
      <toc>KDE Libraries (Doxygen)</toc>
    </ignoredoxygen>
  </kdevdoctreeview>
  <kdevfilecreate>
    <filetypes>
      <type icon="php" ext="php" create="template" name="PHP" >
        <descr>A new empty PHP source file.</descr>
      </type>
      <type icon="source_c" ext="c" create="template" name="C Source" >
        <descr>A new empty C file.</descr>
      </type>
      <type icon="source_h" ext="h" create="template" name="C/C++ Header" >
        <descr>A new empty header file for C or C++.</descr>
      </type>
      <type icon="source_cpp" ext="cpp" create="template" name="C++ Source" >
        <descr>A new empty C++ file.</descr>
      </type>
      <type icon="" ext="v" create="template" name="Verilog HDL" >
        <descr>A new emptyVerilog HDL file.</descr>
      </type>
      <type icon="" ext="tf" create="template" name="Verilog test fixture" >
        <descr>A new empty Verilog HDL test fixture file.</descr>
      </type>
      <type icon="" ext="ucf" create="template" name="HDL User constraints file" >
        <descr>New HDL User constraints file</descr>
      </type>
    </filetypes>
    <useglobaltypes/>
  </kdevfilecreate>
  <kdevcppsupport>
    <qt>
      <used>false</used>
      <version>3</version>
      <includestyle>3</includestyle>
      <root>/usr/share/qt3</root>
      <designerintegration>EmbeddedKDevDesigner</designerintegration>
      <qmake>/usr/bin/qmake-qt3</qmake>
      <designer></designer>
      <designerpluginpaths/>
    </qt>
    <codecompletion>
      <automaticCodeCompletion>false</automaticCodeCompletion>
      <automaticArgumentsHint>true</automaticArgumentsHint>
      <automaticHeaderCompletion>true</automaticHeaderCompletion>
      <codeCompletionDelay>250</codeCompletionDelay>
      <argumentsHintDelay>400</argumentsHintDelay>
      <headerCompletionDelay>250</headerCompletionDelay>
      <showOnlyAccessibleItems>false</showOnlyAccessibleItems>
      <completionBoxItemOrder>0</completionBoxItemOrder>
      <howEvaluationContextMenu>true</howEvaluationContextMenu>
      <showCommentWithArgumentHint>true</showCommentWithArgumentHint>
      <statusBarTypeEvaluation>false</statusBarTypeEvaluation>
      <namespaceAliases>std=_GLIBCXX_STD;__gnu_cxx=std</namespaceAliases>
      <processPrimaryTypes>true</processPrimaryTypes>
      <processFunctionArguments>false</processFunctionArguments>
      <preProcessAllHeaders>false</preProcessAllHeaders>
      <parseMissingHeaders>false</parseMissingHeaders>
      <resolveIncludePaths>true</resolveIncludePaths>
      <alwaysParseInBackground>true</alwaysParseInBackground>
      <usePermanentCaching>true</usePermanentCaching>
      <alwaysIncludeNamespaces>false</alwaysIncludeNamespaces>
      <includePaths>.;</includePaths>
    </codecompletion>
    <creategettersetter>
      <prefixGet/>
      <prefixSet>set</prefixSet>
      <prefixVariable>m_,_</prefixVariable>
      <parameterName>theValue</parameterName>
      <inlineGet>true</inlineGet>
      <inlineSet>true</inlineSet>
    </creategettersetter>
    <splitheadersource>
      <enabled>true</enabled>
      <synchronize>true</synchronize>
      <orientation>Vertical</orientation>
    </splitheadersource>
    <references/>
  </kdevcppsupport>
  <cppsupportpart>
    <filetemplates>
      <interfacesuffix>.h</interfacesuffix>
      <implementationsuffix>.cpp</implementationsuffix>
    </filetemplates>
  </cppsupportpart>
  <kdevdocumentation>
    <projectdoc>
      <docsystem>Doxygen Documentation Collection</docsystem>
      <docurl>docs/html/index.html</docurl>
      <usermanualurl/>
    </projectdoc>
  </kdevdocumentation>
  <kdevfileview>
    <groups>
      <hidenonprojectfiles>false</hidenonprojectfiles>
      <hidenonlocation>false</hidenonlocation>
    </groups>
    <tree>
      <hidepatterns>*.o,*.lo,CVS,*~</hidepatterns>
      <hidenonprojectfiles>true</hidenonprojectfiles>
      <showvcsfields>false</showvcsfields>
    </tree>
  </kdevfileview>
  <ctagspart>
    <customArguments>-R --c++-types=+px --excmd=pattern --links=no --exclude=Makefile --exclude=. --exclude=@$PWD/ctags_excludes</customArguments>
    <customTagfilePath>$PWD/tags</customTagfilePath>
    <activeTagsFiles/>
  </ctagspart>
  <kdevcvsservice>
    <recursivewhenupdate>true</recursivewhenupdate>
    <prunedirswhenupdate>true</prunedirswhenupdate>
    <createdirswhenupdate>true</createdirswhenupdate>
    <recursivewhencommitremove>true</recursivewhencommitremove>
    <revertoptions>-C</revertoptions>
  </kdevcvsservice>
</kdevelop>

KDEVELOP;
}


function create_elphel353_ctags_exclusions() {
  global $PWD;
  return <<<EXCLUSIONS
$PWD/target/*
$PWD/os/*/arch/alpha/*
$PWD/os/*/arch/arm/*
$PWD/os/*/arch/arm26/*
$PWD/os/*/arch/avr32/*
$PWD/os/*/arch/cris/arch-v10/*
$PWD/os/*/arch/frv/*
$PWD/os/*/arch/h8300/*
$PWD/os/*/arch/i386/*
$PWD/os/*/arch/ia64/*
$PWD/os/*/arch/list/*
$PWD/os/*/arch/m32r/*
$PWD/os/*/arch/m68k/*
$PWD/os/*/arch/m68knommu/*
$PWD/os/*/arch/mips/*
$PWD/os/*/arch/parisc/*
$PWD/os/*/arch/powerpc/*
$PWD/os/*/arch/ppc/*
$PWD/os/*/arch/s390/*
$PWD/os/*/arch/sh/*
$PWD/os/*/arch/sh64/*
$PWD/os/*/arch/sparc/*
$PWD/os/*/arch/sparc64/*
$PWD/os/*/arch/um/*
$PWD/os/*/arch/v850/*
$PWD/os/*/arch/x86_64/*
$PWD/os/*/arch/xtensa/*
$PWD/os/*/include/asm-alpha/*
$PWD/os/*/include/asm-arm/*
$PWD/os/*/include/asm-arm26/*
$PWD/os/*/include/asm-avr32/*
$PWD/os/*/include/asm-frv/*
$PWD/os/*/include/asm-generic/*
$PWD/os/*/include/asm-h8300/*
$PWD/os/*/include/asm-i386/*
$PWD/os/*/include/asm-ia64/*
$PWD/os/*/include/asm-m32r/*
$PWD/os/*/include/asm-m68k/*
$PWD/os/*/include/asm-m68knommu/*
$PWD/os/*/include/asm-mips/*
$PWD/os/*/include/asm-parisc/*
$PWD/os/*/include/asm-powerpc/*
$PWD/os/*/include/asm-ppc/*
$PWD/os/*/include/asm-s390/*
$PWD/os/*/include/asm-sh/*
$PWD/os/*/include/asm-sh64/*
$PWD/os/*/include/asm-sparc/*
$PWD/os/*/include/asm-sparc64/*
$PWD/os/*/include/asm-um/*
$PWD/os/*/include/asm-v850/*
$PWD/os/*/include/asm-x86_64/*
$PWD/os/*/include/asm-xtensa/*
$PWD/os/linux-2.6/*
$PWD/os/*/arch/cris/arch/*
$PWD/os/*/arch/cris/drivers/*
$PWD/os/*/usr/include/*

EXCLUSIONS;
}


function test_kdevelop_depend($executable,$package) {
   if (($path=trim(shell_exec ("which $executable")))=="") {
     echo "You need $executable to be installed in your system.\n";
     echo "In Ubuntu you can install it with a command:\n";
     echo "sudo apt-get install $package\n\n";
     return false;
   } else {
     echo "$path found.\n";
     return true;
   }

}

function walk_src_doxyfiles(&$line) { /// removes unrecognized by Doxygen file types
   global $DOXYGEN_EXTENSIONS;
   $pos=strpos($line,'#');
   if ($pos!==false) $line=substr($line,0,$pos);
   $line=ltrim($line);
   if (strpos($line,'===')!==false) $line="";
//   if (strpos($line,'->')!==false)  $line=""; // remove links - leave for now
   $pos=strpos($line,' ');
   if ($pos!==false) $line=substr($line,0,$pos);
   $line=rtrim($line);
/// Filter with recognized extensions
   if (($pos=strrpos($line,'.'))) {
     $ext=substr($line,$pos+1);
     $good=false;
     foreach ($DOXYGEN_EXTENSIONS as $extension) if ($ext==$extension) {
       $good=true;
       break;
     }
     if (!$good) $line="";
   } else $line="";
   if (($line!='') && (strpos($line,'/')===false)) $line='./'.$line;
}


function create_doxyfile() {
   global $PWD, $DOXYGEN_EXTENSIONS;
   $file_patterns=   "";
   for ($i=0; $i< (count($DOXYGEN_EXTENSIONS) -1);$i++) {
     $file_patterns.="                         *.".$DOXYGEN_EXTENSIONS[$i]." \\\n";
   }
   $file_patterns.=  "                         *.".$DOXYGEN_EXTENSIONS[count($DOXYGEN_EXTENSIONS) -1]."\n";

   $list=file('src.list');
   array_walk($list,'walk_src_doxyfiles');
   $list=array_filter  ($list); // remove empty lines
   sort ($list);
   $files_list="";
   for ($i=0; $i< (count($list) -1);$i++) {
     $files_list.="                         ".$list[$i]." \\\n";
   }
   $files_list.=  "                         ".$list[count($list) -1]."\n";

   return <<<EOT
# Doxyfile 1.5.1-KDevelop

#---------------------------------------------------------------------------
# Project related configuration options
#---------------------------------------------------------------------------
PROJECT_NAME           = elphel$HWID
PROJECT_NUMBER         = $REVISION
OUTPUT_DIRECTORY       = docs/
CREATE_SUBDIRS         = NO
OUTPUT_LANGUAGE        = English
USE_WINDOWS_ENCODING   = NO
BRIEF_MEMBER_DESC      = YES
REPEAT_BRIEF           = YES
ABBREVIATE_BRIEF       = "The \$name class" \
                         "The \$name widget" \
                         "The \$name file" \
                         is \
                         provides \
                         specifies \
                         contains \
                         represents \
                         a \
                         an \
                         the
ALWAYS_DETAILED_SEC    = NO
INLINE_INHERITED_MEMB  = NO
FULL_PATH_NAMES        = YES
STRIP_FROM_PATH        = $PWD
STRIP_FROM_INC_PATH    = 
SHORT_NAMES            = NO
JAVADOC_AUTOBRIEF      = NO
MULTILINE_CPP_IS_BRIEF = NO
DETAILS_AT_TOP         = NO
INHERIT_DOCS           = YES
SEPARATE_MEMBER_PAGES  = NO
TAB_SIZE               = 8
ALIASES                = 
OPTIMIZE_OUTPUT_FOR_C  = YES
OPTIMIZE_OUTPUT_JAVA   = NO
BUILTIN_STL_SUPPORT    = NO
DISTRIBUTE_GROUP_DOC   = NO
SUBGROUPING            = YES
#---------------------------------------------------------------------------
# Build related configuration options
#---------------------------------------------------------------------------
EXTRACT_ALL            = YES
EXTRACT_PRIVATE        = YES
EXTRACT_STATIC         = YES
EXTRACT_LOCAL_CLASSES  = YES
EXTRACT_LOCAL_METHODS  = NO
HIDE_UNDOC_MEMBERS     = NO
HIDE_UNDOC_CLASSES     = NO
HIDE_FRIEND_COMPOUNDS  = NO
HIDE_IN_BODY_DOCS      = NO
INTERNAL_DOCS          = NO
CASE_SENSE_NAMES       = YES
HIDE_SCOPE_NAMES       = NO
SHOW_INCLUDE_FILES     = YES
INLINE_INFO            = YES
SORT_MEMBER_DOCS       = YES
SORT_BRIEF_DOCS        = NO
SORT_BY_SCOPE_NAME     = NO
GENERATE_TODOLIST      = YES
GENERATE_TESTLIST      = YES
GENERATE_BUGLIST       = YES
GENERATE_DEPRECATEDLIST= YES
ENABLED_SECTIONS       = 
MAX_INITIALIZER_LINES  = 30
SHOW_USED_FILES        = YES
SHOW_DIRECTORIES       = NO
FILE_VERSION_FILTER    = 
#---------------------------------------------------------------------------
# configuration options related to warning and progress messages
#---------------------------------------------------------------------------
QUIET                  = NO
WARNINGS               = YES
WARN_IF_UNDOCUMENTED   = YES
WARN_IF_DOC_ERROR      = YES
WARN_NO_PARAMDOC       = NO
WARN_FORMAT            = "\$file:\$line: \$text"
WARN_LOGFILE           = 
#---------------------------------------------------------------------------
# configuration options related to the input files
#---------------------------------------------------------------------------
INPUT                  = \
$files_list
FILE_PATTERNS          = \
$file_patterns
RECURSIVE              = YES
EXCLUDE                = 
EXCLUDE_SYMLINKS       = NO
EXCLUDE_PATTERNS       = 
EXAMPLE_PATH           = 
EXAMPLE_PATTERNS       = *
EXAMPLE_RECURSIVE      = NO
IMAGE_PATH             = 
INPUT_FILTER           = 
FILTER_PATTERNS        = 
FILTER_SOURCE_FILES    = NO
#---------------------------------------------------------------------------
# configuration options related to source browsing
#---------------------------------------------------------------------------
SOURCE_BROWSER         = YES
INLINE_SOURCES         = NO
STRIP_CODE_COMMENTS    = YES
REFERENCED_BY_RELATION = YES
REFERENCES_RELATION    = YES
REFERENCES_LINK_SOURCE = YES
USE_HTAGS              = NO
VERBATIM_HEADERS       = YES
#---------------------------------------------------------------------------
# configuration options related to the alphabetical class index
#---------------------------------------------------------------------------
ALPHABETICAL_INDEX     = YES
COLS_IN_ALPHA_INDEX    = 5
IGNORE_PREFIX          = 
#---------------------------------------------------------------------------
# configuration options related to the HTML output
#---------------------------------------------------------------------------
GENERATE_HTML          = YES
HTML_OUTPUT            = html
HTML_FILE_EXTENSION    = .html
HTML_HEADER            = 
HTML_FOOTER            = 
HTML_STYLESHEET        = 
HTML_ALIGN_MEMBERS     = YES
GENERATE_HTMLHELP      = NO
CHM_FILE               = 
HHC_LOCATION           = 
GENERATE_CHI           = NO
BINARY_TOC             = NO
TOC_EXPAND             = NO
DISABLE_INDEX          = NO
ENUM_VALUES_PER_LINE   = 4
GENERATE_TREEVIEW      = YES
TREEVIEW_WIDTH         = 250
#---------------------------------------------------------------------------
# configuration options related to the LaTeX output
#---------------------------------------------------------------------------
GENERATE_LATEX         = YES
LATEX_OUTPUT           = latex
LATEX_CMD_NAME         = latex
MAKEINDEX_CMD_NAME     = makeindex
COMPACT_LATEX          = NO
PAPER_TYPE             = a4wide
EXTRA_PACKAGES         = 
LATEX_HEADER           = 
PDF_HYPERLINKS         = YES
USE_PDFLATEX           = YES
LATEX_BATCHMODE        = NO
LATEX_HIDE_INDICES     = NO
#---------------------------------------------------------------------------
# configuration options related to the RTF output
#---------------------------------------------------------------------------
GENERATE_RTF           = YES
RTF_OUTPUT             = rtf
COMPACT_RTF            = NO
RTF_HYPERLINKS         = NO
RTF_STYLESHEET_FILE    = 
RTF_EXTENSIONS_FILE    = 
#---------------------------------------------------------------------------
# configuration options related to the man page output
#---------------------------------------------------------------------------
GENERATE_MAN           = YES
MAN_OUTPUT             = man
MAN_EXTENSION          = .3
MAN_LINKS              = NO
#---------------------------------------------------------------------------
# configuration options related to the XML output
#---------------------------------------------------------------------------
GENERATE_XML           = YES
XML_OUTPUT             = xml
XML_SCHEMA             = 
XML_DTD                = 
XML_PROGRAMLISTING     = YES
#---------------------------------------------------------------------------
# configuration options for the AutoGen Definitions output
#---------------------------------------------------------------------------
GENERATE_AUTOGEN_DEF   = NO
#---------------------------------------------------------------------------
# configuration options related to the Perl module output
#---------------------------------------------------------------------------
GENERATE_PERLMOD       = NO
PERLMOD_LATEX          = NO
PERLMOD_PRETTY         = YES
PERLMOD_MAKEVAR_PREFIX = 
#---------------------------------------------------------------------------
# Configuration options related to the preprocessor   
#---------------------------------------------------------------------------
ENABLE_PREPROCESSING   = YES
MACRO_EXPANSION        = NO
EXPAND_ONLY_PREDEF     = NO
SEARCH_INCLUDES        = YES
INCLUDE_PATH           = 
INCLUDE_FILE_PATTERNS  = 
PREDEFINED             = 
EXPAND_AS_DEFINED      = 
SKIP_FUNCTION_MACROS   = YES
#---------------------------------------------------------------------------
# Configuration::additions related to external references   
#---------------------------------------------------------------------------
TAGFILES               = 
GENERATE_TAGFILE       = 
ALLEXTERNALS           = NO
EXTERNAL_GROUPS        = YES
PERL_PATH              = /usr/bin/perl
#---------------------------------------------------------------------------
# Configuration options related to the dot tool   
#---------------------------------------------------------------------------
CLASS_DIAGRAMS         = NO
HIDE_UNDOC_RELATIONS   = YES
HAVE_DOT               = YES
CLASS_GRAPH            = YES
COLLABORATION_GRAPH    = YES
GROUP_GRAPHS           = YES
UML_LOOK               = NO
TEMPLATE_RELATIONS     = NO
INCLUDE_GRAPH          = YES
INCLUDED_BY_GRAPH      = YES
CALL_GRAPH             = NO
CALLER_GRAPH           = NO
GRAPHICAL_HIERARCHY    = YES
DIRECTORY_GRAPH        = YES
DOT_IMAGE_FORMAT       = png
DOT_PATH               = 
DOTFILE_DIRS           = 
MAX_DOT_GRAPH_WIDTH    = 1024
MAX_DOT_GRAPH_HEIGHT   = 1024
MAX_DOT_GRAPH_DEPTH    = 1000
DOT_TRANSPARENT        = YES
DOT_MULTI_TARGETS      = NO
GENERATE_LEGEND        = YES
DOT_CLEANUP            = YES
#---------------------------------------------------------------------------
# Configuration::additions related to the search engine   
#---------------------------------------------------------------------------
SEARCHENGINE           = YES

EOT;
}



?>

